Package: MorphSim
Type: Package
Title: Simulate Discrete Character Data along Phylogenetic Trees
Version: 1.2.0
Authors@R: c(person(given = "Laura", family = "Mulvey", role = c("aut", "cre", "cph"), email = "lauramulvey479@gmail.com"),
    person(given = "Tim", family = "Brandler",role=c("aut","cph")),
    person(given = "Alessio", family = "Capobianco",role=c("aut","cph")),
    person(given = "Rachel", family = "Warnock", role=c("aut","cph")),
    person(given =  "Joelle", family = "Barido-Sottani", role=c("aut","cph")))
Description: Tools to simulate morphological traits along phylogenetic trees
 with branch lengths representing evolutionary distance or time.
 Includes functions for visualizing evolutionary processes along trees
 and within morphological character matrices.
License: GPL-3
Encoding: UTF-8
RoxygenNote: 7.3.3
Imports: ape, FossilSim, stats, graphics, phangorn
Suggests: TreeSim, testthat (>= 3.0.0), knitr, rmarkdown
VignetteBuilder: knitr
Config/testthat/edition: 3
Depends: R (>= 3.5)
LazyData: true
NeedsCompilation: no
Packaged: 2026-05-06 13:45:15 UTC; lauramulvey
Author: Laura Mulvey [aut, cre, cph],
  Tim Brandler [aut, cph],
  Alessio Capobianco [aut, cph],
  Rachel Warnock [aut, cph],
  Joelle Barido-Sottani [aut, cph]
Maintainer: Laura Mulvey <lauramulvey479@gmail.com>
Repository: CRAN
Date/Publication: 2026-05-06 14:30:02 UTC
Built: R 4.5.2; ; 2026-05-06 16:27:25 UTC; unix

# Generated by roxygen2: do not edit by hand

S3method(as.morpho,default)
S3method(plot,morpho)
S3method(print,morpho)
S3method(summary,morpho)
export(as.morpho)
export(combine.morpho)
export(convergent_evol)
export(get.convergent)
export(get.matrix)
export(get.reconstructed)
export(get.transitions)
export(get_gamma_rates)
export(get_lognormal_rates)
export(is.morpho)
export(morpho)
export(plotMorphoGrid)
export(sim.missing.data)
export(sim.morpho)
export(stats.morpho)
export(symmetric.Q.matrix)
export(write.morpho)
export(write.recon.matrix)
export(write.recon.tree)
export(write.recon.tsv)
export(write.tsv)
import(graphics)
import(stats)
importFrom(ape,Ntip)
importFrom(ape,node.depth.edgelength)
importFrom(ape,nodepath)
importFrom(ape,write.nexus.data)
importFrom(ape,write.tree)
importFrom(phangorn,CI)
importFrom(phangorn,RF.dist)
importFrom(phangorn,RI)
importFrom(phangorn,phyDat)

## MorphSim 

R package for simulating discrete character data along rooted phylogenies. Through integration with established R packages, MorphSim enables the simulation of biologically meaningful datasets across a variety of scenarios. 


### Installation

MorphSim is available on CRAN:
```r
install.packages("MorphSim")
```

The development version can be installed from GitHub:
```r
library(devtools)
devtools::install_github("https://github.com/fossilsim/morphsim")
```

### Dependencies

MorphSim depends on R (≥ 3.5) and imports ape, FossilSim, and phangorn. TreeSim is suggested for tree simulation.

### License

GPL-3  

## Quick Start Tutorials 
For a more in-depth explanation of the functionality of MorphSim see the introductory vignette.

## Example 1: Simulating Data Along a Phylogenetic Tree - Basic set up
To simulate morphological data you will first need a phylogenetic tree. MorphSim can take either, a time tree or a tree with branch lengths in genetic distance as input. In this example we will use a time tree. You can simulate a tree using the existing R package `TreeSim`

```r
install.packages("TreeSim")
library(TreeSim)
```

#### Simulate a birth death tree using TreeSim
```r
lambda = 0.1         # speciation rate
mu = 0.05            # extinction rate
tips = 5             # number of extant tips

tree = TreeSim::sim.bd.taxa(n = tips, 
                         numbsim = 1, 
                         lambda = lambda, 
                         mu = mu)[[1]]                       
```
#### Simulate Morphological data using MorphSim. 
MorphSim lets you specify a variety of parameters for your simulations. In this example, we will focus on a few key parameters to get started.
- **k**: the maximum number of character states. This can be a single integer greater than 1 or a vector of integers if you want to simulate more than 1 partitions. In the following example, we are therefore simulating two partitions, the first with 2 states and the second with 3.
- **timetree**: if using a time tree specify here.
- **tree**: if using a genetic distance tree specify here.
- **partitions**: specify the number of traits per partition. This must match the number of states specified in k.
- **trait.num**: specify the total number of traits 
- **br.rates**: This can be a single integer (strict clock) or a vector of integers (relaxed clock). This must be provided when using a time tree to convert the branch lengths into genetic distance.
- **ancestral**: return the states at the internal nodes in the tree.

> Setting up a simulation as follows will simulate characters under an Mk (equal rates) model with a strict clock.


```r
morpho_data <-  sim.morpho(k = c(2,3), 
                           time.tree = tree,
                           tree = NULL,
                           partition = c(10,10),
                           trait.num = 20,
                           br.rates = 0.1,
                           ancestral = TRUE)  

```
#### Explore the simulated data
Morphsim has a plotting function which allows you to plot the simulated data along the branches of the tree. 
```r
plot(morpho_data, 
     trait= 1) # specify which trait you want to plot along the tree
```
This function returns a number of characteristics about your data. It calculates the consistency index, the retention index, identifies convergent traits, and the number of extinct and extant taxa.
```r
stats <- stats.morpho(morpho_data)
```




## Example 2: Simulating Biologically Realistic Data Sets
The previous example, simulated characters according to an Mk model under a strict clock. However, there are more complex models we may want to use for simulations. Firstly, with MorphSim we can use a relaxed clock model where each branch has a different rate. We can use an existing R package, `simclock` to simulate rates along our tree which we can input into MorphSim 


####  Simulate branch lengths using a independent log normal relaxed clock model

```r
devtools::install_github("dosreislab/simclock")
reltt <- simclock::relaxed.tree(tree, 
                                model="iln", 
                                r=.04e-1,     # mean mutation rate
                                s2=.26e-2)   # diffusion rate
```
For the character data we can relax the assumptions of the Mk model in a number of ways as described below. The br.rates argument accepts any numeric vector of rates, one per branch, in the same order as tree$edge. Here we pass reltt$edge.length from simclock, but any custom vector of branch rates can be used directly, for example, br.rates = rep(0.1, nrow(tree$edge)) for a strict clock, or a user-defined vector reflecting prior knowledge about lineage-specific rates.

- **ACRV**: Here you can specify if you would like to model among character rate variation. This is commonly used for morphological data sets as traits may evolve at different rates. There are three input options here, `gamma`, `lgn`, and `user`. Gamma uses a discrete gamma distribution, lgn will use a discrete lognormal, and user allows the user to specify rates.

- **ACRV.ncats**: the number of rate categories you want to simulate under

- **alpha.gamma**: the shape of the alpha distribution

- **variable**: When set to true this will only return traits which vary across taxa, i.e., the MkV model

```r
morpho_data <-  sim.morpho(k = c(2,3), 
                           time.tree = tree,
                           tree = NULL,
                           partition = c(10,10),
                           trait.num = 20,
                           br.rates = reltt$edge.length,
                           ACRV = "gamma",
                           alpha.gamma = 1,
                           ACRV.ncats = 4,
                           variable = TRUE)

```

> Note: If you choose to simulate data under an ACRV using a lognormal distribution, you will need to provide values for the `meanlog` and the `sdlog`. Similarly for the user option the rates should be passed through `define.ACRV.rates`

### Ordered characters
In morphological datasets, transitions between character states are often restricted, meaning that a change can only occur between specific states. For example, a character might be able to transition from 0 → 1, but not directly from 0 → 2; the species must pass through the intermediate state. These are known as correlated characters. In MorphSim, we can simulate such characters by defining a Q matrix that enforces these allowed transitions.

Define a Q matrix the allows 0 &harr; 1 &harr; 2

```r
ord_Q <- matrix(c(
  -0.5, 0.5, 0.0,
  0.3333333, -0.6666667, 0.3333333,
  0.0, 0.5, -0.5
), nrow = 3, byrow = TRUE)
```
This Q matrix can then be used to simulate characters

```r
morpho_data <-  sim.morpho(k = 3, 
                           define.Q = ord_Q,
                           time.tree = tree,
                           tree = NULL,
                           trait.num = 10,
                           br.rates = reltt$edge.length,
                           ACRV = "gamma",
                           alpha.gamma = 1,
                           ACRV.ncats = 4,
                           variable = TRUE)
```

## Example 3: Simulating Data sets with Sampled Ancestors
MorphSim integrates with `FossilSim`, allowing fossils simulated in FossilSim to be incorporated when simulating character traits. Because MorphSim tracks where along branches transitions occur, it can generate traits that are specific to the time and lineage in which a fossil is sampled.

```r
install_github("fossilsim/fossilsim")
```
#### Simulate fossil and extant species sampling

```r
rate = 0.1 # fossil sampling rate
f = FossilSim::sim.fossils.poisson(rate = rate, 
                                   tree = tree, 
                                   root.edge = F)

rho = 0.5 # extant species sampling rate
f2 = FossilSim::sim.extant.samples(fossils = f, 
                                   tree = tree, 
                                   rho = rho)
```

This fossils object can be passed into the `sim.morpho` function

```r
morpho_data <-  sim.morpho(k = c(2,3), 
                           time.tree = tree,
                           tree = NULL,
                           partition = c(10,10),
                           trait.num = 20,
                           br.rates = reltt$edge.length,
                           ACRV = "gamma",
                           alpha.gamma = 1,
                           ACRV.ncats = 4,
                           variable = TRUE,
                           fossil = f2)
```
We now have character data for all the tips in the true tree, as well as all the sampled ancestors. The inclusion of fossils also adds a number of options to the plotting function. We can choose if we want to plot the fossils along the tree, and to show the reconstructed version. 

```r
plot(x = morpho_data, 
     trait= 1, 
     timetree = T, 
     show.fossil = T,
     root.edge = T, 
     reconstructed = T)
```

## Example 4: Simulating Parsimony Informative characters
MorphSim allows you to filter simulated characters based on their parsimony informativeness. By changing the default `parsimony = NULL` to either `"standard"` or `"strict"`, you can control which traits are simulated in your dataset.

- **standard**: Retains characters with at least two effective states, including autapomorphies (traits unique to a single taxon)

- **strict**: Only retains parsimony-informative characters (traits shared by at least two taxa)

```r
morpho_data <-  sim.morpho(k = c(2,3), 
                           time.tree = tree,
                           tree = NULL,
                           partition = c(10,10),
                           trait.num = 20,
                           br.rates = 0.1,
                           parsimony = "standard",   # alternatively use "strict" to exclude autapomorphies
                           ancestral = TRUE,
                           fossil = f2)
``` 

## MorphSim 

R package for simulating discrete character data along rooted phylogenies. Through integration with established R packages, MorphSim enables the simulation of biologically meaningful datasets across a variety of scenarios. 

The latest version can be installed from github using `devtools` 

``` r   
library(devtools)
devtools::install_github("https://github.com/fossilsim/morphsim")
library(MorphSim)
```
    


## Example 1: Simulating Data Along a Phylogenetic Tree - Basic set up
To simulate morphological data you will first need a phylogenetic tree. MorphSim can take either, a time tree or a tree with branch lengths in genetic distance as input. In this example we will use a time tree. You can simulate a tree using the existing R package `TreeSim`

```r
install.packages("TreeSim")
library(TreeSim)
```

#### Simulate a birth death tree using TreeSim
```r
lambda = 0.1         # speciation rate
mu = 0.05            # extinction rate
tips = 5             # number of extant tips

tree = TreeSim::sim.bd.taxa(n = tips, 
                         numbsim = 1, 
                         lambda = lambda, 
                         mu = mu)[[1]]                       
```
#### Simulate Morphological data using MorphSim. 
MorphSim lets you specify a variety of parameters for your simulations. In this example, we will focus on a few key parameters to get started.
- **k**: the maximum number of character states. This can be a single integer greater than 1 or a vector of integers if you want to simulate more than 1 partitions. In the following example, we are therefore simulating two partitions, the first with 2 states and the second with 3.
- **timetree**: if using a time tree specify here.
- **tree**: if using a genetic distance tree specify here.
- **partitions**: specify the number of traits per partition. This must match the number of states specified in k.
- **trait.num**: specify the total number of traits 
- **br.rates**: This can be a single integer (strict clock) or a vector of integers (relaxed clock). This must be provided when using a time tree to convert the branch lengths into genetic distance.
- **ancestral**: return the states at the internal nodes in the tree.

> Setting up a simulation as follows will simulate characters under an Mk (equal rates) model with a strict clock.


```r
morpho_data <-  sim.morpho(k = c(2,3), 
                           time.tree = tree,
                           tree = NULL,
                           partition = c(10,10),
                           trait.num = 20,
                           br.rates = 0.1,
                           ancestral = TRUE)  

```
#### Explore the simulated data
Morphsim has a plotting function which allows you to plot the simulated data along the branches of the tree. 
```r
plot(morpho_data, 
     trait= 1) # specify which trait you want to plot along the tree
```
This function returns a number of characteristics about your data. It calculates the consistency index, the retention index, identifies convergent traits, and the number of extinct and extant taxa.
```r
stats <- stats.morpho(morpho_data)
```




## Example 2: Simulating Biologically Realistic Data Sets
The previous example, simulated characters according to an Mk model under a strict clock. However, there are more complex models we may want to use for simulations. Firstly, with MorphSim we can use a relaxed clock model where each branch has a different rate. We can use an existing R package, `simclock` to simulate rates along our tree which we can input into MorphSim 


####  Simulate branch lengths using a independent log normal relaxed clock model

```r
devtools::install_github("dosreislab/simclock")
reltt <- simclock::relaxed.tree(tree, 
                                model="iln", 
                                r=.04e-1,     # mean mutation rate
                                s2=.26e-2)   # diffusion rate
```
For the character data we can relax the assumptions of the Mk model in a number of ways as described below

- **ACRV**: Here you can specify if you would like to model among character rate variation. This is commonly used for morphological data sets as traits may evolve at different rates. There are three input options here, `gamma`, `lgn`, and `user`. Gamma uses a discrete gamma distribution, lgn will use a discrete lognormal, and user allows the user to specify rates.

- **ACRV.ncats**: the number of rate categories you want to simulate under

- **alpha.gamma**: the shape of the alpha distribution

-**variable**: When set to true this will only return traits which vary across taxa, i.e., the MkV model

```r
morpho_data <-  sim.morpho(k = c(2,3), 
                           time.tree = tree,
                           tree = NULL,
                           partition = c(10,10),
                           trait.num = 20,
                           br.rates = reltt$edge.length,
                           ACRV = "gamma",
                           alpha.gamma = 1,
                           ACRV.ncats = 4,
                           variable = TRUE)

```

> Note: If you choose to simulate data under an ACRV using a lognormal distribution, you will need to provide values for the `meanlog` and the `sdlog`. Similarly for the user option the rates should be passed through `define.ACRV.rates`

### Ordered characters
In morphological datasets, transitions between character states are often restricted, meaning that a change can only occur between specific states. For example, a character might be able to transition from 0 → 1, but not directly from 1 → 2; the species must pass through the intermediate state. These are known as correlated characters. In MorphSim, we can simulate such characters by defining a Q matrix that enforces these allowed transitions.

Define a Q matrix the allows 0 &harr; 1 &harr; 2

```r
ord_Q <- matrix(c(
  -0.5, 0.5, 0.0,
  0.3333333, -0.6666667, 0.3333333,
  0.0, 0.5, -0.5
), nrow = 3, byrow = TRUE)
```
This Q matrix can then be used to simulate characters

```r
morpho_data <-  sim.morpho(k = 3, 
                           define.Q = ord_Q,
                           time.tree = tree,
                           tree = NULL,
                           trait.num = 10,
                           br.rates = reltt$edge.length,
                           ACRV = "gamma",
                           alpha.gamma = 1,
                           ACRV.ncats = 4,
                           variable = TRUE)
```

## Example 3: Simulating Data sets with Sampled Ancestors
MorphSim integrates with `FossilSim`, allowing fossils simulated in FossilSim to be incorporated when simulating character traits. Because MorphSim tracks where along branches transitions occur, it can generate traits that are specific to the time and lineage in which a fossil is sampled.

```r
install_github("fossilsim/fossilsim")
```
#### Simulate fossil and extant species sampling

```r
rate = 0.1 # fossil sampling rate
f = FossilSim::sim.fossils.poisson(rate = rate, 
                                   tree = tree, 
                                   root.edge = F)

rho = 0.5 # extant species sampling rate
f2 = FossilSim::sim.extant.samples(fossils = f, 
                                   tree = tree, 
                                   rho = rho)
```

This fossils object can be passed into the `sim.morpho` function

```r
morpho_data <-  sim.morpho(k = c(2,3), 
                           time.tree = tree,
                           tree = NULL,
                           partition = c(10,10),
                           trait.num = 20,
                           br.rates = reltt$edge.length,
                           ACRV = "gamma",
                           alpha.gamma = 1,
                           ACRV.ncats = 4
                           variable = TRUE,
                           fossil = f2)
```
We now have character data for all the tips in the true tree, as well as all the sampled ancestors. The inclusion of fossils also adds a number of options to the plotting function. We can choose if we want to plot the fossils along the tree, and to show the reconstructed version. 

```r
plot(x = morpho_data, 
     trait= 1, 
     timetree = T, 
     show.fossil = T,
     root.edge = T, 
     reconstructed = T)
```

## Licence 
Apache 2.0 License

sim.morpho.old = function(tree, k = 2, nchar = c(10), variable.coding = FALSE, partitions = 1, alpha = NULL){
  # Mk
  # + V
  # + P
  # + G
  # wish list
  # missing data
  # FBD range
  # note k = max state number

  #TODO check tree etc, e.g., tree must tree, tree must root, or we arbitrarily root the tree, check is fully bifurcating
  #TODO add check partitions and character make sense
  # if (p > 1 && nchar length == 1, nchar[[1]] / p)

  if(is.null(alpha)) rates = 1
  else rates = phangorn::discrete.gamma(alpha, 4) # TODO: NEED TO CHECK WITH MIKE

  for(p in partitions:1){ # for now, goes backwards from p max. Otherwise we have to deal with attr(seq, "levels"); or you can use cbind(x, seq)
    if(max(partitions) == 1) states = as.character(c(0:(k-1)))
    else states = as.character(c(0:p))

    for(i in 1:nchar[p]){
      if(nchar[p] == 0) next

      x = phangorn::simSeq(tree, l = 1, type = "USER", levels = states, rate = sample(rates, 1))

      if(variable.coding){
        while( length(unique(as.character(x))) == 1 ){
          x = phangorn::simSeq(tree, l = 1, type = "USER", levels = states, rate = sample(rates, 1))
        }
      }

      if(i == 1 && p == max(partitions)){
        seq = x
      }
      else seq = cbind(seq, x)
    }
  }
  seq
}






set.seed(123)
phy <- ape::rtree(10)
plot(phy)
simulated_morpho <- sim.morpho(phy, k = 2, trait.num = 20)
simulated_morpho

make.evol(phy, char=simulated_morpho$sequences[1])

tree <- ape::rtree(6)
p = 2
states = as.character(c(0:p))
levels = states
x = tree

# phangorn
sim.site <- function(x, states, rate = 1, ancestral = FALSE, Q = NULL) {

  #TODO check if levels is vector, change to state

  # x = tree
  # levels = states
  # bf = base frequencies
  # Q
  states = 2
  Q = NULL
  bf = NULL
  l = 2 #alignment length

  lbf <- length(states)
  if (is.null(bf)) bf <- rep(1 / lbf, lbf)
  if (is.null(Q)) {
    Q <- rep(1, lbf * (lbf - 1) / 2) #?
  }
  if (is.matrix(Q)) Q <- Q[lower.tri(Q)]
  eig <- edQt(Q, bf) # decomposition #?

  m <- length(levels) # state number

  # simulate the root state
  rootseq <- sample(levels, l, replace = TRUE, prob = bf)

  # reorder branches in the phylogeny
  x <- reorder(x)
  edge <- x$edge
  nNodes <- max(edge)
  # create an empty matrix
  res <- matrix(NA, nNodes, l)
  parent <- as.integer(edge[, 1])
  child <- as.integer(edge[, 2])
  root <- as.integer(parent[!match(parent, child, 0)][1])
  # add the root state to the matrix
  res[root, ] <- rootseq
  # edge lengths
  tl <- x$edge.length
  # loop through all edges
  for (i in seq_along(tl)){
    from <- parent[i]
    to <- child[i]
    # generate a matrix of probabilities
    P <- getP(tl[i], eig, rate)[[1]]
    # avoid numerical problems for larger P and small t
    if (any(P < 0)) P[P < 0] <- 0
    #this is how FossilSim::sim.trait.values does it for a symmetic Mk matrix
    #P = (( (k - 1) / k ) * (1 - exp(-( k / (k - 1) ) * v * blength) ) ) )
    #r = sample(c(0:(k-1))[-(r+1)], 1)
    for (j in 1:m) {
      # boolean, if current state is TRUE then we calc the prob of changing from that state. Useful loop is sites > 1, otherwise some redundancy
      ind <- res[from, ] == levels[j]
      res[to, ind] <- sample(levels, sum(ind), replace = TRUE, prob = P[, j])
    }
  }
  k <- length(x$tip.label)
  label <- c(x$tip.label, as.character( (k + 1):nNodes))
  rownames(res) <- label
  # remove ancestral sequences
  if (!ancestral) res <- res[x$tip.label, , drop = FALSE]

  #return(phangorn::as.phyDat(res, type ="USER", levels = levels, return.index = TRUE))
  return(res)
}

edQt <- function(Q = c(1, 1, 1, 1, 1, 1), bf = c(0.25, 0.25, 0.25, 0.25)) {
  l <- length(bf)
  res <- matrix(0, l, l)
  res[lower.tri(res)] <- Q
  res <- res + t(res)
  res <- res * bf
  res2 <- res * rep(bf, each = l)
  diag(res) <- -colSums(res)
  res <- res / sum(res2)
  e <- eigen(res, FALSE)
  e$inv <- solve.default(e$vec)
  e
}

getP <- function(el, eig = edQt(), g = 1.0) {
 n <- length(eig$values)
 res <- .Call("getPM", eig, as.integer(n), as.double(el), as.double(g))
 attr(res, "dim") <- c(length(g), length(el))
 res
}

q <- list(rbind(c(-.5, .5), c(.5, -.5)))

# make.modelmatrix=function(m){
#     if(is.matrix(m)){
#       m=list(m)
#       for(j in 1:length(m)){
#         .check.Qmatrix(m[[j]])
#       }
#     }
#   return(m)
# }

### geiger

#?  model.matrix = .make.modelmatrix(par, model)

Q <- list(rbind(c(-.5, .5), c(.5, -.5)))
Q <- list(rbind(c(-.75, .25, .25, .25), c(.25, -.75, .25, .25), c(.25, .25, -.75, .25), c(.25, .25, .25, -.75) ))

phy <- tree
root <- 1

# number of branches and tips
nbranches <- nrow(phy$edge)
nspecies <- ape::Ntip(phy)
# identify the root
rt <- nspecies + 1 # rename this root
# postorder nodes
zphy <- ape::reorder.phylo(phy, "postorder")
el <- zphy$edge.length

# we don't know what this is (yet)
nchar <- length(Q)

# create output object #TODO decide how to generate starting values
nsim <-1
result <- array(0, dim = c(nspecies, nchar, nsim))

# chooses a new state
.get.state <- function(s, p) { # state at node anc, p between anc and cur. NOT ACTUAL P
  pp = cumsum(p[s, ])
  min(which(runif(1) < pp))
}

# for each ?
for (j in 1:nchar) {
  m <- Q[[j]] # we probably don't want/need to do this

  if (!root %in% c(1:nrow(m))) stop(paste("'root' must be a character state from 1 to ", nrow(m), sep = ""))

  # probability of change along each branch
  p = lapply(el, function(l) ape::matexpo(m * l))

  # for each ?
  for (k in 1:nsim) {
    # identify root value - starts at route
    node.value <- numeric(nspecies + ape::Nnode(zphy)) # ?
    node.value[rt] <- root

    for (i in nbranches:1) {
      cur = zphy$edge[i, 2]
      anc = zphy$edge[i, 1]
      curp = p[[i]] # prob of change along branch between anc and cur
      s = node.value[anc] # state of the ancestral node
      node.value[cur] = .get.state(s, curp)
      print(node.value[cur])
    }
    result[, j, k] <- node.value[1:nspecies]
  }
}
rownames(result) <- zphy$tip.label

#####





make.modelmatrix=function(m, model=c("discrete")){

  if(is.matrix(m)){
      m=list(m)
      for(j in 1:length(m)){
        .check.Qmatrix(m[[j]])
      }
    }
  else {
    if(is.numeric(m)) m=as.matrix(m) else stop("Supply 'm' as a matrix of rates")
  }
  if(any(diag(m)<0)) stop("'m' appears to have negative variance component(s)")
  return(m)
}

q

make.modelmatrix(q, c("discrete"))

#'@param tree A phylogenetic tree.
#'@param bp  Basepair data for the change at each branch. Should be a character vector with length euqal to the number of nodes.
#'@param noroot TRUE means exlude root from the final product. FALSE means include the root. Default is set to TRUE.

make.evol<-function(t, char=NA, noroot=TRUE){

  #get ages (including node and species)
  ages<-dispRity::tree.age(t, order = "past", fossil = TRUE, digits = 3)

  #get only node ages
  node_ages<-ages[(length(t$tip.label)+1):length(ages$ages),]

  #remove label/info for root of the tree
  if(noroot){node_ages<-node_ages[node_ages$ages!=max(node_ages$ages),] }

  #data frame format
  data<-data.frame(sp = as.numeric(node_ages$elements), edge = as.numeric(node_ages$elements),
                   hmin = node_ages$ages, hmax = node_ages$ages, char_change = char)

  #change position of labels to further back (for future plotting)
  data$hmax<-data$hmax + max(ages$ages)/10
  data$hmin<-data$hmin + max(ages$ages)/10

  #change class to evol object class
  attr(data, "class") <- c("evol", class(data))

  return(data)
}

N<-10
b<-0.1
d<-0.05
L<-100
u<-0.5
c<-1

simulate_treeseq <- function(N, b, d, L, u, c) {
  phy <- ape::rphylo(N, b, d, fossils = T)
  data <- phangorn::simSeq(
    phy,
    l = L,
    type = "USER",
    levels = as.character(0:(c-1)),
    rate = u,
    rootseq = rep("0", L),
    ancestral = TRUE
  )
  list(phy = phy, data = data, N = N, L = L, c = c, b = b, d = d)
}

simulate_treeseq(10, 2, 2, 3, 4, 1)


plot_treeseq <- function(.l) {
  phy <- .l$phy
  data <- as.character(.l$data)
  N <- .l$N
  L <- .l$L
  c <- .l$c
  palet <- c("#0088FF","#FF8800","#FFFF88","#88FF88","#8844FF","#FF4488","#DCDC00","#00DD88","#88DDFF","#DD7799","#CDCD99","#77DC11","#4477AA","#AA4477","#EEEE77","#43CD93","#0F0F4F","#400040","#000000","#FFFFFF")
  coolers <- palet[1:c]
  x1 <- as.data.frame(phy$edge)
  colnames(x1) <- c("parent", "node")
  x1$parent <- as.character(x1$parent)
  x1$node <- ifelse(x1$node <= N, paste0("t", x1$node), as.character(x1$node))
  x2 <- as.data.frame(data)
  x2 <- tibble::rownames_to_column(x2, "node")
  x2 <- dplyr::mutate_if(x2, is.factor, as.character)
  x2 <- tidyr::pivot_longer(x2, cols = -node, names_to = "position")
  x3 <- dplyr::left_join(dplyr::select(x1, node = parent), x2, by = "node")
  x3 <- dplyr::rename(x3, parent = node, parent_value = value)
  x4 <- dplyr::full_join(x2, x1, by = "node")
  x4 <- dplyr::full_join(x4, x3, by = c("parent", "position"))
  x4 <- dplyr::distinct(x4)
  x4 <- dplyr::filter(x4, !is.na(parent_value))
  x4.1 <- dplyr::mutate(x4,
                        label = node,
                        node = as.integer(stringr::str_remove(node, "^t")),
                        is_mutated = value != parent_value,
                        mutation_position = ifelse(is_mutated, stringr::str_remove(position, "^V"), NA)
  )
  x4.1 <- dplyr::filter(x4.1, !is.na(mutation_position))
  x4.1 <- dplyr::select(x4.1, parent, node, is_mutated, mutation_position)
  x4.1 <- dplyr::group_by(x4.1, parent, node)
  x4.1 <- dplyr::summarize(x4.1, edge_label_m = stringr::str_c(mutation_position, collapse = ","))
  #  x4.2 <-  x4 <- dplyr::mutate(x4,
  #                               label = node,
  #                               node = as.integer(stringr::str_remove(node, "^t")),
  #                               is_returned = is.mutated != 1,
  #                               returned_position = ifelse(is_returned, stringr::str_remove(position, "^V"), NA)
  #  )
  #  x4.2 <- dplyr::filter(x4.2, !is.na(returned_position))
  #  x4.2 <- dplyr::select(x4.2, parent, node, is_returned, returned_position)
  #  x4.2 <- dplyr::group_by(x4.2, parent, node)
  #  x4.2 <- dplyr::summarize(x4.2, edge_label_r = stringr::str_c(mutation_position, collapse = ","))
  p <- ggtree(phy, ladderize = FALSE, size = 2) +
    geom_rootedge() +
    ggtitle("Species tree")
  p <- p %<+% x4.1 +
    geom_label(aes(x = branch, label = edge_label_m), fill = "#FF00BB") +
    scale_y_continuous(limits = c(0.5, N + 0.5)) +
    theme(legend.position = "none", text = element_text(size = 18))
  #  p <- p %<+% x4.2 +
  #    geom_label(aes(x = branch, label = edge_label_r), fill = "steelblue") +
  #    scale_y_continuous(limits = c(0.5, N + 0.5)) +
  #    theme(legend.position = "none", text = element_text(size = 18))
  x5 <- dplyr::mutate(x2,
                      x = as.numeric(stringr::str_remove(position, "^V")),
                      y = as.numeric(stringr::str_remove(node, "^t")),
                      text = ifelse(value %in% 0:3, x, NA)  # <- can use this below but cleaner without
  )
  x5 <- dplyr::filter(x5, node %in% phy$tip.label)
  q <- ggplot(x5, aes(x, y, fill = value, label = "")) +
    geom_tile(width = 1, height = 1, color = "black") +
    geom_text() +
    scale_fill_manual(values = coolers) + # Add colors for levels 0 to 3
    scale_y_continuous(limits = c(0.5, N + 0.5)) +
    ggtitle("Character alignment") +
    theme_void() +
    theme(legend.position = "none")
  r_data <- data.frame(x = 1, y = 1:length(phy$tip.label), az = phy$tip.label)
  r <- ggplot(r_data, aes(x, y, label = az)) +
    scale_y_continuous() + #limits = c(0.5, length(phy$tip.label) + 0.5)
    geom_text() +
    theme_void()
  plot_grid(p, r, q, ncol = 3, rel_widths = c(1, 0.1, 1), align = "hv")
}


simulated_morpho <- sim.morpho(phy, k = 2, trait.num = 20)
sim.morpho.process(phy, k = 2, trait.num = 20)


plot(0,type='n', axes=FALSE, ann=FALSE)
grid(nx=5, ny=5, lty=2)


set.seed(123)
phy <- ape::rtree(10)
plot(phy)
simulated_morpho <- sim.morpho(phy, k = 2, trait.num = 20)


morph<-sim.morpho.process(tree=phy, trait.num = 8)

x<-simulated_morpho

#'@param x morpho object
#'@param col1 Color
#'@param col2

plot.morpho.grid <- function(x, col1 = "white", col2 ="gray"){

  #tip/taxon labels
  tips<-x[[2]][[2]]

  # make empty container
  char_matrix<-matrix(nrow= length(tips),ncol = length(x[[1]][[1]]))
  rownames(char_matrix)<-tips

# add character information to the data frame

  for (i in 1:length(x[[1]])){

    #traits for a given taxon

    traits<-x[[1]][[i]]

    #make sure the values are numeric

    char_matrix[i,]<-as.numeric(traits)}

  #character states: Question, do we want to use 0 and 1 always? Having issues coding
  #it to be whatever the character states are
  #charas<-as.numeric(unique(x[[1]][[1]]))

#Plotting

  dat <- reshape2::melt(char_matrix)

  #Question: What should we name the axes? And legend?
  ggplot2::ggplot(dat, aes(Var2, Var1, fill=factor(value))) +
    ggplot2::geom_tile(color="black") +
    ggplot2::scale_fill_manual(values = c("1"=col1, "0"=col2), name="Character") +
    ggplot2::theme_minimal() +
    ggplot2::labs(x="", y="Species") +
    ggplot2::theme(axis.text.x = element_text(angle = 90, hjust = 1))

}
#Part 1: Store the information in a dataframe
tips<-simulated_morpho[[2]][[2]]
traits<-simulated_morpho[[1]]
char_states<-simulated_morpho[[1]][i]
char_matrix<-matrix(nrow= length(tips),ncol = length(simulated_morpho[[1]][[1]]))
rownames(char_matrix)<-tips

for (i in 1:length(simulated_morpho[[1]])){
    traits<-as.numeric(simulated_morpho[[1]][[i]])
char_matrix[i,]<-as.numeric(traits)
}

#Part 2: Plot

library(ggplot2)
library(reshape2)

# Reshape matrix into a data frame
matrix_data_df <- melt(char_matrix)

# Create the plot
ggplot(matrix_data_df, aes(Var2, Var1, fill=factor(value))) +
  geom_tile(color="black") +
  scale_fill_manual(values = c("0" ="lightblue", "1"="gray"), name="Value") +
  theme_minimal() +
  labs(x="Traits", y="Taxon") +
  theme(axis.text.x = element_text(angle = 90, hjust = 1))



###########plotting without ggplot#################
set.seed(1234)
phy <- ape::rtree(10)
plot(phy)
simulated_morpho <- sim.morpho(phy, k = 2, trait.num = 10)
simulated_morpho
x<-simulated_morpho


plot.morpho.grid(x, trait.labels=FALSE, tip.labels=TRUE){
  #tip/taxon label

  tips<-x[[2]][[2]]

  # make empty container
  char_matrix<-matrix(nrow= length(tips),ncol = length(x[[1]][[1]]))
  rownames(char_matrix)<-tips

  # add character information to the data frame

  for (i in 1:length(x[[1]])){

    #traits for a given taxon

    traits<-x[[1]][[i]]

    #make sure the values are numeric

    char_matrix[i,]<-as.numeric(traits)}

  matrix_data<-char_matrix

  colors <- c("white", "blue") # White for 0, Blue for 1

  # Create the grid
  #plot(NULL)
  image(t(matrix_data[nrow(matrix_data):1, ]),
        col = colors,
        axes = FALSE)

  # Add gridlines
  grid(nx = ncol(matrix_data), ny = nrow(matrix_data), col = "black", lty = 1)



  #add axis labels
  if(tip.label){
    #rows (tip labels)
  tip_labs <- tips

  axis(2, at=seq(0,1, length=length(tip_labs)),
       labels=tip_labs, lwd=0, pos=0 cex.axis=1, las=1)

  #columns (traits)
  if(trait.labels){
    morph_labels<- seq(1,length(x$sequences[[1]]))
    axis(3, at=seq(0,1, length=length(morph_labels)),
         labels=morph_labels, lwd=0, pos=1, cex.axis=0.75)
  }
  }
}

rect(xleft = -0.036,
     ybottom = -0.056,
     xright = 1.036,
     ytop = 1.056,
     border = "black",
     lwd = 2) # Adjust `lwd` for line thickness





plot(0, xaxt = 'n', yaxt = 'n', bty = 'n', pch = '', ylab = '', xlab = '')# Add text labels for each cell
num_rows <- nrow(matrix_data)
num_cols <- ncol(matrix_data)

# Loop through rows and columns to add text
for (i in 1:num_rows) {
  for (j in 1:num_cols) {
    # Compute coordinates for text (reversed rows)
    x <- j - 0.5
    y <- num_rows - i + 0.5
    # Add the text (value of the matrix)
    text(x, y, labels = matrix_data[i, j], col = "black", cex = 0.8)
  }
}

# Add row labels beside the grid
for (i in 1:num_rows) {
  # Compute y-coordinate for each label
  y <- num_rows - i + 0.5
  # Add the row label to the left of the grid
  text(-0.5, y, labels = row_labels[i], col = "black", cex = 0.8, adj = 1)
}

### small helper function to map the fossil object onto the tree that
### same way as for the transisitions

## saved in terms of the descendant tip.
# First get the branch number by using $edge. Can determine the parent and then order
# number is the branch number. Add this to f object and call it ape.branch

f.morpho <- f
f.morpho$ape.branch <-NA
f.morpho$specimen <- NA
f.morpho$specimen <- seq(1,length(f.morpho$sp))

state_at_fossils <- matrix(nrow = length(f.morpho$sp), ncol = trait.num)
rownames(state_at_fossils) <- f.morpho$specimen


for ( i in 1:length(f$edge)){
child <- f$edge[i]
f.morpho$ape.branch[i] <- which(t$edge[,2] == child)
}



for ( tr.num in 1:trait.num){


##go through each fossil for trait n

for (spec in 1:length(f.morpho$specimen)){

branch <- f.morpho$ape.branch[[spec]]
parent <- tree.ordered$edge[branch,1]
current_state <- state_at_nodes[[which(rownames(state_at_nodes) == parent),tr.num]]

## does this branch have any changes?

if (!(f.morpho$ape.branch[spec]  %in% transition_history[[tr.num]][[1]])){
  state_at_fossils[spec,tr.num] <- current_state
  } else

    position_fossil <- f.morpho$hmin[spec]  * br.rates
    changes <- which(transition_history[[tr.num]][[1]] == f.morpho$ape.branch[spec])
    changes_along_edge <- transition_history[[tr.num]][changes,]

    ## remove changes after fossil occurance
    later <- which(changes_along_edge[,3] > position_fossil)
    changes_along_edge <- changes_along_edge[-later,]


   if ( length(changes_along_edge[,1]) == 0) {
     state_at_fossils[spec,tr.num] <- current_state
   } else {
      tran <- which(changes_along_edge$hmin == max(changes_along_edge$hmin))
      state_at_fossils[spec,tr.num] <- changes_along_edge[tran,2]
     }

}

}

sim.morpho.process <- function(tree = NULL, time.tree= NULL, br.rates = NULL,
                       k = 2, trait.num = 2){



  # check that a tree is provided
  if (is.null(tree) && is.null(time.tree)) stop(cat("Must provide a tree object"))
  # check that there are more then two states
  if (k < 2) stop(cat("Trait data must have more than 1 state"))



  ## if provided with time tree, need to transform branches in genetic distance
  ## rates can be a single value or a vector for each branch
  if (is.null(tree) && !is.null(time.tree)){
    tree <- time.tree

    if(is.null(br.rates)){
      print("No branch rate provide, using default of 0.1 for all branches")
      tree$edge.length <- time.tree$edge.length * 0.1
    } else {
      tree$edge.length <- time.tree$edge.length * br.rates
    }
  }



  ### for a symmetric Q matrix (default?)
  Q <- symmetric.Q.matrix(k)
  trait.num <- trait.num

  states <- as.character(c(0:(k-1)))

  # reorder branches in the phylogeny
  #tree.ordered <- ape::reorder.phylo(tree, "postorder")
  tree.ordered <- reorder(tree)
  edge <- tree.ordered$edge
  num.nodes <- max(edge)
  num.tips <- ape::Ntip(tree.ordered)

  ##reorder the nodes on the time tree to match the format of the genetic distance tree
  if (!is.null(time.tree)) {
    time.tree.order <- reorder(time.tree)
  } else {time.tree.order = NULL}

  edge <- tree.ordered$edge
  num.nodes <- max(edge)
  num.tips <- ape::Ntip(tree.ordered)


  ### for (trait in 1:length(triat.num))


  # container for the transitions
  transitions <- matrix(ncol = 4, nrow= 0)
  colnames(transitions) <- c("edge", "state", "hmin", "hmax")

  # collect what branches have been "done"
  sampled_branches <-  matrix(ncol=2, nrow = 0)
  colnames(sampled_branches) <- c("parent", "child")
  num_tips <- ape::Ntip(tree.ordered)

for (branches in 1:num_tips){

trace_node <- tree.ordered$tip.label[branches]
tip_path <- find_path_to_tip(tree.ordered, trace_node)
full_tip_path <- tip_path

# how many rows are the same?
num_rows_to_remove <- merge(sampled_branches, tip_path)

if (!nrow(num_rows_to_remove) == 0){
num_rows <- length(tip_path[,1])
num_rows_to_rem <- length(num_rows_to_remove[,1]) + 1
tip_path <- tip_path[num_rows_to_rem:num_rows,]

}
# if intger convert into a matrix again, easier for the rest of the set up
if (!is.matrix(tip_path)){
  tip_path <- matrix(tip_path, nrow=1, ncol = 2)
}
### get the edge numbers
edges <- c()
for(p in 1:length(tip_path[,1])){
  for (t in 1:nrow(tree.ordered$edge)) {
    if (identical(tree.ordered$edge[t, ], as.vector(tip_path[p,]))) {
      edges <- c(edges, t)
      #print(t)
    }
  }
}

### get the edge numbers for full path
full_edges <- c()
for(p in 1:length(full_tip_path[,1])){
  for (t in 1:nrow(tree.ordered$edge)) {
    if (identical(tree.ordered$edge[t, ], as.vector(full_tip_path[p,]))) {
      full_edges <- c(full_edges, t)
      #print(t)
    }
  }
}


# can use this to say if this branch has already been sampled ignore it
sampled_branches <- rbind(sampled_branches, tip_path)



# Is the process starting from the root of an internal node?
if(as.vector(tip_path[1,1]) == ape::Ntip(tree.ordered) + 1){

# define the node labels
parent <- as.integer(edge[, 1])
child <- as.integer(edge[, 2])

#identify the root
root <- as.integer(parent[!match(parent, child, 0)][1])

if(!exists("root.state")){
 # simulate the root state
root.state <- sample(states, trait.num, replace = TRUE, prob = rep(1/k, k)) #later prob would need to take into account for bf
# set the root state as current state. This will be updated
}
current_state <- as.numeric(root.state)

} else{
#### if we are starting from an internal node we need to get the state at that node
parent_node_here <- as.vector(tip_path[1,1])
associated_edge_length <- which(tree.ordered$edge[,2] == parent_node_here)



# was the a transition along this branch, if not go back to the node before
# is this the root? if so set root state
matrix_row <- which(transitions[,"edge"] == associated_edge_length)

if (length(matrix_row) == 1){
last_transition <- max(which(transitions[,"edge"] == associated_edge_length))
current_state <- as.numeric(transitions[last_transition,"state"])

} else {
  # what edges do we have transitions for ?
  transitions_edges <- unique(as.numeric(as.vector(sort(transitions[,"edge"]))))
  associated_edge_length <-  intersect(transitions_edges, full_edges)
if (length(associated_edge_length)  == 0){
  current_state <- as.numeric(root.state)
} else {
  associated_edge_length <- max(associated_edge_length)
  last_transition <- max(which(transitions[,"edge"] == associated_edge_length))
  current_state <- as.numeric(transitions[last_transition,"state"])
}

  #}
}
# get the rate of change acorss the matrix

}
current_rate <- Q[(current_state+ 1),(current_state+1)] * -1


# calculate the total distance in the evolutionary process
blength <- 0
for ( bl in 1:length(edges)){
blength <- blength +  tree.ordered$edge.length[edges][bl]
}

## rate is taken from the Qmatrix
rand = rpois(1, blength*current_rate)

if (rand > 0 ){
# get the minmum changes as we need to restart the process as the rate may change. For a
# symetic matrix however we have all the changes now.

# for an asymetric matrix
#h = min(runif(rand, min = 0, max = blength))

# for a symetric matrix
h = runif(rand, min = 0, max = blength)
h <- sort(h)

P  <- ape::matexpo(Q * blength)

# calculated the number of changes there for we need to ensure that a new state is sampled.
state_changes <- c()
new_state <- current_state
for (r in 1:rand){
while(as.numeric(new_state == current_state) ){
new_state <- sample(states, 1, replace = TRUE, prob = P[,(current_state+1)])
}
  state_changes <- rbind(state_changes, new_state)
  current_state <- as.numeric(new_state)
  #print(current_state)

}


## if we have more than one edge we need to determine which branch and where along
# the branch the transitions occurred
if (length(edges) == 1){
  for (p in 1:length(h)){
    position <- h[p]
    exact_branch_position <- h[p]
    add_t <- c(as.numeric(edges[1]), state_changes[p], as.numeric(exact_branch_position), as.numeric(exact_branch_position))
    transitions <- rbind(transitions, add_t)
  }

} else{

## create matrix of the transitions on the correct branch length
for (p in 1:length(h)){
  position <- h[p]
  br_len <- 0
for ( e in 1:length(edges)){
  br_len <- br_len + tree.ordered$edge.length[edges[e]]
if (position < br_len){
  if (e > 1){
  exact_branch_position <- h[p] - sum(tree.ordered$edge.length[1:(e-1)])
  } else {
    exact_branch_position <- h[p]
  }
  add_t <- c(edges[e], state_changes[p], exact_branch_position, exact_branch_position)
  transitions <- rbind(transitions, add_t)
  break
}
}
}

}
}

}

  return(transitions)
}

##### to get from the tip to the root of the tree
find_path_to_tip <- function(tree, tip) {
  # Ensure the tip is valid
  if (!tip %in% tree$tip.label) {
    stop("The specified tip is not found in the tree.")
  }

  # Get the edge matrix and root node
  edge <- tree$edge
  root <- ape::Ntip(tree) + 1

  # the node number of the tip
  tip_node <- which(tree$tip.label == tip)

  # Start from the tip node and move upward toward the root
  path <- tip_node
  current_node <- tip_node

  # loop until reaching the root node
  while (current_node != root) {
    # find the parent of the current node
    parent_node <- edge[edge[, 2] == current_node, 1]
    path <- c(parent_node, path)
    current_node <- parent_node
  }
  length(path)-1
  output <- matrix(nrow =length(path)-1, ncol=2)
  colnames(output) <- c("parent", "child")

  for (i in 1:length(path)-1){
    output[i,] <- c(path[i], path[i+1])
  }

  return(output)
}

setwd("~/Documents/MkvDiscretizedImplementation/")

library(phytools)
library(phangorn)
library(geiger)

phy <- ape::rtree(10)

for (SIM in 1:250){
  
  phylo <- read.tree(paste("data/sim_trees/sim_tree_", SIM, ".tre", sep=""))
  
  
  NCAT = 4
  
  
  ## this is taken directly from phangorn. 
  
  disc.gamma <- function(alpha, k) {
    if (k == 1) return(1)
    # Compute quantiles to divide the gamma distribution into k categories
    quants <- qgamma((1:(k - 1)) / k, shape = alpha, rate = alpha)
    diff(c(0, pgamma(quants * alpha, alpha + 1), 1)) * k
  }
  
  gammaCats <- disc.gamma(alpha=1, k=NCAT)
  
  
  NCHAR=10
  
  char <- list()
  rate <- rep(NA, NCHAR)
  
  for (i in 1:NCHAR) {
    
    rate[i] <- gammaCats[sample(1:NCAT, 1)]
    
    Q <- matrix (c( -rate[i], rate[i], rate[i], -rate[i]), 2, 2, byrow = T, dimnames=list(0:1,0:1))
    
    char[[i]] <- sim.Mk(tree=phylo, Q=Q, nsim=1)
    
    while (length(levels(char[[i]])) == 1) {
      rate[i] <- gammaCats[sample(1:NCAT, 1)]
      
      Q <- matrix (c( -rate[i], rate[i], rate[i], -rate[i]), 2, 2, byrow = T, dimnames=list(0:1,0:1))
      
      char[[i]] <- sim.Mk(tree=phyl, Q=Q, nsim=1)
    }
    
  }
  
  hist(rate)
  mean(rate)
  
  rate_p2 <- rate
  
  v <- list()
  
  for (i in 1:NCHAR) {
    v[[i]] <- as.vector(char[[i]])
    names(v[[i]]) <- names(char[[i]])
  }
  
  dataMatrix <- as.data.frame(do.call(rbind, v))
  
  write.nexus.data(dataMatrix, file=paste("data/Sim1000Chars_Tree", SIM, "_mMkv_DiscretizedGamma.nex", sep=""), format="standard", interleaved = F)
  write.csv(rate, file=paste("data/Sim1000Chars_Tree", SIM, "_mMkv_DiscretizedGamma.rates.txt", sep=""), quote = F)
  
}

## ----setup, include = FALSE---------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  fig.width = 7,
  fig.height = 5
)

## ----tree---------------------------------------------------------------------
library(MorphSim)
library(TreeSim)
library(FossilSim)

set.seed(1234)
tree <- TreeSim::sim.bd.taxa(n = 10, numbsim = 1,
                              lambda = 0.1, mu = 0.05)[[1]]

## ----basic--------------------------------------------------------------------
morpho_data <- sim.morpho(
  time.tree = tree,
  k = 2,
  trait.num = 10,
  br.rates = 0.1
)
morpho_data

## ----morpho_check-------------------------------------------------------------
is.morpho(morpho_data)

## ----access-------------------------------------------------------------------
# Tip data as a matrix
get.matrix(morpho_data, seq = "tips")

# Transition history for a specific trait (including root state)
get.transitions(morpho_data, trait = 1)

## ----clock--------------------------------------------------------------------
# Strict clock
strict_data <- sim.morpho(time.tree = tree, k = 2, trait.num = 10,
                           br.rates = 0.1)

# Relaxed clock (random rates per branch for illustration)
relaxed_rates <- runif(nrow(tree$edge), min = 0.01, max = 0.2)
relaxed_data <- sim.morpho(time.tree = tree, k = 2, trait.num = 10,
                            br.rates = relaxed_rates)

## ----mkv----------------------------------------------------------------------
mkv_data <- sim.morpho(time.tree = tree, k = 2, trait.num = 10,
                        br.rates = 0.1, variable = TRUE)

## ----acrv_gamma---------------------------------------------------------------
gamma_data <- sim.morpho(
  time.tree = tree, k = 2, trait.num = 10, br.rates = 0.1,
  ACRV = "gamma", alpha.gamma = 1, ACRV.ncats = 4
)

# The discrete rate categories
gamma_data$model$RateVar

# Which category was assigned to each trait
gamma_data$model$RateVarTrait

## ----acrv_lgn-----------------------------------------------------------------
lgn_data <- sim.morpho(
  time.tree = tree, k = 2, trait.num = 10, br.rates = 0.1,
  ACRV = "lgn", meanlog = 0, sdlog = 1, ACRV.ncats = 4
)

# The discrete rate categories
lgn_data$model$RateVar

## ----acrv_user----------------------------------------------------------------
user_data <- sim.morpho(
  time.tree = tree, k = 2, trait.num = 10, br.rates = 0.1,
  ACRV = "user", define.ACRV.rates = c(0.5, 1, 1.5, 2),
  ACRV.ncats = 4
)

## ----mkv_gamma----------------------------------------------------------------
mkvg_data <- sim.morpho(
  time.tree = tree, k = 3, trait.num = 10, br.rates = 0.1,
  variable = TRUE, ACRV = "gamma", alpha.gamma = 1, ACRV.ncats = 4
)

## ----custom_q-----------------------------------------------------------------
ord_Q <- matrix(c(
  -0.5, 0.5, 0.0,
   0.3333333, -0.6666667, 0.3333333,
   0.0, 0.5, -0.5
), nrow = 3, byrow = TRUE)

ordered_data <- sim.morpho(
  time.tree = tree, k = 3, trait.num = 10, br.rates = 0.1,
  define.Q = ord_Q
)

## ----root_state---------------------------------------------------------------
fixed_root <- sim.morpho(
  time.tree = tree, k = 3, trait.num = 10,
  br.rates = 0.1, root.state = 0
)

## ----strict-------------------------------------------------------------------
strict_data <- sim.morpho(
  time.tree = tree, k = 3, trait.num = 10,
  br.rates = 0.1, full.states = TRUE
)

## ----partitions---------------------------------------------------------------
part_data <- sim.morpho(
  time.tree = tree,
  k = c(2, 3, 4),
  trait.num = 20,
  partition = c(10, 5, 5),
  br.rates = 0.1
)

# Model specification per partition
part_data$model$Specified

## ----combine------------------------------------------------------------------
# Partition 1: ordered characters with a custom Q matrix
ord_Q <- matrix(c(
  -0.5, 0.5, 0.0,
   0.3333333, -0.6666667, 0.3333333,
   0.0, 0.5, -0.5
), nrow = 3, byrow = TRUE)

partition_1 <- sim.morpho(
  time.tree = tree, k = 3, trait.num = 10,
  br.rates = 0.1, define.Q = ord_Q
)

# Partition 2: unordered binary characters with MkV + Gamma
partition_2 <- sim.morpho(
  time.tree = tree, k = 2, trait.num = 10,
  br.rates = 0.1, variable = TRUE,
  ACRV = "gamma", alpha.gamma = 1, ACRV.ncats = 4
)

# Combine into a single morpho object
combined <- combine.morpho(partition_1, partition_2)

# The combined object has 20 traits total
length(combined$sequences$tips[[1]])

# Model specifications from both partitions are preserved
combined$model$Specified

## ----fossils------------------------------------------------------------------
# Simulate fossil and extant sampling
f <- FossilSim::sim.fossils.poisson(rate = 0.1, tree = tree,
                                     root.edge = FALSE)
f2 <- FossilSim::sim.extant.samples(fossils = f, tree = tree, rho = 0.5)

# Simulate morphological data with fossils
fossil_data <- sim.morpho(
  time.tree = tree, k = c(2,3), trait.num = 10, partition = c(5,5),
  br.rates = 0.1, fossil = f2
)

# Sampled ancestor data
names(fossil_data$sequences$SA)

## ----missing_random-----------------------------------------------------------
missing_random <- sim.missing.data(
  data = fossil_data, method = "random",
  seq = "tips", probability = 0.3
)

## ----missing_partition--------------------------------------------------------
part_fossil <- sim.morpho(
  time.tree = tree, k = c(2, 3), trait.num = 10,
  partition = c(5, 5), br.rates = 0.1, fossil = f2
)

# Hard tissues (partition 1): 10% missing
# Soft tissues (partition 2): 70% missing
missing_part <- sim.missing.data(
  data = part_fossil, method = "partition",
  seq = "tips", probability = c(0.1, 0.7)
)

## ----missing_rate-------------------------------------------------------------
gamma_fossil <- sim.morpho(
  time.tree = tree, k = 2, trait.num = 20, br.rates = 0.1,
  ACRV = "gamma", alpha.gamma = 1, ACRV.ncats = 4, fossil = f2
)

# Slow characters preserved, fast characters increasingly lost
missing_rate <- sim.missing.data(
  data = gamma_fossil, method = "rate",
  seq = "tips", probability = c(0, 0.2, 0.5, 0.8)
)

## ----missing_trait------------------------------------------------------------
missing_trait <- sim.missing.data(
  data = fossil_data, method = "trait",
  seq = "tips", probability = 1, traits = c(1, 2, 3)
)

## ----missing_taxa-------------------------------------------------------------
# Remove all data from the first two taxa
taxa_to_remove <- names(fossil_data$sequences$tips)[1:2]
missing_taxa <- sim.missing.data(
  data = fossil_data, method = "taxa",
  seq = "tips", probability = 1, taxa = taxa_to_remove
)

## ----missing_extinct, eval = FALSE--------------------------------------------
# missing_extinct <- sim.missing.data(
#   data = fossil_data, method = "extinct",
#   seq = "tips", probability = 0.5
# )

## ----missing_trait_taxa-------------------------------------------------------
missing_trait_taxa <- sim.missing.data(
  data = fossil_data, method = "trait_taxa",
  seq = "tips", probability = 0.8,
  traits = c(1, 2, 3),
  taxa = taxa_to_remove
)

## ----stats--------------------------------------------------------------------
stats <- stats.morpho(fossil_data)
stats$Statistics
stats$Tree

## ----convergent---------------------------------------------------------------
# Which traits are convergent?
get.convergent(fossil_data)

# Detail for a specific trait
get.convergent(fossil_data, trait = 1)

## ----transitions--------------------------------------------------------------
get.transitions(fossil_data, trait = 1)

## ----plot_basic, fig.height = 6-----------------------------------------------
plot(fossil_data, trait = 6, timetree = FALSE,
     show.fossil = FALSE, reconstructed = FALSE,
     show.convergent = FALSE)

## ----plot_fossils, fig.height = 6---------------------------------------------
plot(fossil_data, trait = 6, timetree = TRUE,
     show.fossil = TRUE, reconstructed = FALSE,
     show.convergent = FALSE)

## ----plot_recon, fig.height = 6-----------------------------------------------
plot(fossil_data, trait = 6, timetree = TRUE,
     show.fossil = TRUE, reconstructed = TRUE,
     show.convergent = FALSE)

## ----plot_convergent, fig.height = 6------------------------------------------
plot(fossil_data, trait = 6, timetree = TRUE,
     show.fossil = TRUE, reconstructed = TRUE,
     show.convergent = TRUE)

## ----plot_default, eval = FALSE-----------------------------------------------
# plot(fossil_data, trait = 7)

## ----grid, fig.height = 5-----------------------------------------------------
plotMorphoGrid(fossil_data, seq = "tips")

## ----export, eval = FALSE-----------------------------------------------------
# # Full tree
# write.morpho(fossil_data, file = "tree.tre", type = "tree")
# 
# # Reconstructed tree
# write.morpho(fossil_data, file = "recon_tree.tre", type = "tree",
#              reconstructed = TRUE)
# 
# # Character matrix
# write.morpho(fossil_data, file = "matrix.nex", type = "matrix")
# 
# # Reconstructed matrix
# write.morpho(fossil_data, file = "recon_matrix.nex", type = "matrix",
#              reconstructed = TRUE)
# 
# # Fossil ages (compatible with RevBayes)
# write.morpho(fossil_data, file = "ages.tsv", type = "ages")
# 
# # With age uncertainty
# write.morpho(fossil_data, file = "ages.tsv", type = "ages",
#              uncertainty = 2)

## ----export_ape, eval = FALSE-------------------------------------------------
# ape::write.tree(fossil_data$trees$EvolTree, file = "full_tree.tre")
# ape::write.nexus.data(fossil_data$sequences$tips, file = "full_matrix.nex",
#                       format = "standard")

## ----ggtree, eval = FALSE-----------------------------------------------------
# library(treeio)
# library(ggtree)
# library(ggplot2)
# library(tibble)
# 
# # Convert tip data to a data frame
# tip_df <- as.data.frame(t(as.data.frame(fossil_data$sequences$tips)))
# colnames(tip_df) <- paste0("trait_", seq_len(ncol(tip_df)))
# tip_df$node <- match(rownames(tip_df),
#                      fossil_data$trees$EvolTree$tip.label)
# 
# # Create treedata object
# td <- treedata(phylo = fossil_data$trees$EvolTree,
#                data = as_tibble(tip_df))
# 
# # Plot tree with coloured tip points for one trait
# ggtree(td) +
#   geom_tiplab(offset = 0.01) +
#   geom_tippoint(aes(color = factor(trait_1)), size = 3) +
#   scale_color_brewer(palette = "Set2", name = "State") +
#   theme(legend.position = "right")
# 
# # Plot tree with heatmap of all traits
# heat_df <- tip_df[, grep("trait_", colnames(tip_df))]
# heat_df[] <- lapply(heat_df, factor)
# 
# gheatmap(ggtree(td) + geom_tiplab(offset = 0.05),
#          heat_df, offset = 0.02, width = 0.5,
#          colnames_angle = 90, hjust = 1) +
#   scale_fill_brewer(palette = "Set2", name = "State")

tree <- ape::rtree(6)
tree$edge.length <-  tree$edge.length * 10

q <- list(rbind(c(-.5, .5), c(.5, -.5)))
geiger::sim.char(tree, q, model = "discrete", 10)

x <- tree

# count number of tips and branches
nedges <- nrow(x$edge) #nbranches
tips <- ape::Ntip(x) #nspecies
# identify the root
root = tips+1 # root

# "postorder" branch lengths
zphy <- ape::reorder.phylo(tree, "postorder")

el=zphy$edge.length
nchar<-length(model.matrix);
result<-array(0, dim=c(nspecies, nchar, nsim))

#' This function plots your morpho object to a character matrix
#' @param x an object of class "morpho"
#' @param col a string bound by c() containing colors to match the amount of possible character states
#' @export 

plot.morpho.grid <- function(x, xlab = "", ylab = "",name = "" ,  col = c("white", "gray", "lightblue", "pink", "yellow", "green", "orange")){
  #tip/taxon labels
  tips<-x[[2]][[2]]
  
  # make empty container
  char_matrix<-matrix(nrow= length(tips),ncol = length(x[[1]][[1]]))
  rownames(char_matrix)<-tips
  
  # add character information to the data frame
  
  for (i in 1:length(x[[1]])){
    
    #traits for a given taxon
    
    traits<-x[[1]][[i]]
    
    #make sure the values are numeric
    
    char_matrix[i,]<-as.numeric(traits)
  }
  
  #character states: Question, do we want to use 0 and 1 always? Having issues coding
  #it to be whatever the character states are
  #charas<-as.numeric(unique(x[[1]][[1]]))
  
  #Plotting
  
  dat <- reshape2::melt(char_matrix)
  
  unique_states <- sort(unique(dat$value))
  color_mapping <- setNames(col[1:length(unique_states)], unique_states)
  
  #Question: What should we name the axes? And legend?
  ggplot2::ggplot(dat, aes(Var2, Var1, fill=factor(value))) +
    ggplot2::geom_tile(color="black") +
    ggplot2::geom_text(aes(label = value), color = "black", size = 4) +
    ggplot2::scale_fill_manual(values = color_mapping, name = name) +
    ggplot2::theme_minimal() +
    ggplot2::labs(x = xlab, y = ylab) +  # add it to function parameters :TB
    ggplot2::theme(
      axis.text.x = element_blank(),
      axis.text.y = element_text(size = 10)
    )
}

#turning it into a function
#'@param t phylogenetic tree
#'@param base.pair vector which contains the basepair change data to be displayed on the tree. Length should equal number of nodes
#'@param base.pair.color vector for colors of the basepair labels
plot.evolution<-function(t, base.pair, base.pair.col=NULL){
  
  #getting node ages
  
  #get node and tip ages
  ages<-dispRity::tree.age(t, 2.94, order = "past", fossil = TRUE, digits = 3)
  
  #get only node ages
  node_ages<-ages[(length(t$tip.label)+1):length(ages$ages),]
  
  data<-data.frame(matrix(nrow=length(TreeSim::getx(t)), ncol=4))
  colnames(data)<-c("sp","edge", "hmin", "hmax")
  data$hmin<-node_ages$ages
  data$hmax<-node_ages$ages
  data$sp<-as.numeric(node_ages$elements)
  data$edge<-as.numeric(node_ages$elements)
  data<-FossilSim::as.fossils(test.obj)
  bp_data<-base.pair
  
  plot(data[2:length(data$sp),],tree = t, pch=bp_data, fossil.col=base.pair.col)}

#plot

plot.evolution(t=t, test.obj.bp, base.pair.col=c("red", "green", "blue", "yellow"))

#Part 4: Make everything work without using it as a fossil class#

#outline of what the object should look like

test<-data.frame(sp = numeric(), edge = numeric(), hmin = numeric(), hmax = numeric(), bpchange=character())
class(test)<-"evol"

setClass("evol", slots= data.frame(sp = numeric(), edge = numeric(), hmin = numeric(), hmax = numeric(), bpchange=character()))



#change plot.fossils function to plot.evolution

#first: plot dots only
#plot fossils function
plot.evol = function(x, tree, show.fossils = TRUE, show.tree = TRUE, show.ranges = FALSE,
                     # age info/options
                     show.strata = FALSE, strata = 1, max.age = NULL, interval.ages = NULL, binned = FALSE, show.axis = TRUE,
                     # proxy stuff
                     show.proxy = FALSE, proxy.data = NULL,
                     show.preferred.environ = FALSE, preferred.environ = NULL,
                     # taxonomy
                     show.taxonomy = FALSE, taxonomy = NULL, show.unknown = FALSE, rho = 1,
                     # tree appearance
                     root.edge = TRUE, hide.edge = FALSE, edge.width = 1, show.tip.label = FALSE, align.tip.label = FALSE, reconstructed = FALSE,
                     # fossil appearance
                     fossil.col = 1, range.col = rgb(0,0,1), extant.col = 1, taxa.palette = "viridis",
                     col.axis = "gray35", cex = 1.2, pch = evol$bp_change, ...) {
  
  evol = x
  
  # hard coded options for tree appearance
  edge.color = "black"
  edge.lty = 1
  font = 3 # italic
  tip.color = "black"
  label.offset = 0.02
  align.tip.label.lty = 3
  underscore = FALSE
  plot = TRUE
  node.depth = 1
  no.margin = FALSE
  adj = NULL
  srt = 0
  
  if(!show.tree) align.tip.label = TRUE
  
  if(class(evol)[1]!="evol")
    stop("x must be an object of class \"evol\"")
  
  if(!"phylo" %in% class(tree))
    stop("tree must be an object of class \"phylo\"")
  
  if(!all( as.vector(na.omit(evol$edge)) %in% tree$edge))
    stop("Mismatch between evol and tree objects")
  
  if(!(rho >= 0 && rho <= 1))
    stop("rho must be a probability between 0 and 1")
  
  # tolerance for extant tips and interval/ fossil age comparisons
  tol = min((min(tree$edge.length)/100), 1e-8)
  
  # If there are no extant samples, simulate extant samples
  #if(!any( abs(evol$hmax) < tol )){
  #do we need this?
  # evol = sim.extant.samples(evol, tree = tree, rho = rho)
  #}
  
  if(is.null(tree$root.edge))
    root.edge = FALSE
  
  if(is.null(max.age))
    ba = tree.max(tree, root.edge = root.edge)
  else ba = max.age
  
  offset = 0 # distance from youngest tip to present
  if(!is.null(tree$origin.time)) offset = min(n.ages(tree))
  
  # note max.age defined above based on the complete tree
  if(reconstructed){
    out = reconstructed.tree.evol.objects(evol, tree)
    evol = out$evol
    tree = out$tree
    if(is.null(tree$root.edge)) root.edge = FALSE
  }
  
  # check the tree
  Ntip <- length(tree$tip.label)
  if (Ntip < 2) {
    warning("found less than 2 tips in the tree")
    return(NULL)
  }
  
  if (any(tabulate(tree$edge[, 1]) == 1))
    stop("there are single (non-splitting) nodes in your tree; you may need to use collapse.singles()")
  
  if(!ape::is.rooted(tree))
    stop("tree must be rooted")
  
  if(is.null(tree$edge.length))
    stop("tree must have edge lengths")
  
  Nnode <- tree$Nnode
  if (any(tree$edge < 1) || any(tree$edge > Ntip + Nnode))
    stop("tree badly conformed; cannot plot. Check the edge matrix.")
  ROOT <- Ntip + 1
  
  # check interval ages & proxy data
  if(any(evol$hmin != evol$hmax)) binned = TRUE
  
  if(show.strata || show.proxy){
    if( (is.null(interval.ages)) && (is.null(strata)) )
      stop("To plot interval info specify interval.ages OR number of strata, else use show.strata = FALSE")
  }
  
  if(show.proxy && is.null(proxy.data))
    stop("Specify sampling profile")
  
  if(show.proxy){
    if(!is.null(interval.ages)){
      if( (length(interval.ages) - 1) != length(proxy.data) )
        stop("Make sure number of sampling proxy data points matches the number of intervals")
    } else {
      if(strata != length(proxy.data))
        stop("Make sure number of sampling proxy data points matches the number of intervals")
    }
    if(any(is.na(proxy.data)))
      stop("Function can't handle NA proxy values right now, please use 0")
  }
  
  # check taxonomy data
  if(show.taxonomy && is.null(taxonomy))
    stop("Specify taxonomy using \"taxonomy\"")
  if(show.taxonomy && !"taxonomy" %in% class(taxonomy))
    stop("taxonomy must be an object of class \"taxonomy\"")
  if(show.taxonomy && !all(evol$edge %in% taxonomy$edge))
    stop("Mismatch between fossils and taxonomy objects")
  if(show.taxonomy && !all(tree$edge %in% taxonomy$edge))
    stop("Mismatch between tree and taxonomy objects")
  
  # collect data for plotting the tree
  type = "phylogram"
  direction = "rightwards"
  horizontal = TRUE # = "rightwards"
  
  xe = tree$edge # used in the last part of the fxn
  yy = numeric(Ntip + Nnode)
  TIPS = tree$edge[tree$edge[, 2] <= Ntip, 2]
  yy[TIPS] = 1:Ntip
  
  yy = ape::node.height(tree)
  xx = ape::node.depth.edgelength(tree)
  
  if(root.edge)
    xx = xx + tree$root.edge
  
  # x.lim is defined by max(ba, tree age)
  if(ba - offset > max(xx))
    xx = xx + (ba - offset - max(xx))
  
  x.lim = c(0, NA)
  pin1 = par("pin")[1]
  strWi = graphics::strwidth(tree$tip.label, "inches", cex = par("cex"))
  xx.tips = xx[1:Ntip] * 1.04
  alp = try(stats::uniroot(function(a) max(a * xx.tips + strWi) - pin1, c(0, 1e+06))$root, silent = TRUE)
  if (is.character(alp)) {
    tmp = max(xx.tips)
    if (show.tip.label)
      tmp = tmp * 1.5
  } else {
    tmp = if (show.tip.label)
      max(xx.tips + strWi/alp)
    else max(xx.tips)
  }
  if (show.tip.label)
    tmp = tmp + label.offset
  x.lim[2] = tmp
  
  if(show.unknown)
    y.lim = c(0, Ntip)
  else y.lim = c(1, Ntip)
  
  use.species.ages = FALSE
  
  # define interval ages
  if(show.strata || show.axis || binned || show.proxy){
    if( (is.null(interval.ages)) ){
      s1 = (ba - offset) / strata # horizon length (= max age of youngest horizon)
      horizons.max = seq(s1 + offset, ba, length = strata)
      horizons.min = horizons.max - s1
      s1 = horizons.max - horizons.min
    } else {
      horizons.min = utils::head(interval.ages, -1)
      horizons.max = interval.ages[-1]
      ba = max(horizons.max)
      s1 = rev(horizons.max - horizons.min)
      strata = length(horizons.max)
    }
    # the following is to catch datasets with hmin != hmax combinations that differ from user specified interval bounds
    if(binned & any(evol$hmax != evol$hmin)){
      if(length( unlist( sapply(evol$hmax, function(x){
        if(x < tol) return(1)
        which(abs(horizons.max - x) < tol) }))) != length(evol$hmax)){
        use.species.ages = TRUE
        binned = FALSE
      }
    }
  }
  
  # the order in which you use plot and par here is very important
  if(show.proxy){
    old.par = par("usr", "mar", "oma", "xpd", "mgp","fig")
    par(mar=c(1, 3.5, 0, 0.5)) # to reduce the margins around each plot - bottom, left, top, right -- this is harder to manipulate
    par(oma=par("oma") + c(2, 0, 2, 0)) # to add an outer margin to the top and bottom of the graph -- bottom, left, top, right
    par(xpd=NA) # allow content to protrude into outer margin (and beyond)
    par(mgp=c(1.5, .5, 0)) # to reduce the spacing between the figure plotting region and the axis labels -- axis label at 1.5 rows distance, tick labels at .5 row
    par(fig=c(0,1,0,0.4))
    graphics::plot.default(0, type = "n", xlim = x.lim, ylim = y.lim, xlab = "", ylab = "", axes = FALSE, asp = NA, ...)
    par(fig=c(0,1,0.4,1))
  }
  else{
    old.par = par("xpd", "mar")
    par(xpd=NA)
    par(mar=c(5,4,2.5,2) + 0.1) ## default is c(5,4,4,2) + 0.1
    # open a new plot window
    graphics::plot.default(0, type = "n", xlim = x.lim, ylim = y.lim, xlab = "", ylab = "", axes = FALSE, asp = NA, ...)
    
  }
  
  if (plot) {
    
    # add colored strata
    # rect(xleft, ybottom, xright, ytop)
    if(show.strata || show.axis || show.proxy){
      
      # y-axis:
      #y.bottom = 0
      #y.top = max(yy)+1
      y.bottom = 0.5
      y.top = max(yy) + 0.5
      
      # x-axis:
      #s1 = ba / strata
      x.left = 0 - (ba - offset - max(xx))
      x.right = x.left + s1[1]
      cc = 1 # color switch
      
      axis.strata = x.left
      
      for(i in 1:strata){
        if(cc %% 2 == 0)
          col="grey90"
        else
          col="grey95"
        if(show.strata)
          rect(xleft = x.left, xright = x.right, ybottom = y.bottom, ytop = y.top, col=col, border=NA)
        x.left = x.right
        x.right = x.left + s1[i+1]
        cc = cc + 1
        axis.strata = c(axis.strata, x.left)
      }
      
      x.labs = rev(round(c(offset, horizons.max),2))
      
      if(show.proxy)
        labs = FALSE
      else
        labs = x.labs
      
      if(show.axis){
        axis(1, col = col.axis, at = axis.strata, labels = labs, lwd = 2, line = 0.5, col.axis = col.axis, cex.axis = .9)
        if(!show.proxy)
          mtext(1, col = 'grey35', text="Time before present", line = 3, cex = 1.2)
      }
    }
    
    # plot the tree
    if(show.tree)
      ape::phylogram.plot(tree$edge, Ntip, Nnode, xx, yy, horizontal, edge.color, edge.width, edge.lty)
    
    # format the root edge
    if (root.edge && show.tree && !hide.edge) {
      rootcol = if(length(edge.color) == 1)
        edge.color
      else "black"
      rootw = if(length(edge.width) == 1)
        edge.width
      else 1
      rootlty = if(length(edge.lty) == 1)
        edge.lty
      else 1
      
      # plot the root edge
      segments(xx[ROOT] - tree$root.edge, yy[ROOT], xx[ROOT], yy[ROOT], col = rootcol, lwd = rootw, lty = rootlty)
    }
    
    # format tip labels
    if (show.tip.label) {
      
      # x and y co-ordinates
      adj = 0
      MAXSTRING <- max(graphics::strwidth(tree$tip.label, cex = cex))
      loy <- 0
      lox <- label.offset + MAXSTRING * 1.05 * adj
      
      if (is.expression(tree$tip.label))
        underscore <- TRUE
      if (!underscore)
        tree$tip.label <- gsub("_", " ", tree$tip.label)
      
      if (align.tip.label) {
        xx.tmp <- max(xx[1:Ntip])
        yy.tmp <- yy[1:Ntip]
        segments(xx[1:Ntip], yy[1:Ntip], xx.tmp, yy.tmp, lty = align.tip.label.lty)
      }
      else {
        xx.tmp <- xx[1:Ntip]
        yy.tmp <- yy[1:Ntip]
      }
      text(xx.tmp + lox, yy.tmp + loy, tree$tip.label,adj = adj, font = font, srt = srt, cex = cex, col = tip.color)
    }
    
    if(length(evol$sp)){
      # binned but assigned to an independent set of interval ages
      if(use.species.ages)
        evol$h = (evol$hmax - evol$hmin)/2 + evol$hmin
      # not binned
      else if(!binned) evol$h = evol$hmin
      # binned & already assigned to intervals
      else if (binned & any(evol$hmin != evol$hmax))
        evol$h = evol$hmax
      # binned but not assigned to intervals
      else if(binned & all(evol$hmin == evol$hmax))
        evol$h = sim.interval.ages(evol, tree, interval.ages = c(0, horizons.max))$hmax
      
      evol$col = fossil.col
      
      # taxonomy colours
      if(show.taxonomy){
        sps = unique(evol$sp)
        col = grDevices::hcl.colors(length(sps), palette = taxa.palette)
        j = 0
        for(i in sps){
          j = j + 1
          evol$col[which(evol$sp == i)] = col[j]
        }
      }
      
      evol$col[which(evol$h < tol)] = extant.col
      #needs to be changed later
      if(show.fossils || show.ranges){
        if(binned) {
          evol$r = sapply(evol$h - offset, function(x) {
            if(x < tol) return(max(xx) - x)
            y = max(which(abs(horizons.max - x) < tol))
            max(xx) - horizons.max[y] + (rev(s1)[y]/2) })
        } else {
          evol$r = max(xx) - evol$h + offset
        }
      }
      
      # fossils unaffiliated with taxonomy
      if( show.unknown && any(is.na(evol$sp)) ){
        unknownx = evol[ which(is.na(evol$sp)), ]$r
        unknowny = rep(0, length(unknownx))
        points(unknownx, unknowny, cex = cex, pch = pch, col = fossil.col)
        evol = na.omit(evol)
      }
      
      # evol
      if(show.fossils & !show.ranges){
        points(evol$r, yy[evol$edge], cex = cex, pch = pch, col = evol$col)
      }
      
      # ranges
      if(show.ranges){
        
        # for show.taxonomy = TRUE
        # fetch oldest and youngest edge associated with a species
        # deal with the oldest part of the ranges
        # deal with the youngest part of the ranges
        # everyting inbetween
        if(show.taxonomy){
          
          for(i in unique(evol$sp)){
            
            mn = min(evol$h[which(evol$sp == i)])
            mx = max(evol$h[which(evol$sp == i)])
            edge.mn = evol$edge[which(evol$sp == i & evol$h == mn)]
            edge.mx = evol$edge[which(evol$sp == i & evol$h == mx)]
            
            edges = find.edges.inbetween(edge.mn, edge.mx, tree)
            
            col = evol$col[which(evol$sp == i)]
            
            for(j in edges){
              
              # singletons
              if(mn == mx) {
                range = evol$r[which(evol$edge == edge.mn & evol$sp == i)]
              }
              
              # single edge
              else if(edge.mn == edge.mx) range = evol$r[which(evol$edge == edge.mn
                                                               & evol$sp == i)]
              
              # multiple edges: FA edge
              else if(j == edge.mx) {
                range =  c(evol$r[which(evol$edge == edge.mx & evol$sp == i)],
                           max(xx) + offset - taxonomy$end[which(taxonomy$edge == j & taxonomy$sp == i)])
              }
              # multiple edges: LA edge
              else if(j == edge.mn){
                range =  c(evol$r[which(evol$edge == edge.mn & evol$sp == i)],
                           max(xx) + offset - taxonomy$start[which(taxonomy$edge == j & taxonomy$sp == i)])
              }
              # multiple edges: in-between edges
              else{
                range =  c(max(xx) + offset - taxonomy$start[which(taxonomy$edge == j & taxonomy$sp == i)],
                           max(xx) + offset - taxonomy$end[which(taxonomy$edge == j & taxonomy$sp == i)])
              }
              # plot ranges
              sp = yy[j]
              
              w = 0.1
              
              # plot ranges & fossils
              if(length(range) > 1){
                rect(min(range), sp+w, max(range), sp-w, col=adjustcolor(col, alpha.f = 0.3))
                if(show.fossils)
                  points(range, rep(sp, length(range)), cex = cex, pch = pch, col = col)
                else
                  points(c(min(range), max(range)), c(sp, sp), cex = cex, pch = pch, col = col)
              } else # plot singletons
                points(range, sp, cex = cex, pch = pch, col = col)
              
            }
          }
        } else{
          for(i in unique(evol$edge)) {
            
            range = evol$r[which(evol$edge == i)]
            
            sp = yy[i]
            
            w = 0.1
            
            # plot ranges & fossils
            if(length(range) > 1){
              rect(min(range), sp+w, max(range), sp-w, col=adjustcolor(range.col, alpha.f = 0.2))
              if(show.fossils)
                points(range, rep(sp, length(range)), cex = cex, pch = pch, col = fossil.col)
              else
                points(c(min(range), max(range)), c(sp, sp), cex = cex, pch = pch, col = fossil.col)
            } else # plot singletons
              points(range, sp, cex = cex, pch = pch, col = fossil.col)
          }
        }
      }
      evol$r = NULL
    }
    
    # water depth profile
    add.depth.axis = TRUE
    if(show.proxy){
      add.depth.profile(proxy.data, axis.strata, strata, show.axis, add.depth.axis,
                        show.preferred.depth = show.preferred.environ, PD = preferred.environ, x.labs = x.labs)
    }
  }
  
  if(!is.na(old.par[1])) par(old.par)
  L <- list(type = type, use.edge.length = TRUE,
            node.pos = NULL, node.depth = node.depth, show.tip.label = show.tip.label,
            show.node.label = FALSE, font = font, cex = cex,
            adj = adj, srt = srt, no.margin = no.margin, label.offset = label.offset,
            x.lim = x.lim, y.lim = y.lim, direction = direction,
            tip.color = tip.color, Ntip = Ntip, Nnode = Nnode, root.time = tree$root.time,
            align.tip.label = align.tip.label)
  assign("last_plot.phylo", c(L, list(edge = xe, xx = xx, yy = yy)),
         envir = ape::.PlotPhyloEnv)
  invisible(L)
  
  
}

### Example Workflow

# simulate tree using treesim

# simulate a tree using ape
tips = 8
t = ape::rtree(tips)
plot(t)
# simulate fossils using fossilsim
rate = 2
f = sim.fossils.poisson(rate, t)
# plot the complete output
plot(f, t)
# plot the reconstructed output
plot(f, t, reconstructed = TRUE)



## use Marios R package
# has uncorrelated and correlated characters



# Mk. done
# V. done
# G/lognormal?   half done
# Partitioning  done
# non-applicable
# missing (percentages)
# link gamma yes but added function to amalgamate morpho objects
# ordered
# correlated characters



plot(0, 0, xlim = c(0, 10), ylim = c(0, 1), type = "n")
curve(dgamma(x, 1), from = 0, to = 10,lwd = 2 ,add = TRUE, col = "forestgreen")

breakpoints <- qgamma(seq(0, 1, length.out = 5), shape = 1)

# Add vertical lines at the breakpoints
abline(v = breakpoints, col = "skyblue", lty = 2, lwd = 2)


# Add ticks at the mean value of each part
axis(3, at = unique(test$ACRV_rate), labels = round(unique(test$ACRV_rate), 4), col = "goldenrod3",
     col.axis = "goldenrod3", las = 2)

# Annotate the means
text(unique(test$ACRV_rate), 0.02, labels = round(unique(test$ACRV_rate), 2), col = "goldenrod3", pos = 3, las=1)

#' Write morpho data to file
#'
#' @description
#' Export components of a morpho object to file. Can write trees, character
#' matrices, or fossil ages in various formats.
#'
#' @param data A morpho object
#' @param file File name
#' @param type type to write: "tree", "matrix", or "ages"
#' @param reconstructed If TRUE, write the reconstructed version. Default FALSE.
#' @param uncertainty Numeric. Age uncertainty for fossil ages. Default 0.
#'
#' @return No return value, called for its side effect of writing data to a file.
#'
#' @importFrom ape write.tree write.nexus.data
#'
#' @export
#'
#' @examples
#' data(morpho_data)
#' tmp <- tempfile(fileext = ".tre")
#' write.morpho(morpho_data, file = tmp, type = "tree")
#'
write.morpho <- function(data, file, type = "tree",
                         reconstructed = FALSE, uncertainty = 0) {

  if (!is.morpho(data)) stop("Error: data must be a morpho object")
  if (is.null(file)) stop("Error: No file name specified")

  if (type == "tree") {
    if (reconstructed) {
      write.recon.tree(data, file)
    } else {
      ape::write.tree(data$trees$EvolTree, file)
    }

  } else if (type == "matrix") {
    if (reconstructed) {
      write.recon.matrix(data, file)
    } else {
      ape::write.nexus.data(data$sequences$tips, file, format = "standard")
    }

  } else if (type == "ages") {
    if (reconstructed) {
      write.recon.tsv(data, file, uncertainty)
    } else {
      write.tsv(data, file, uncertainty)
    }

  } else {
    stop("Error: 'type' must be 'tree', 'matrix', or 'ages'")
  }
}



#' Write reconstructed tree to file
#'
#' @description
#' Write the reconstructed tree to Newick string
#'
#' @param data Morpho object
#' @param file File name
#'
#' @return
#' No return value, called for its side effect of writing data to a file.
#'
#' @examples
#' data(morpho_data)
#' tmp <- tempfile(fileext = ".tre")
#' write.recon.tree(data = morpho_data, file = tmp)
#'
#' @export
#'
write.recon.tree <- function (data = NULL, file = NULL) {

  if (is.null(data) || !inherits(data, "morpho")) {
    stop("Error: `data` must be a morpho object.")
  }

  if (is.null(file)) stop("Error: No file name specified")

  if(is.null(data$fossil)) stop ("Error: Cannot reconstruct tree as no fossil data in morpho object")

  r_tree <- FossilSim::reconstructed.tree.fossils.objects(fossils  = data$fossil,
                                                          tree = data$trees$TimeTree,
                                                          tip_order = "youngest_first")
  ape::write.tree(r_tree$tree, file = file)
}

#' Write reconstructed character matrix to file
#'
#' @description
#' Write the character matrix for the reconstructed tree to a nexus file
#'
#' @param data Morpho object
#' @param file File name
#'
#' @return
#' No return value, called for its side effect of writing data to a file.
#'
#' @examples
#' data(morpho_data)
#' tmp <- tempfile(fileext = ".nex")
#' write.recon.matrix(data = morpho_data, file = tmp)
#'
#' @export
#'
write.recon.matrix <- function (data, file = NULL) {

  if (is.null(data) || !inherits(data, "morpho")) {
    stop("Error: `data` must be a morpho object.")
  }

  if (is.null(file)) stop("Error: No file name specified")
  if(is.null(data$fossil)) stop ("Error: Cannot reconstruct tree as no fossil data in morpho object")


  mat <- reconstruct.matrix(data)
  ape::write.nexus.data(mat, file = file, format = "standard")

  }


#' Write the taxa ages
#'
#' @description
#' Writes the ages of the specimens in the true tree to a file. The tsv format used
#' here is directly compatible with RevBayes
#'
#' @param data Morpho object
#' @param file File name
#' @param uncertainty Numeric. Adds uncertainty to fossil ages in the morpho object.
#'  The ages in the object are point estimates by default; setting `uncertainty`
#'  will create an age range of ± this value (in millions of years).
#'
#' @return
#' No return value, called for its side effect of writing data to a file.
#'
#' @examples
#' data(morpho_data)
#' tmp <- tempfile(fileext = ".tsv")
#' write.tsv(data = morpho_data, file = tmp)
#'
#' @export
write.tsv <- function (data, file, uncertainty = 0) {

  if (is.null(data) || !inherits(data, "morpho")) {
    stop("Error: `data` must be a morpho object.")
  }

  if (is.null(file)) stop("Error: No file name specified")

  ## ages of full tree
  tip_depths <- ape::node.depth.edgelength(data$trees$TimeTree)[1:length(data$trees$TimeTree$tip.label)]
  tree_height <- max(ape::node.depth.edgelength(data$trees$TimeTree))
  tip_ages <-   round(abs(tree_height - tip_depths),3)
  # extant_tips <- data$trees$TimeTree$tip.label[abs(tip_depths - tree_height) < 1e-8]

  cat("taxon", "min_age", "max_age", sep = "\t", "\n", file = file)
  for ( i in 1:length(tip_ages)){
    if (tip_ages[i] == 0){
      cat(data$trees$TimeTree$tip.label[i], tip_ages[i] ,
          tip_ages[i], sep = "\t", file = file, append = T )
      cat("\n", file = file, append = TRUE)
    } else {
      cat(data$trees$TimeTree$tip.label[i], (tip_ages[i] -  uncertainty) ,
          (tip_ages[i] + uncertainty), sep = "\t", file = file, append = T )
      cat("\n", file = file, append = TRUE)
    }
  }
}

#' Write the taxa ages of reconstructed tree
#'
#' @description
#' Writes the ages of the specimen in the reconstructed tree to a file. The tsv format used
#' here is directly compatible with RevBayes
#'
#' @param data Morpho object
#' @param file File name
#' @param uncertainty Numeric. Adds uncertainty to fossil ages in the morpho object.
#'  The ages in the object are point estimates by default; setting `uncertainty`
#'  will create an age range of ± this value (in millions of years).
#'
#' @return
#' No return value, called for its side effect of writing data to a file.
#'
#' @examples
#' data(morpho_data)
#' tmp <- tempfile(fileext = ".tsv")
#' write.recon.tsv(data = morpho_data, file = tmp)
#'
#' @export

write.recon.tsv <- function (data, file, uncertainty = 0){

  if (is.null(data) || !inherits(data, "morpho")) {
    stop("Error: `data` must be a morpho object.")
  }

  if (is.null(file)) stop("Error: No file name specified")

  if(is.null(data$fossil)) stop ("Error: Cannot reconstruct tree as no fossil data in morpho object")


  r_tree <- FossilSim::reconstructed.tree.fossils.objects(fossils  = data$fossil,
                                                          tree = data$trees$TimeTree,
                                                          tip_order = "youngest_first")
  transformations <- morphsim_fossilsim(data)

  cat("taxon", "min_age", "max_age", sep = "\t", "\n", file = file)


  ## true tree tips
  tps <- unname(r_tree$tree$tip.label)
  matches <- grepl("_1$", tps )
  # Extract elements that match
  reconTreeTips <- gsub("_1$", "", tps[matches])

  ## add these tip labels to the file + plus all sampled ancestor
  seq_tips <- which(names(data$sequences$tips) %in% reconTreeTips)


  for ( i in 1:length(reconTreeTips)){
    ord <-  which(data$trees$TimeTree$tip.label ==reconTreeTips[i])
    node_pos <- ape::node.depth.edgelength(data$trees$TimeTree)[ord]
    tree_height <- max(ape::node.depth.edgelength(data$trees$TimeTree))
    tip_ages <-   round(abs(node_pos - tree_height),3)

    if(tip_ages == 0){
      nm <- paste0(reconTreeTips[i], "_1")
      cat(nm, tip_ages, tip_ages,
          sep = "\t", file = file, append = T )
      cat("\n", file = file, append = TRUE)
    } else {
      nm <- paste0(reconTreeTips[i], "_1")
      if (tip_ages - uncertainty < 0){
        min_age <- 0
      } else {
        min_age <- tip_ages - uncertainty
      }
      cat(nm,min_age, (tip_ages + uncertainty),
          sep = "\t", file = file, append = T )
      cat("\n", file = file, append = TRUE)

    }
  }

  ## sampled ancestors

  if (length(transformations[,"Morphsim"]) > 0){
    for (i in 1:length(transformations[,1])){

      parts <- as.numeric(strsplit(transformations[i, "Morphsim"], "_")[[1]])
      specimen_num <- parts[1]
      branch_num   <- parts[2]

      # Subset the data frame to get hmin
      hmin <- data$fossil$hmin[data$fossil$ape.branch == branch_num &
                                 data$fossil$specimen  == specimen_num]

      nm <- paste0(transformations[i, "Fossilsim"], "_2")
      if (hmin - uncertainty < 0){
        min_age <- 0
      } else {
        min_age <- hmin - uncertainty
      }
      cat(nm,min_age, (hmin + uncertainty),
          sep = "\t", file = file, append = T )
      cat("\n", file = file, append = TRUE)
    }
  }
}

#' Match sampled ancestor labels
#'
#' @description
#' Match the sampled ancestor labels from \code{Morphsim} and \code{Fossilsim}
#' @param data Morpho object containing fossils
#' @return
#' A character matrix mapping sampled ancestor labels between the naming
#' conventions used by \code{Morphsim} and \code{Fossilsim}
#'
#' @examples
#' data(morpho_data)
#' morphsim_fossilsim <- function(data = morpho_data)
#'
morphsim_fossilsim <- function (data = NULL){


  if(is.null(data$fossil)) stop("Error: Morpho object does not contian fossils")

  r_tree <- FossilSim::reconstructed.tree.fossils.objects(fossils  = data$fossil,
                                                          tree = data$trees$TimeTree,
                                                          tip_order =  "youngest_first")
  SA_tips <- c()
  tps <- unname(r_tree$tree$tip.label)
  matches <- grepl("_1$", tps )
  # Extract elements that match
  reconTreeTips <- gsub("_1$", "", tps[matches])

  ## add these tip labels to the file + plus all sampled ancestor
  seq_tips <- which(names(data$sequences$tips) %in% reconTreeTips)

  matches <- grepl("_2$", tps )
  reconSA <-  gsub("_2$", "", tps[matches])

  transformation <- matrix(ncol = 2, nrow = length(reconSA))
  colnames(transformation) <- c("Morphsim", "Fossilsim")
  if (length(reconSA) > 0){

    for (l in 1:length(reconSA)){
      t_label <- which(data$trees$TimeTree$tip.label == reconSA[l])
      b_num <- which(data$trees$TimeTree$edge[,2] == t_label)
      spec_min <- min(data$fossil$hmin[data$fossil$ape.branch == b_num])
      spec_num <- data$fossil$specimen[ data$fossil$hmin == spec_min ]
      SA_tips <- rbind(SA_tips, c(paste0(spec_num, "_", b_num)))

      transformation[l,"Morphsim"] <- SA_tips[l]
      transformation[l,"Fossilsim"] <- reconSA[l]
    }
  }
   return(transformation)
}

#' Simulate characters along branches in a tree
#'
#' @description
#' This function simulates discrete character data along the branches of a
#' phylogenetic tree using a continuous-time Markov chain. It implements the
#' Mk model and its extensions (Lewis 2001), including among-character rate
#' variation (ACRV) using discretized gamma distributions (Yang 1994) or
#' lognormal distributions (Wagner 2012), following the implementation
#' described in Capobianco and Hohna (2025). It can be used with either a
#' time tree or a tree with branch lengths in evolutionary distance. If using
#' a time tree, branch rates can be specified either as a single value for all
#' branches or as a vector with different rates per branch. If no branch rates
#' are specified a default of 0.1 is applied to all branches.
#'
#' @param tree A phylogenetic tree (class \code{"phylo"}) with branches
#'   representing genetic distance.
#' @param time.tree A phylogenetic tree (class \code{"phylo"}) with branches
#'   representing time.
#' @param k Integer (>= 2). Number of trait states. Can be a vector if using
#'   partitions.
#' @param trait.num Integer (> 0). The total number of traits to simulate.
#' @param partition Integer vector. Specifies the number of traits per
#'   partition. Must sum to \code{trait.num}.
#' @param br.rates Numeric. Clock rates per branch. Can be a single value
#'   (strict clock) or a numeric vector of length equal to the number of
#'   branches in the tree, one rate per branch in the same order as
#'   \code{tree$edge}. Custom vectors of rates can be provided directly here,
#'   for example rates simulated using \code{simclock::relaxed.tree()}.
#' @param root.state Integer. The root state at the start of the simulation.
#'   Must be within the range of states implied by \code{k}.
#' @param variable Logical. If \code{TRUE}, simulate only varying characters
#'   (MkV model). Default is \code{FALSE}.
#' @param full.states Logical. If \code{TRUE}, ensures that all character
#'   states specified by \code{k} are present in at least one tip for each
#'   trait. Default is \code{FALSE}.
#' @param parsimony Character. If \code{"standard"}, retain only characters
#'   where at least two states each occur in two or more taxa; autapomorphic
#'   states are tolerated. If \code{"strict"}, every observed state must occur
#'   in two or more taxa, excluding characters with any autapomorphic states
#'   entirely. If \code{NULL}, no parsimony filter is applied. Default is
#'   \code{NULL}.
#' @param ACRV Character. Among-character rate variation model. One of
#'   \code{"gamma"}, \code{"lgn"}, \code{"user"}, or \code{NULL}.
#'   \code{"gamma"} draws rates from a discretized gamma distribution (Yang
#'   1994); requires \code{ACRV.ncats} and \code{alpha.gamma}.
#'   \code{"lgn"} draws rates from a discretized lognormal distribution
#'   (Wagner 2012); requires \code{ACRV.ncats}, \code{meanlog}, and
#'   \code{sdlog}. \code{"user"} uses user-supplied rates via
#'   \code{define.ACRV.rates}. \code{NULL} simulates all traits under the
#'   same rate. Default is \code{NULL}.
#' @param alpha.gamma Numeric. Shape parameter \eqn{\alpha} for the gamma
#'   distribution. Default is 1.
#' @param ACRV.ncats Integer. Number of discrete rate categories for
#'   among-character rate variation.
#' @param meanlog Numeric. Mean of the lognormal distribution on the log
#'   scale. Required when \code{ACRV = "lgn"}.
#' @param sdlog Numeric. Standard deviation of the lognormal distribution on
#'   the log scale. Required when \code{ACRV = "lgn"}.
#' @param define.ACRV.rates Numeric vector. User-supplied rate categories.
#'   Required when \code{ACRV = "user"}.
#' @param ancestral Logical. If \code{TRUE}, return the states at all
#'   ancestral nodes. Default is \code{TRUE}.
#' @param fossil A fossil object from \code{FossilSim} to simulate
#'   morphological characters for sampled ancestors.
#' @param define.Q Numeric matrix. A custom transition rate matrix Q for
#'   simulation. Must be square and rows must sum to zero.
#'
#' @return An object of class `morpho`, with the following components:
#' \describe{
#'   \item{sequences}{A list containing up to 3 elements: morphological data for the `tips`
#'   of the tree, the `nodes`, and, if provided, the sampled ancestors (`SA`). For `SA`,
#'   the naming scheme differs from that of `FossilSim`: the morphological data are named
#'   using the specimen number (`data$fossil$specimen`) and the branch number along which
#'   the fossil was sampled.}
#'
#'   \item{tree}{A list containing up to 3 elements: the `EvolTree` (branch lengths in
#'   genetic distance), the `TimeTree` (branch lengths in time units), and `BrRates`
#'   (the evolutionary rate per branch).}
#'
#'   \item{model}{Information about the model used to simulate the data. `Specified`
#'   states the exact model used per partition, as well as the number of traits and
#'   character states respectively. `RateVar` contains the relative rates used to simulate
#'   the data, and `RateVarTrait` contains information about which rate category was used
#'   to simulate each trait. These values are listed from lowest rate (1) to highest.}
#'
#'   \item{transition_history}{A list containing *n* data frames, where *n* is the number
#'   of simulated traits. Each data frame contains information about transitions that
#'   occurred for that trait, including the branch number (`edge`), the new state number (`state`),
#'   and the point along the branch where the transition occurred (`hmin`).}
#'
#'   \item{root.states}{A vector of root states for each trait.}
#'
#'   \item{fossil}{The fossil object provided to `morphsim` from `FossilSim`. The naming
#'   scheme therefore matches that of `FossilSim`.}
#' }
#' @references
#' Lewis, P.O. (2001) A likelihood approach to estimating phylogeny from
#' discrete morphological character data. \emph{Systematic Biology} 50:913-925.
#'
#' Yang, Z. (1994) Maximum likelihood phylogenetic estimation from DNA
#' sequences with variable rates over sites: approximate methods.
#' \emph{Journal of Molecular Evolution} 39:306-314.
#'
#' Wagner, P.J. (2012) Modelling rate distributions using character
#' compatibility: implications for morphological evolution among fossil
#' invertebrates. \emph{Biology Letters} 8:143-146.
#'
#' Capobianco, A. and Hohna, S. (2025) On the MkV model with
#' among-character rate variation. \emph{Systematic Biology}.
#'
#' @importFrom ape Ntip
#'
#' @export

#' @examples
#'# simulated tree
#' phy <- ape::rtree(10)
#'
#'# simulate characters along the branches of the tree
#' morpho_data <-  sim.morpho(tree = phy,
#'                            k = c(2,3,4),
#'                            trait.num = 20,
#'                            partition = c(10,5,5),
#'                            ACRV = "gamma",
#'                            ACRV.ncats = 4,
#'                            variable = TRUE,
#'                            ancestral = TRUE,
#'                            define.Q = NULL)
#'
#' # To simulate ordered characters:
#' # First define a Q-matrix. The following is for ordered characters where transitions can only occur
#' # between states 0 and 1 and 1 and 2
#'
#' ord_Q <- matrix(c(
#' -0.5, 0.5, 0.0,
#' 0.3333333, -0.6666667, 0.3333333,
#' 0.0, 0.5, -0.5
#' ), nrow = 3, byrow = TRUE)
#'
#' # This Q matrix can be then used to simulate character data.
#'
#' morpho_data <- sim.morpho(tree = phy,
#'                           k = 3,
#'                           trait.num = 20,
#'                           ancestral = TRUE,
#'                           ACRV = "gamma",
#'                           variable = TRUE,
#'                           ACRV.ncats = 4,
#'                          define.Q = ord_Q)



sim.morpho <- function(tree = NULL,
                       time.tree = NULL,
                       k = 2,
                       trait.num,
                       partition = NULL,
                       br.rates = NULL,
                       root.state = NULL,
                       variable = FALSE,
                       full.states = FALSE,
                       parsimony = NULL,
                       ACRV = NULL,
                       alpha.gamma = 1,
                       ACRV.ncats = 4,
                       meanlog = NULL,
                       sdlog = NULL,
                       define.ACRV.rates = NULL,
                       ancestral = TRUE,
                       fossil = NULL,
                       define.Q = NULL) {

  if (is.null(tree) && is.null(time.tree)) stop("Error: Must provide a tree object")
  if (any(k < 2)) stop("Error: Need to simulate at least 2 states")
  if (is.null(trait.num)) stop("Error: Specify the total number of traits to simulate")

  if (!is.null(ACRV) && ACRV == "user" && is.null(define.ACRV.rates)) {
    stop("Error: Need to provide a vector of rates if ACRV is set to 'user'")
  }

  if (is.null(partition) && length(k) != 1) {
    stop("Error: Data being simulated under 1 partition, supply 1 character state for k")
  }

  if (!is.null(partition) && length(k) != length(partition)) {
    stop("Error: Need to specify maximum character state for each partition")
  }

  if (!is.null(partition) && sum(partition) != trait.num) {
    stop("Error: The total number characters in partitions and trait.num must match")
  }

  if (!is.null(define.Q)) {
    if (!is.matrix(define.Q)) stop("Error: `define.Q` must be a matrix.")
    if (nrow(define.Q) != ncol(define.Q)) stop("Error: `define.Q` must be square.")
    if (any(abs(rowSums(define.Q)) > 1e-6)) {
      stop("Error: Incorrect Q matrix specified: rows must sum to zero.")
    }
  }

  if (!is.logical(variable)) stop("Error: `variable` must be TRUE or FALSE.")
  if (!is.logical(ancestral)) stop("Error: `ancestral` must be TRUE or FALSE.")
  if (!is.null(fossil) && !FossilSim::is.fossils(fossil)) {
    stop("Error: `fossil` must be a FossilSim object.")
  }


  if (!is.null(root.state) && root.state > min(k) -1){
    stop("Root state specified not incluced in state matrix specified for simulation")
  }

  if (!is.null(parsimony) && !parsimony %in% c("standard", "strict")) {
    stop("Error: `parsimony` must be NULL, 'standard', or 'strict'")
  }


  ## if provided with time tree, need to transform branches in genetic distance
  if (is.null(tree) && !is.null(time.tree)) {
    tree <- time.tree
    if (is.null(br.rates)) {
      print("Warning: No branch rate provided, using default of 0.1 for all branches")
      br.rates <- rep(0.1, length(tree$edge.length))
    }
    if (length(br.rates) == 1) {
      br.rates <- rep(br.rates, length(tree$edge.length))
    }
    tree$edge.length <- time.tree$edge.length * br.rates
  }

  tree.ordered <- reorder(tree)

  if (!is.null(time.tree)) {
    time.tree.order <- reorder(time.tree)
  } else {
    time.tree.order <- NULL
  }


  # counters and setup
  tr.num <- 1
  edge <- tree.ordered$edge
  num.nodes <- max(edge)
  num.tips <- ape::Ntip(tree.ordered)
  rs <- c()

  # define the node labels
  parent <- as.integer(edge[, 1])
  child <- as.integer(edge[, 2])
  tips <- setdiff(child, parent)
  nodes <-unique(parent)

  # create an result matrix
  transition_history <- list()
  state_at_tips <- matrix(nrow = num.tips, ncol = trait.num)
  rownames(state_at_tips) <- tree.ordered$tip.label
  state_at_nodes <- matrix(nrow = tree$Nnode, ncol = trait.num)
  rownames(state_at_nodes) <- nodes
  ACRV_rate <- matrix(ncol = trait.num, nrow = 1)

  # identify the root
  root <- as.integer(parent[!match(parent, child, 0)][1])
  if (is.null(partition)) partition <- trait.num

  ### Among character rate variation
  if (!is.null(ACRV)) {
    if (ACRV == "gamma") {
      disc_rates <- get_gamma_rates(alpha.gamma, ACRV.ncats)
    } else if(ACRV == "user"){
      disc_rates <- define.ACRV.rates
      } else if (ACRV == "lgn") {
      disc_rates <- get_lognormal_rates(meanlog, sdlog, ACRV.ncats)
    }
  }

  ## start the partition loop
  for (part in 1:length(partition)) {
    part.trait.num <- partition[part]
    part_k <- k[part]

    if (is.null(define.Q)) {
      Q <- symmetric.Q.matrix(part_k)
    } else {
      Q <- define.Q
    }

    states <- as.character(c(0:(part_k - 1)))
    bl <- tree.ordered$edge.length

    # start of simulation loop
    for (tr in 1:part.trait.num) {
      iter <- 0
      repeat {
        iter <- iter + 1
        if (iter > 1e6) {
          stop("sim.morpho: character ", tr, " has not converged after 1e6 iterations. ",
                  "Consider reducing k, increasing branch the rates, or relaxing the full.states or parsimony conditions.")
          }

        if (is.null(root.state)){
        root.state <- sample(states, 1, replace = TRUE, prob = rep(1 / part_k, part_k))
        } else {
          root.state <- root.state
        }
        state_at_nodes[as.character(root), tr.num] <- as.numeric(root.state)

        if (!is.null(ACRV)) {
            trait_rate <- disc_rates[sample(1:ACRV.ncats, 1)]
            Q_r <- Q * trait_rate
             } else {
          Q_r <- Q
        }

        # container for the transitions
        transitions <- matrix(ncol = 3, nrow = 0)
        colnames(transitions) <- c("edge", "state", "hmin")

        # loop through all branches
        for (i in seq_along(bl)) {
          total_wait <- 0
          from <- parent[i]
          to <- child[i]
          current_state <- as.numeric(unname(state_at_nodes[as.character(from), tr.num]))
           # Time tracking
          time_remaining <- bl[i]

          while (time_remaining > 0) {
            # Get the rates for the current state
            rates <- Q_r[current_state + 1, ]
            rates[current_state + 1] <- 0
            # calculate the rate of any transition occurring
            total_rate <- -Q_r[current_state + 1, current_state + 1]
            waiting_time <- rexp(1, rate = total_rate)
            time_remaining <- time_remaining - waiting_time
            if (time_remaining <= 0) break

            # Transition to a new state
            next_state <- sample(states, 1, prob = rates / total_rate)
            current_state <- as.numeric(next_state)
            add_t <- c(i, next_state, waiting_time + total_wait)
            transitions <- rbind(transitions, as.numeric(add_t))
            total_wait <- total_wait + waiting_time
          }

          if (to %in% tips) state_at_tips[tree$tip.label[to], tr.num] <- current_state
          if (to %in% nodes) state_at_nodes[as.character(to), tr.num] <- current_state
        }

        # filtering conditions
        if (!variable && !full.states && is.null(parsimony)) break
        vari <- !variable || (length(unique(state_at_tips[, tr.num])) > 1 & length(unique(state_at_nodes[, tr.num])) > 1)
        has_max_state <- !full.states || all(states %in% state_at_tips[, tr.num])
        pars <- if (is.null(parsimony)) {
          TRUE
        } else if (parsimony == "standard") {
          # Literal parsimony informative from paup, TNT
          # At least two states each appear in >= 2 taxa (allows autapomorphies)
          sum(table(state_at_tips[, tr.num]) >= 2) >= 2
        } else if (parsimony == "strict") {
          # Strictly parsimony-informative: no autapomorphies permitted.
          # Every observed state must occur in >= 2 taxa.
          all(table(state_at_tips[, tr.num]) >= 2)
        }
        if (vari && has_max_state && pars) break
          } # end of simulation loop

      if (!is.null(ACRV)) {
         ACRV_rate[tr.num] <- which(disc_rates == trait_rate)
      }

      transition_history[[tr.num]] <- as.data.frame(transitions)
      tr.num <- tr.num + 1
      rs <- rbind(rs, root.state)
    }
  }

  ### fossil object
  if (!is.null(fossil)) {
    f.morpho <- fossil
    f.morpho$ape.branch <- NA
    f.morpho$specimen <- seq(1, length(f.morpho$sp))

    state_at_fossils <- matrix(nrow = length(f.morpho$sp), ncol = trait.num)
    rownames(state_at_fossils) <- f.morpho$specimen

    for (i in 1:length(f.morpho$edge)) {
      child <- f.morpho$edge[i]
      f.morpho$ape.branch[i] <- which(tree.ordered$edge[, 2] == child)
    }

    tree.age <- max(ape::node.depth.edgelength(time.tree))

    for (tr.num in 1:trait.num) {
      for (spec in 1:length(f.morpho$specimen)) {
        branch <- f.morpho$ape.branch[[spec]]
        parent <- tree.ordered$edge[branch, 1]
        current_state <- state_at_nodes[[which(rownames(state_at_nodes) == parent), tr.num]]

        if (!(f.morpho$ape.branch[spec] %in% transition_history[[tr.num]][[1]])) {
          state_at_fossils[spec, tr.num] <- current_state
        } else {
          time_position_fossil <- tree.age - f.morpho$hmin[spec] + time.tree$root.edge
          changes <- which(transition_history[[tr.num]][[1]] == f.morpho$ape.branch[spec])
          changes_along_edge <- transition_history[[tr.num]][changes, ]
          node_age <- ape::node.depth.edgelength(time.tree)[time.tree$edge[branch, 1]]
          changes_along_edge$time <- (changes_along_edge[, 3] / br.rates[branch]) + node_age + time.tree$root.edge

          later <- which(changes_along_edge[, 4] > time_position_fossil)
          if (!length(later) == 0) changes_along_edge <- changes_along_edge[-later, ]

          if (length(changes_along_edge[, 1]) == 0) {
            state_at_fossils[spec, tr.num] <- current_state
          } else {
            tran <- which(changes_along_edge$hmin == max(changes_along_edge$hmin))
            state_at_fossils[spec, tr.num] <- changes_along_edge[tran, 2]
          }
        }
      }
    }

    fossil_names <- paste0(f.morpho$specimen, "_", f.morpho$ape.branch)
    fossil_sequence <- list()
    rownames(state_at_fossils) <- fossil_names
    for (i in 1:length(fossil_names)) {
      fossil_sequence[[fossil_names[i]]] <- state_at_fossils[fossil_names[i], ]
    }
  }

  ## create morpho object
  seq <- list(NA, NA, NA)
  names(seq) <- c("tips", "nodes", "SA")
  tip_names <- rownames(state_at_tips)
  tip_sequence <- list()

  for (i in 1:length(tip_names)) {
    tip_sequence[[tip_names[i]]] <- state_at_tips[tip_names[i], ]
  }
  seq[["tips"]] <- tip_sequence

  if (ancestral) {
    node_names <- rownames(state_at_nodes)
    node_sequence <- list()
    for (i in 1:length(node_names)) {
      node_sequence[[node_names[i]]] <- state_at_nodes[node_names[i], ]
    }
  } else {
    node_sequence <- NULL
  }
  seq[["nodes"]] <- node_sequence

  if (is.null(fossil)) {
    fossil_sequence <- NULL
    f.morpho <- NULL
  }
  seq[["SA"]] <- fossil_sequence

  if (is.null(ACRV)) ACRV_rate <- NULL


  if (exists("disc_rates")) {
    discrete_rates <- disc_rates
  } else {
    discrete_rates <- NULL
  }

  if (isTRUE(variable)) {
    var <- "V"
  } else {
    var <- NULL
  }
  model_components <- paste0("Mk", ACRV, var, "_Part:", partition, "states:", k)

  trees <- list(NA, NA, NA)
  names(trees) <- c("EvolTree", "TimeTree", "BrRates")
  trees[["EvolTree"]] <- tree.ordered
  trees[["TimeTree"]] <- time.tree
  trees[["BrRates"]] <- br.rates

  model <- list(NA, NA, NA)
  names(model) <- c("Specified", "RateVar", "RateVarTrait")
  model[["Specified"]] <- model_components
  model[["RateVar"]] <- discrete_rates
  model[["RateVarTrait"]] <- ACRV_rate

  sim.output <- as.morpho(sequences = seq,
                          trees = trees,
                          model = model,
                          transition_history = transition_history,
                          root.states = rs,
                          fossil = f.morpho)

  return(sim.output)
}

#' Calculates statistics for a morpho object
#'
#' @description
#' Computes three key pieces of information for a morpho object:
#' 1. The Consistency Index (CI) and Retention Index (RI) based on the tip sequence data.
#' 2. Convergent traits, identifying traits that have evolved independently multiple times.
#' 3. Summary information about the size and structure of the tree.
#'
#' @param data A morpho object
#' @return A list with three elements:
#'   - \code{Statistics}: data.frame with CI and RI
#'   - \code{Convergent_Traits}: data.frame listing convergent traits
#'   - \code{Tree}: data.frame summarizing extant/extinct tips and sampled ancestors
#' @importFrom phangorn phyDat CI RI
#' @importFrom ape node.depth.edgelength
#'
#' @export
#'
#' @examples
#' data(morpho_data)
#'
#' summary <- stats.morpho(data = morpho_data)
#'
#' @seealso \code{\link{get.convergent}} for detailed per-trait
#' convergence information
stats.morpho <- function(data){

  if (is.null(data) || !inherits(data, "morpho")) {
    stop("Error: `data` must be a morpho object.")
  }

  morpho_summary <- vector("list", 3)
  names(morpho_summary) <- c("Statistics", "Convergent_Traits", "Tree")

  ## consistency index & retention index
  phylev <- unique(unlist(unique(data$sequences$tips)))
  tree <- data$trees$EvolTree
  data_ph <-  phangorn::phyDat(data$sequences$tips, type="USER", levels=phylev, return.index = TRUE)

  morpho_summary[["Statistics"]] <- data.frame(
    CI =  phangorn::CI(tree, data_ph),
    RI = phangorn::RI(tree, data_ph)
  )

  ## convergent characters
  morpho_summary[["Convergent_Traits"]] <- convergent_evol(data = data)

  ## number of sampled ancestors
  if (!is.null(data$sequences$SA)){
    num_sa <- length(data$sequences$SA)
  } else {
    num_sa <- NA
  }

  ## number of extant tips
  if(!is.null(data$trees$TimeTree)){
    tip_depths <- ape::node.depth.edgelength(data$trees$TimeTree)[1:length(data$trees$TimeTree$tip.label)]
    tree_height <- max(ape::node.depth.edgelength(data$trees$TimeTree))
    extant_tips <- length(data$trees$TimeTree$tip.label[abs(tip_depths - tree_height) < 1e-8])

    ## number extinct tips
    extinct_tips <- length(data$trees$TimeTree$tip.label) - extant_tips

  } else {
    extant_tips <- NA
    extinct_tips <- NA
  }

  morpho_summary[["Tree"]] <- data.frame(
    Extant =  extant_tips,
    Extinct = extinct_tips,
    Sampled_Ancestors = num_sa
  )

  return(morpho_summary)
}


#' Determines the number of convergently evolved traits
#'
#' @description
#' Identifies which traits have evolved independently multiple times
#' (convergent evolution) in a morpho object.
#'
#' @param data A morpho object
#' @return A data.frame listing convergent traits, their state, and number of transitions
#'
#' @examples
#' phy <- ape::rtree(10)
#' morpho_data <- sim.morpho(tree = phy, k = 2, trait.num = 20)
#' convergent_evol(morpho_data)
#'
#' @export
convergent_evol <- function(data = NULL) {

  if (is.null(data) || !inherits(data, "morpho")) {
    stop("Error: `data` must be a morpho object.")
  }

  dat <- data[["transition_history"]]
  tree <- data$trees$EvolTree
  tip_states <- as.data.frame(data$sequences$tips)
  ntax <- length(tree$tip.label)

  convergent_traits <- matrix(ncol = 3)
  colnames(convergent_traits) <- c("trait", "state", "num.transitions")

  for (convT in seq_along(dat)) {
    temp <- dat[[convT]]
    trait_n <- tip_states[convT, ]

    # Skip if there are no transitions
    if (length(temp$edge) == 0) next

    # Prepare transition matrix
    trans_trait <- matrix(nrow = ntax, ncol = 2)
    rownames(trans_trait) <- tree$tip.label
    colnames(trans_trait) <- c("transition", "state")

    # Number transitions
    temp$num <- seq_along(temp$edge)

    for (tip in seq_along(tree$tip.label)) {
      route_n <- find_path_to_tip(tree, tree$tip.label[tip])

      for (l in seq(nrow(route_n), 1)) {
        bran <- which(tree$edge[, 1] == route_n[l, "parent"] &
                        tree$edge[, 2] == route_n[l, "child"])

        if (bran %in% temp$edge) {
          temp_sub <- temp[temp[["edge"]] == bran, ]
          max_tran <- max(temp_sub$hmin)

          trans_trait[tree$tip.label[tip], "transition"] <-
            temp_sub$num[temp_sub$hmin == max_tran]

          trans_trait[tree$tip.label[tip], "state"] <-
            temp_sub$state[temp_sub$hmin == max_tran]

          break
        }
      }
    }

    # Fill unchanged tips with root state
    rows_na <- apply(trans_trait, 1, function(row) any(is.na(row)))
    if (any(rows_na)) {
      trans_trait[rows_na, ] <- matrix(
        rep(c(0, data$root.state[convT]), sum(rows_na)),
        nrow = sum(rows_na),
        byrow = TRUE
      )
    }

    # Identify convergent traits
    states <- unique(trans_trait[, "state"])
    for (s in states) {
      by_state <- trans_trait[trans_trait[, "state"] == s, "transition"]
      if (length(unique(by_state)) > 1) {
        n <- as.numeric(c(convT, s, length(unique(by_state))))
        convergent_traits <- rbind(convergent_traits, n)
      }
    }
  }

  # Clean up output
  convergent_traits <- convergent_traits[-1, , drop = FALSE]
  if (nrow(convergent_traits) > 0) {
    convergent_traits <- as.data.frame(convergent_traits)
    rownames(convergent_traits) <- seq_len(nrow(convergent_traits))
  }

  return(convergent_traits)
}


#' Determines the route (nodes and branches) for a tip in a phylogenetic tree
#'
#' @description
#' Traverses the tree to determine the evolutionary path (branches) from root to a given tip.
#'
#' @param tree A phylogenetic tree of class \code{phylo}
#' @param tip Tip label (character)
#' @return A matrix with columns \code{parent} and \code{child} representing the path
#' @importFrom ape Ntip
find_path_to_tip <- function(tree, tip) {
  # Ensure the tip is valid
  if (!tip %in% tree$tip.label) {
    stop("The specified tip is not found in the tree.")
  }

  # Get the edge matrix and root node
  edge <- tree$edge
  root <- ape::Ntip(tree) + 1

  # the node number of the tip
  tip_node <- which(tree$tip.label == tip)

  # Start from the tip node and move upward toward the root
  path <- tip_node
  current_node <- tip_node

  # loop until reaching the root node
  while (current_node != root) {
    # find the parent of the current node
    parent_node <- edge[edge[, 2] == current_node, 1]
    path <- c(parent_node, path)
    current_node <- parent_node
  }
  length(path)-1
  output <- matrix(nrow =length(path)-1, ncol=2)
  colnames(output) <- c("parent", "child")

  for (i in 1:length(path)-1){
    output[i,] <- c(path[i], path[i+1])
  }

  return(output)
}


#' Combine Two Morpho Objects
#'
#' This function merges two `morpho` objects, combining their sequences,
#' model parameters, and transition histories, while ensuring tree and fossil
#' consistency.
#'
#' @param x A `morpho` object.
#' @param y A `morpho` object.
#'
#' @return A combined `morpho` object.
#'
#' @importFrom phangorn RF.dist
#' @export
#'
#' @examples
#' phy <- ape::rtree(10)
#'
#' # simulate characters along the branches of the tree
#' morpho1 <-  sim.morpho(tree = phy,
#'                            k = c(2,3,4),
#'                            trait.num = 20,
#'                            ancestral = TRUE,
#'                            partition = c(10,5,5),
#'                            ACRV = "gamma",
#'                            variable = TRUE,
#'                            ACRV.ncats = 4,
#'                            define.Q = NULL)
#'
#' morpho2 <-  sim.morpho(tree = phy,
#'                            k = c(2,3,4),
#'                            trait.num = 20,
#'                            ancestral = TRUE,
#'                            partition = c(10,5,5),
#'                            ACRV = "gamma",
#'                            variable = TRUE,
#'                            ACRV.ncats = 4,
#'                            define.Q = NULL)
#'
#' combined <- combine.morpho(morpho1, morpho2)
combine.morpho <- function(x, y) {

  if (!inherits(x, "morpho")) stop("Error: x must be a 'morpho' object")
  if (!inherits(y, "morpho")) stop("Error: y must be a 'morpho' object")
  if (!identical(x$fossil, y$fossil)) stop("Error: morpho objects have different fossil objects")
  if (!phangorn::RF.dist(x$trees$EvolTree,
                         y$trees$EvolTree) == 0) stop("Error: morpho objects have different trees")
  if (!identical(x$trees$BrRates, y$trees$BrRate)) stop("Error: morpho objects have different branch lengths")

  combined_tips <- list()
  tip_names <- names(x$sequences$tips)
  for (i in seq_along(tip_names)) {
    combined_tips[[tip_names[i]]] <- c(unname(x$sequences$tips[[i]]),
                                       unname(y$sequences$tips[[i]]))
  }

  combined_nodes <- list()
  nodes_names <- names(x$sequences$nodes)
  for (i in seq_along(nodes_names)) {
    combined_nodes[[nodes_names[i]]] <- c(unname(x$sequences$nodes[[i]]),
                                          unname(y$sequences$nodes[[i]]))
  }

  combined_SA <- list()
  sa_names <- names(x$sequences$SA)
  for (i in seq_along(sa_names)) {
    combined_SA[[sa_names[i]]] <- c(unname(x$sequences$SA[[i]]),
                                    unname(y$sequences$SA[[i]]))
  }

  out <- list(
    sequences = list(
      tips  = combined_tips,
      nodes = combined_nodes,
      SA    = combined_SA
    ),
    trees = list(
      EvolTree = x$trees$EvolTree,
      TimeTree = x$trees$TimeTree,
      BrRates  = x$trees$BrRates
    ),
    model = list(
      Specified    = c(x$model$Specified, y$model$Specified),
      RateVar      = rbind(x$model$RateVar, y$model$RateVar),
      RateVarTrait = rbind(x$model$RateVarTrait, y$model$RateVarTrait)
    ),
    transition_history = c(x$transition_history, y$transition_history),
    root.states        = c(x$root.states, y$root.states),
    fossil             = x$fossil,
    combined = TRUE
  )

  class(out) <- class(x)
  return(out)
}


#' Add reconstructed tree and matrix to morpho object
#'
#' @description This function should be called if you want to access the
#' reconstructed tree and character matrix directly from the morpho object.
#' It adds the reconstructed tree in Newick format ($trees$Recon), containing
#' only the sampled lineages, and a character matrix ($sequences$recon)
#' containing only the taxa present in that reconstructed tree.
#'
#' @param data `morpho object` containing a fossil object
#'
#' @return a `morpho object`
#'
#' @export
#' @examples
#' # simulate tree
#' lambda = 0.1
#' mu = 0.05
#' tips = 10
#' t = TreeSim::sim.bd.taxa(n = tips, numbsim = 1, lambda = lambda, mu = mu)[[1]]
#'
#' # Simulate fossils and extant taxa
#' rate = 0.1 # poisson sampling rate
#' f = FossilSim::sim.fossils.poisson(rate = rate, tree = t, root.edge = FALSE)
#' rho = 0.5
#' f2 = FossilSim::sim.extant.samples(fossils = f, tree = t, rho = rho)

#' morpho_data <-  sim.morpho(k = c(2,3),
#'                            time.tree = t,
#'                            trait.num = 6,
#'                            ancestral = TRUE,
#'                            br.rates = 0.1,
#'                            partition = c(4,2),
#'                            ACRV = "gamma",
#'                            variable = TRUE,
#'                            ACRV.ncats = 4,
#'                            fossil = f2)
#'
#' re <- get.reconstructed(morpho_data)
#'
get.reconstructed <- function(data) {
  if (!is.morpho(data)) stop ("Error: must provide a morpho object")
  if (is.null(data$fossil)) stop ("Error: must provide a morpho object with fossil information")

  r_tree <- FossilSim::reconstructed.tree.fossils.objects(fossils  = data$fossil,
                                                        tree = data$trees$TimeTree,
                                                        tip_order = "youngest_first")
  recon <- reconstruct.matrix(data)

  data$sequences[["recon"]] <- recon
  data$trees[["Recon"]] <- r_tree

  return(data)
}


#' Identify convergent traits
#'
#' @description
#' Identifies convergent evolution in a morpho object. When called without a
#' trait, returns a summary of which traits are convergent. When called with a
#' specific trait, returns tip groupings by state and evolutionary origin.
#'
#' @param data A morpho object
#' @param trait The trait number to examine. If \code{NULL} (default), returns a summary.
#'
#' @return A data frame. Without \code{trait}: trait index and number of convergent states.
#'   With \code{trait}: state, origin, tips, and whether the state is convergent.
#' @export
#'
#' @examples
#' phy <- ape::rtree(10)
#' morpho_data <- sim.morpho(tree = phy, k = 2, trait.num = 20)
#' get.convergent(morpho_data)
#' get.convergent(morpho_data, trait = 3)
#'

get.convergent <- function(data, trait = NULL) {
  if (!is.morpho(data)) stop("Error: data must be a morpho object")

  # Summary mode: return which traits are convergent
  if (is.null(trait)) {
    conv <- convergent_evol(data)
    if (nrow(conv) == 0) return(data.frame(trait = integer(0),
                                           num.convergent.states = integer(0)))
    summary_df <- aggregate(state ~ trait, data = conv, FUN = length)
    colnames(summary_df) <- c("trait", "num.convergent.states")
    return(summary_df)
  }

  # Detail mode: group tips by state and origin for a specific trait
  tree <- data$trees$EvolTree
  tip_states <- sapply(data$sequences$tips, function(x) x[trait])
  trans <- data$transition_history[[trait]]
  root_state <- data$root.states[trait]

  origin_labels <- character(length(tip_states))
  names(origin_labels) <- names(tip_states)

  for (tip in names(tip_states)) {
    path <- find_path_to_tip(tree, tip)
    origin <- "root"

    for (l in seq(nrow(path), 1)) {
      bran <- which(tree$edge[, 1] == path[l, "parent"] &
                      tree$edge[, 2] == path[l, "child"])
      if (nrow(trans) > 0 && bran %in% trans$edge) {
        matches <- trans[trans$edge == bran, ]
        last_change <- matches[which.max(matches$hmin), ]
        origin <- paste0("edge_", last_change$edge, "_t_", round(last_change$hmin, 4))
        break
      }
    }
    origin_labels[tip] <- origin
  }

  result <- data.frame(
    tip = names(tip_states),
    state = tip_states,
    origin = origin_labels,
    row.names = NULL
  )

  summary <- aggregate(tip ~ state + origin, data = result,
                       FUN = function(x) paste(x, collapse = ", "))

  # flag convergent states
  origins_per_state <- table(summary$state)
  convergent_states <- names(origins_per_state[origins_per_state > 1])

  summary$convergent <- summary$state %in% convergent_states
  summary <- summary[order(summary$state, summary$origin), ]

  return(summary)
}


#' Get Tranisition History
#'
#' @description
#' Returns the full transition history for a given trait,
#' including the root state.
#'
#' @param data A morpho object
#' @param trait The trait number .
#'
#' @return A data frame with columns edge (branch number), state, and
#' hmin (where along the branch the transition occurred)
#'
#' @export
#' @examples
#' phy <- ape::rtree(10)
#' morpho_data <- sim.morpho(tree = phy, k = 2, trait.num = 20)
#' get.transitions(morpho_data, trait = 2)
#'
get.transitions <- function(data, trait) {
  if (!is.morpho(data)) stop("Error: data must be a morpho object")
  if (trait > length(data$transition_history)) {
    stop(paste0("Error: trait ", trait, " is out of range. Object has ",
                length(data$transition_history), " traits."))
  }

  root_row <- data.frame(edge = 0, state = as.numeric(data$root.states[trait]), hmin = 0)
  tr <- data$transition_history[[trait]]

  rbind(root_row, tr)
}


#' Get Specific Morphological Matrix
#'
#' @description
#' Return data frame of morphological matrix
#'
#' @param data A morpho object
#' @param seq The sequence data you want to extract
#'
#' @return A data frame with columns edge (branch number), state, and
#' hmin (where along the branch the transition occurred)
#'
#' @export
#' @examples
#' phy <- ape::rtree(10)
#' morpho_data <- sim.morpho(tree = phy, k = 2, trait.num = 20)
#' get.matrix(morpho_data, seq = "tips")

get.matrix <- function(data, seq) {
  if (!is.morpho(data)) stop("Error: data must be a morpho object")
  if (!seq  %in% names(data$sequences)) {
    stop(paste0(seq, " not found in morpho object"))
  }

  s <- t(as.data.frame(data$sequences[[seq]]))
  return(s)

}

library(testthat)
library(MorphSim)
library(ape)

# ============================================================
# Shared setup
# ============================================================

set.seed(42)
tree <- ape::rtree(6)
time.tree <- TreeSim::sim.bd.taxa(n = 6, numbsim = 1, lambda = 0.1, mu = 0.05)[[1]]

# ============================================================
# sim.morpho
# ============================================================

test_that("sim.morpho returns a morpho object with correct structure", {
  data <- sim.morpho(tree = tree, k = 2, trait.num = 5)
  expect_true(is.morpho(data))
  expect_equal(length(data$sequences$tips[[1]]), 5)
  expect_setequal(names(data$sequences$tips), tree$tip.label)
  expect_equal(length(data$transition_history), 5)
  expect_equal(length(data$root.states), 5)
})

test_that("sim.morpho works with time tree and clock models", {
  # strict clock
  data <- sim.morpho(time.tree = time.tree, k = 2, trait.num = 5, br.rates = 0.1)
  expect_true(is.morpho(data))
  expect_s3_class(data$trees$TimeTree, "phylo")

  # relaxed clock
  br_rates <- runif(nrow(time.tree$edge), min = 0.01, max = 0.2)
  data2 <- sim.morpho(time.tree = time.tree, k = 2, trait.num = 5, br.rates = br_rates)
  expect_true(is.morpho(data2))
})

test_that("sim.morpho works with partitions", {
  data <- sim.morpho(tree = tree, k = c(2, 3), trait.num = 10, partition = c(5, 5))
  expect_true(is.morpho(data))
  expect_equal(length(data$sequences$tips[[1]]), 10)
  expect_equal(length(data$model$Specified), 2)
})

test_that("sim.morpho works with MkV and ACRV models", {
  data <- sim.morpho(tree = tree, k = 3, trait.num = 10,
                     variable = TRUE, ACRV = "gamma", alpha.gamma = 1, ACRV.ncats = 4)
  expect_true(is.morpho(data))
  expect_false(is.null(data$model$RateVar))

  # MkV: no invariant characters
  tip_matrix <- t(as.data.frame(data$sequences$tips))
  for (i in seq_len(ncol(tip_matrix))) {
    expect_gt(length(unique(tip_matrix[, i])), 1)
  }
})

test_that("sim.morpho works with custom Q matrix", {
  ord_Q <- matrix(c(-0.5, 0.5, 0.0,
                    0.3333333, -0.6666667, 0.3333333,
                    0.0, 0.5, -0.5), nrow = 3, byrow = TRUE)
  data <- sim.morpho(tree = tree, k = 3, trait.num = 5, define.Q = ord_Q)
  expect_true(is.morpho(data))
})

test_that("sim.morpho validates inputs", {
  expect_error(sim.morpho(k = 2, trait.num = 5), "Must provide a tree")
  expect_error(sim.morpho(tree = tree, k = 1, trait.num = 5), "at least 2 states")
  expect_error(sim.morpho(tree = tree, k = c(2, 3), trait.num = 10), "1 partition")
})

# ============================================================
# FossilSim integration
# ============================================================

test_that("sim.morpho generates data for tips and sampled ancestors", {
  set.seed(42)
  f <- FossilSim::sim.fossils.poisson(rate = 0.5, tree = time.tree, root.edge = FALSE)
  f2 <- FossilSim::sim.extant.samples(fossils = f, tree = time.tree, rho = 1)

  data <- sim.morpho(time.tree = time.tree, k = 2, trait.num = 5,
                     br.rates = 0.1, fossil = f2)
  expect_true(is.morpho(data))
  expect_false(is.null(data$fossil))
  if (nrow(f) > 0) {
    expect_gt(length(data$sequences$SA), 0)
    expect_equal(length(data$sequences$SA[[1]]), 5)
  }
})


test_that("get.reconstructed adds recon tree and matrix to morpho object", {
  set.seed(42)
  f <- FossilSim::sim.fossils.poisson(rate = 0.5, tree = time.tree, root.edge = FALSE)
  f2 <- FossilSim::sim.extant.samples(fossils = f, tree = time.tree, rho = 1)
  data <- sim.morpho(time.tree = time.tree, k = 2, trait.num = 5,
                     br.rates = 0.1, fossil = f2)
  recon <- get.reconstructed(data)
  expect_true(is.morpho(recon))
  expect_false(is.null(recon$sequences$recon))
  expect_false(is.null(recon$trees$Recon))
  expect_s3_class(recon$trees$Recon$tree, "phylo")
  expect_true(all(names(recon$sequences$recon) %in% recon$trees$Recon$tree$tip.label))
})

# ============================================================
# sim.missing.data
# ============================================================

test_that("sim.missing.data introduces and controls missing data", {
  data <- sim.morpho(tree = tree, k = 2, trait.num = 10)

  # probability 0 = no missing
  m0 <- sim.missing.data(data = data, method = "random", seq = "tips", probability = 0)
  expect_false(any(t(as.data.frame(m0$sequences$tips)) == "?"))

  # probability 1 = all missing
  m1 <- sim.missing.data(data = data, method = "random", seq = "tips", probability = 1)
  expect_true(all(t(as.data.frame(m1$sequences$tips)) == "?"))

  # intermediate probability
  m5 <- sim.missing.data(data = data, method = "random", seq = "tips", probability = 0.5)
  expect_true(is.morpho(m5))
  expect_true(any(t(as.data.frame(m5$sequences$tips)) == "?"))
})

test_that("sim.missing.data works with structured methods", {
  # trait method
  data <- sim.morpho(tree = tree, k = 2, trait.num = 10)
  missing <- sim.missing.data(data = data, method = "trait",
                              seq = "tips", probability = 1, traits = c(1, 2))
  tip_matrix <- t(as.data.frame(missing$sequences$tips))
  expect_true(all(tip_matrix[, 1] == "?"))
  expect_true(all(tip_matrix[, 2] == "?"))
})

test_that("sim.missing.data works with trait_taxa method", {
  data <- sim.morpho(tree = tree, k = 2, trait.num = 10)
  taxa_to_remove <- names(data$sequences$tips)[1:2]
  missing <- sim.missing.data(data = data, method = "trait_taxa",
                              seq = "tips", probability = 1,
                              traits = c(1, 2), taxa = taxa_to_remove)
  tip_matrix <- t(as.data.frame(missing$sequences$tips))
  # only the specified taxa and traits should be missing
  expect_true(all(tip_matrix[taxa_to_remove, 1:2] == "?"))
  # other taxa should be unaffected
  other_taxa <- setdiff(rownames(tip_matrix), taxa_to_remove)
  expect_false(any(tip_matrix[other_taxa, 1:2] == "?"))
})


test_that("sim.missing.data rate method removes data in correct order", {
  data <- sim.morpho(tree = tree, k = 2, trait.num = 20,
                     ACRV = "gamma", alpha.gamma = 1, ACRV.ncats = 4)
  # probability 0 for slowest category, 1 for fastest
  missing <- sim.missing.data(data = data, method = "rate",
                              seq = "tips", probability = c(0, 0, 0, 1))
  tip_matrix <- t(as.data.frame(missing$sequences$tips))
  sorted_cats <- sort(unique(data$model$RateVarTrait[1, ]))
  fastest_traits <- which(data$model$RateVarTrait[1, ] == sorted_cats[4])
  slowest_traits <- which(data$model$RateVarTrait[1, ] == sorted_cats[1])
  # fastest traits should all be missing
  expect_true(all(tip_matrix[, fastest_traits] == "?"))
  # slowest traits should not be missing
  expect_false(any(tip_matrix[, slowest_traits] == "?"))
})
# ============================================================
# stats.morpho
# ============================================================

test_that("stats.morpho returns CI, RI, and convergence info", {
  set.seed(42)
  data <- sim.morpho(tree = tree, k = 2, trait.num = 20)
  stats <- stats.morpho(data)

  expect_true(is.list(stats))
  expect_true(stats$Statistics$CI >= 0 & stats$Statistics$CI <= 1)
  expect_true(stats$Statistics$RI >= 0 & stats$Statistics$RI <= 1)
  expect_true("trait" %in% colnames(stats$Convergent_Traits))
})

# ============================================================
# combine.morpho
# ============================================================

test_that("combine.morpho merges two morpho objects", {
  data1 <- sim.morpho(tree = tree, k = 2, trait.num = 5)
  data2 <- sim.morpho(tree = tree, k = 3, trait.num = 5)
  combined <- combine.morpho(data1, data2)
  expect_true(is.morpho(combined))
  expect_equal(length(combined$sequences$tips[[1]]), 10)
  expect_equal(length(combined$transition_history), 10)
})

# ============================================================
# Write / export functions
# ============================================================

test_that("export functions write valid files", {
  set.seed(42)
  f <- FossilSim::sim.fossils.poisson(rate = 0.5, tree = time.tree, root.edge = FALSE)
  f2 <- FossilSim::sim.extant.samples(fossils = f, tree = time.tree, rho = 1)
  data <- sim.morpho(time.tree = time.tree, k = 2, trait.num = 5,
                     br.rates = 0.1, fossil = f2)

  # full tree
  tmp_tree <- tempfile(fileext = ".nwk")
  write.morpho(data, file = tmp_tree, type = "tree")
  expect_s3_class(ape::read.tree(tmp_tree), "phylo")

  # reconstructed tree
  tmp_recon_tree <- tempfile(fileext = ".nwk")
  write.morpho(data, file = tmp_recon_tree, type = "tree", reconstructed = TRUE)
  expect_s3_class(ape::read.tree(tmp_recon_tree), "phylo")

  # character matrix
  tmp_nex <- tempfile(fileext = ".nex")
  write.morpho(data, file = tmp_nex, type = "matrix")
  expect_gt(file.info(tmp_nex)$size, 0)

  # reconstructed matrix
  tmp_recon_nex <- tempfile(fileext = ".nex")
  write.morpho(data, file = tmp_recon_nex, type = "matrix", reconstructed = TRUE)
  expect_gt(file.info(tmp_recon_nex)$size, 0)

  # age file
  tmp_tsv <- tempfile(fileext = ".tsv")
  write.morpho(data, file = tmp_tsv, type = "ages")
  ages <- read.delim(tmp_tsv, sep = "\t")
  expect_true(all(c("taxon", "min_age", "max_age") %in% names(ages)))

  unlink(c(tmp_tree, tmp_recon_tree, tmp_nex, tmp_recon_nex, tmp_tsv))
})

# ============================================================
# Reproducibility
# ============================================================

test_that("sim.morpho is deterministic with same seed", {
  set.seed(123)
  data1 <- sim.morpho(tree = tree, k = 2, trait.num = 5)
  set.seed(123)
  data2 <- sim.morpho(tree = tree, k = 2, trait.num = 5)
  expect_identical(data1$sequences$tips, data2$sequences$tips)
})

#' Example morpho dataset
#'
#' Small example dataset for testing morpho functions.
#'
#' @format A list with components:
#' \describe{
#'   \item{sequences}{List of sequences for each taxon}
#'   \item{trees}{List containing the evolutionary tree}
#' }
#' @usage data(morpho_data)
"morpho_data"

#' Plots morphological matrix
#'
#' @description
#' This function plots the full morphological matrix assocaited with the character data
#' at the tips of a tree. Requires a morpho object as input.
#' @param data A morpho object
#' @param seq the sequence data to plot: "tips", "nodes", "SA", or "recon"
#' @param num.trait default is set to "all" which plots all traits in black font. If you
#' want to focus on a specific trait set it here, e.g. num.trait = 1 and this trait will
#' be highlighted
#' @param col A vector of colors that should be the same length or longer than the number of different character states (k). if not specified, the traits from 0 to 6 can be differentiated
#' @param timetree  TRUE or FALSE Indicate whether you want to plot a time tree or not. default FALSE, uses distance tree if FALSE
#'
#' @return
#' No return value, called for its side effect of producing a plot.
#'
#' @importFrom ape write.tree
#'
#' @export
#'
#' @examples
#' data(morpho_data)
#' # plot the character matrix
#' plotMorphoGrid(data = morpho_data, seq = "tips", num.trait = "all")


plotMorphoGrid <- function(data = NULL,
                           timetree = FALSE,
                           seq = "tips",
                           num.trait = "all",
                           col =  c("lavender", "white", "lightskyblue1", "pink", "gold2", "forestgreen", "coral")){


  if (is.null(data)) stop("Error: 'data' cannot be NULL.")

  if (!is.morpho(data)) stop("Error: 'data' must be a 'morpho'.")

  if (!seq %in% c("tips", "recon")) {
    stop("Error: 'seq' must be either 'tips' or 'recon'.")
  }

  if (!("transition_history" %in% names(data)) || length(data$transition_history) == 0) {
    stop("Error: 'data' must contain a non-empty 'transition_history' element.")
  }

  n.traits.total <- length(data$transition_history)

  if (!identical(num.trait, "all")) {
    if (!is.numeric(num.trait) || length(num.trait) != 1 || num.trait < 1 || num.trait > n.traits.total) {
      stop(paste0("Error: 'num.trait' must be 'all' or a numeric value between 1 and ", n.traits.total, "."))
    }
  }





   old_par <- par(no.readonly = TRUE)
  on.exit(par(old_par))

  if (inherits(data$sequences[[seq]][[1]][1], "character")) {
    data$sequences[[seq]] <- lapply(data$sequences[[seq]], function(x) {
      x[x == "?"] <- NA        # Replace "?" with NA
      as.numeric(x)            # Convert character vector to numeric
    })
  }

  if (seq == "recon"){
    n.taxa <- length(data$trees$Recon$tree$tip.label)
  } else {
  n.taxa <- length(data$trees$EvolTree$tip.label)
  }
  n.traits <- length( data$sequences[[seq]][[1]])

  ## Are we using a time tree?
  if (timetree) {
    if (seq == "recon") {
      tree_string <-  ape::write.tree(data$trees$Recon$tree)
      tip_labs <- regmatches(tree_string, gregexpr("t\\d+_\\d+", tree_string))[[1]]
    } else {
    tree_string <-  ape::write.tree(data$trees$TimeTree)
    tip_labs <- regmatches(tree_string, gregexpr("t\\d+", tree_string))[[1]]
    }
  } else {
    tree_string <-  ape::write.tree(data$trees$EvolTree)
    tip_labs <- regmatches(tree_string, gregexpr("t\\d+", tree_string))[[1]]
  }

  par(xaxs = "i", yaxs = "i")
  plot(x =c(0,1), y =c(0,1),xaxt = 'n', yaxt = 'n',bty = 'n', pch = '',
       ylab = '', xlab = '', main = "Morphological matrix", cex.main= 1, font.main = 6)

  ## use ablines instead of grid so there is a bit more control
  # need to divide the plot into the number of traits and taxa
  xx <- 1/n.traits
  yy <- 1/n.taxa

  ## to get the center of the yy boxes
  center_a <- yy * 0.5
  center_final <- 1 - center_a
  y_labs <- seq(center_a, center_final, yy )

  axis(2, at=y_labs,labels=tip_labs,
       col.axis="black", las=2, cex.axis=0.8, lwd ="0")

  ## to get the center of the xx boxes
  center_b <- xx * 0.5
  center_final <- 1 - center_b
  x_labs <- seq(center_b, center_final, xx )

  axis(3, at=x_labs,labels=1:n.traits,
       col.axis="black", las=1, cex.axis=0.8, lwd ="0", pos = 0.95)

  if(!is.null(data$model$RateVarTrait) && is.null(data$combined)){
    axis(1, at=x_labs,labels=data$model$RateVarTrait,
         col.axis="black", las=1, cex.axis=0.8, lwd ="0", pos = 0)
    mtext('Rate Category', side=1, line=1, at=-0.07, cex = 0.7, font = 6)
  }
  for (i in 1:n.traits) {
    for (j in 1:n.taxa) {
      state <- as.numeric( data$sequences[[seq]][[tip_labs[j]]][i])
      bg_col <- col[state + 1]
      if (is.na(state)) bg_col = "lightgrey"
      # Draw a rectangle for each box
      rect(
        xleft = x_labs[i]-xx/2, xright = x_labs[i]+xx/2,
        ybottom = y_labs[j]-yy/2, ytop = y_labs[j]+yy/2,
        col = bg_col, border = "black"
      )
    }
  }

  # add border lines
  abline(v=0, col= "grey")
  abline(h=0, col= "grey")


  ## grid up the plot area
  for ( i in 1:n.traits){
    abline(v = xx*i, col= "grey")
  }


  for ( j in 1:n.taxa){
    abline(h = yy*j, col= "grey")
  }


  ## fill in the boxes with the state
  for (i in 1:n.traits) {
    for (j in 1:n.taxa)   {
      state <- as.numeric( data$sequences[[seq]][[tip_labs[j]]][i])
      if (is.na(state)) state = "?"

      # Add the state text in the box
      if (i == num.trait || num.trait == "all") {
        text(x_labs[i], y_labs[j], state, cex = 1, col = "black")
      } else {
        text(x_labs[i], y_labs[j], state, cex = 1, col = "darkgrey")
      }
    }
  }
}

#' Plot full evolutionary history
#'
#' @description
#' This function creates a plot showing continuous evolution of discrete traits.
#'
#' @param x A morpho object
#' @param timetree TRUE or FALSE. Indicate whether you want to plot a time
#'   tree or not. Default = FALSE (uses distance tree if FALSE).
#' @param trait The trait number to plot.
#' @param show.fossil Plot the fossil along the tree. Default = FALSE.
#' @param root.edge If TRUE plot the root edge. Default = FALSE.
#' @param reconstructed Plot the reconstructed tree. Default = FALSE.
#' @param show.convergent When convergent = TRUE, transition boxes are highlighted
#' for any state that arose independently more than once during the simulation.
#' This includes transitions that were subsequently reversed on descendant
#' branches
#' @param edge.width Width of the branches.
#' @param label.offset Distance of tip label to tree tips.
#' @param f.cex Size of fossils.
#' @param e.cex Size of extant taxa.
#' @param box.cex Size of traits on plot
#' @param col A vector of colors that should be the same length or longer than
#'   the number of different character states (k). If not specified, the traits
#'   from 0 to 6 can be differentiated.
#' @param col.timescale A single color for the timescale. Default = "darkgrey".
#' @param ... Other arguments to be passed to methods, such as graphical
#'   parameters.
#'
#' @import stats
#' @import graphics
#' @importFrom ape node.depth.edgelength Ntip
#'
#' @return
#' No return value, called for its side effect of producing a plot.
#'
#' @export
#'
#' @examples
#' # simulate a phylogenetic tree
#' data(morpho_data)
#' plot(morpho_data, trait = 4, timetree = FALSE, show.fossil = FALSE,
#'      root.edge = FALSE, reconstructed = FALSE)
#'
plot.morpho <- function(x = NULL,
                        trait = 1,
                        timetree = TRUE,
                        show.fossil = TRUE,
                        reconstructed = TRUE,
                        show.convergent = TRUE,
                        root.edge = FALSE,
                        edge.width = 1,
                        label.offset = 0.01,
                        e.cex = 0.5,
                        f.cex = 1,
                        box.cex = 4,
                        col = c("#fdfdfd", "lightgray", "lightblue", "pink",
                                "yellow", "green", "orange"),
                        col.timescale = "darkgrey",
                        ...) {



  if (is.null(x)) {
    stop("Error: 'x' cannot be NULL. Please provide a morpho object.")
  }
  if (!is.morpho(x) || is.null(x$trees)) {
    stop("Error: 'x' must be a morpho object containing a 'trees' element.")
  }

  # Fall back to simpler defaults if data not available
  if (timetree && is.null(x$trees$TimeTree)) {
    message("Note: no time tree available, using distance tree")
    timetree <- FALSE
  }
  if (show.fossil && is.null(x$fossil)) {
    message("Note: no fossil data available, fossils will not be shown")
    show.fossil <- FALSE
  }
  if (reconstructed && is.null(x$fossil)) {
    message("Note: no fossil data available, reconstructed tree will not be shown")
    reconstructed <- FALSE
  }

  if (!is.null(trait) && (!is.numeric(trait) || length(trait) != 1)) {
    stop("Error: 'trait' must be a single numeric value or NULL.")
  }
  if (!is.null(trait) && trait > length(x$transition_history)) {
    stop(paste0("Error: 'trait' index (", trait, ") is out of range. The object has only ",
                length(x$transition_history), " traits."))
  }

  if (!is.logical(timetree) || length(timetree) != 1) {
    stop("Error: 'timetree' must be TRUE or FALSE.")
  }
  if (!is.logical(show.fossil) || length(show.fossil) != 1) {
    stop("Error: 'fossil' must be TRUE or FALSE.")
  }
  if (!is.logical(root.edge) || length(root.edge) != 1) {
    stop("Error: 'root.edge' must be TRUE or FALSE.")
  }
  if (!is.logical(reconstructed) || length(reconstructed) != 1) {
    stop("Error: 'reconstructed' must be TRUE or FALSE.")
  }

  if (show.fossil && is.null(x$fossil)) {
    stop("Error: No fossil data found in 'x', but 'fossil = TRUE' was requested.")
  }

  data <- x
  old_par <- par(no.readonly = TRUE)
  on.exit(par(old_par))
  par(mar = c(5, 3, 2.5, 2))

  # choose which tree to plot

  tree_to_plot <- if (timetree) data$trees$TimeTree else data$trees$EvolTree

  label.offset = max(tree_to_plot$edge.length) * 0.02

  # Get branch colours (reconstructed vs. default)

  if (reconstructed) {
    b.cols <- reconstruct.tree(data)
    edge_col <- b.cols[[1]]
  } else {
    edge_col <- "black"
  }

  # plot

  plot(
    tree_to_plot,
    edge.width = edge.width,
    label.offset = label.offset,
    root.edge = root.edge,
    edge.color = edge_col
  )

  # get the plot information

  tree_plot_info <- get("last_plot.phylo", envir = ape::.PlotPhyloEnv)

  edge_start_x <- tree_plot_info$xx[data$trees$EvolTree$edge[, 1]]
  edge_end_x <- tree_plot_info$xx[data$trees$EvolTree$edge[, 2]]
  yy <- tree_plot_info$yy

  edge <- data$trees$EvolTree$edge
  parent <- as.integer(edge[, 1])
  child <- as.integer(edge[, 2])

  # Identify the root

  root <- as.integer(parent[!match(parent, child, 0)][1])

  if (!is.null(trait)) {
    df <- data$transition_history[trait][[1]]

    # identify convergent transitions
    conv_states <- c()
    conv_root <- FALSE
    if (show.convergent) {
      # count how many times each state arose from the transition history
      state_origins <- table(df$state)
      # root is also an origin of its state
      root_st <- as.character(data$root.states[trait])
      if (root_st %in% names(state_origins)) {
        state_origins[root_st] <- state_origins[root_st] + 1
      } else {
        state_origins[root_st] <- 1
      }
      conv_states <- names(state_origins[state_origins > 1])
      conv_root <- root_st %in% conv_states
    }

    root_border <- if (show.convergent && conv_root) "#4292C6" else "black"


    if (root.edge) {
      points(
        data$trees$TimeTree$root.edge, yy[root],
        pch = 22, col = root_border,
        bg = col[as.numeric(data$root.states[trait]) + 1],
        cex = 4
      )
      text(
        data$trees$TimeTree$root.edge, yy[root],
        label = as.numeric(data$root.states[trait])
      )
    } else {
      points(
        0, yy[root], pch = 22, col = root_border,
        bg = col[as.numeric(data$root.states[trait]) + 1],
        cex = box.cex
      )
      text(0, yy[root], label = as.numeric(data$root.states[trait]), cex = box.cex / 4)
    }

    if (nrow(df) > 0) {
      for (i in 1:nrow(df)) {
        branch <- as.numeric(df$edge[i])
        if (timetree) {
          position <- as.numeric(df$hmin[i]) / data$trees$BrRates[branch]
        } else {
          position <- as.numeric(df$hmin[i])
        }
        point_x <- edge_start_x[branch] + position
        if (timetree) {
          point_y <- yy[data$trees$TimeTree[["edge"]][branch, 2]]
        } else {
          point_y <- yy[data$trees$EvolTree[["edge"]][branch, 2]]
        }
        paint <- as.numeric(df$state[i]) + 1

        border_col <- if (show.convergent && as.character(df$state[i]) %in% conv_states) "#4292C6" else "black"

        points(point_x, point_y, pch = 22, col = border_col, bg = col[paint], cex = box.cex)
        text(point_x, point_y, labels = as.numeric(df$state[i]), cex = box.cex / 4)
      }
    }
  }

  if (timetree) {
    tree.age <- max(ape::node.depth.edgelength(data$trees$TimeTree))
  }

  # colour branches (if reconstructed)

  if (reconstructed) {
    if (length(b.cols) == 2) {
      for (p in 1:length(b.cols[[2]])) {
        q <- which(data$fossil$ape.branch == b.cols[[2]][p])
        if (length(q) > 1) {
          fossil_pos <- data$fossil$hmin[which(data$fossil$hmin == min(data$fossil$hmin[q]))]
        } else {
          fossil_pos <- data$fossil$hmin[q]
        }

        tree.age <- max(ape::node.depth.edgelength(data$trees$TimeTree))
        branch <- data$fossil$ape.branch[q]

        time <- fossil_pos
        if (root.edge) {
          actual_position <- tree.age - time + data$trees$TimeTree$root.edge
        } else {
          actual_position <- tree.age - time
        }
        position <- actual_position / data$tree$TimeTree$edge.length[branch]
        point_x <- position * (edge_end_x[branch] - edge_start_x[branch])
        y_vals <- yy[data$trees$TimeTree[["edge"]][b.cols[[2]][p], 2]]
        segments(point_x, y_vals, edge_end_x[b.cols[[2]][p]], y_vals, col = "white")
        segments(point_x, y_vals, edge_end_x[b.cols[[2]][p]], y_vals, col = "grey")
      }
    }
  }

  # add fossils

  if (show.fossil) {
    tree.age <- max(ape::node.depth.edgelength(data$trees$TimeTree))
    for (fsl in 1:length(data$fossil$sp)) {
      branch <- data$fossil$ape.branch[fsl]
      time <- as.numeric(data$fossil$hmax[fsl])
      if (root.edge) {
        actual_position <- tree.age - time + data$trees$TimeTree$root.edge
      } else {
        actual_position <- tree.age - time
      }
      position <- actual_position / data$trees$TimeTree$edge.length[branch]
      point_x <- position * (edge_end_x[branch] - edge_start_x[branch])
      point_y <- yy[data$trees$TimeTree[["edge"]][branch, 2]]

      if (data$fossil$hmax[fsl] == 0) {
        points(point_x, point_y, pch = 16, col = "forestgreen", cex = e.cex)
      } else {
        points(point_x, point_y, pch = 18, col = "black", cex = f.cex)
      }
    }
  }

  # Add a timescale below the plot

  if (timetree) {
    if (root.edge) {
      axis_labels <- c((tree.age + data$trees$TimeTree$root.edge), 0)
      axis(
        1, at = c(0, (tree.age + data$trees$TimeTree$root.edge)),
        labels = round(axis_labels, 2),
        line = 1, col = col.timescale, lwd = 3,
        cex.axis = 1.3, col.axis = col.timescale
      )
    } else {
      axis_labels <- c(tree.age, 0)
      axis(
        1, at = c(0, tree.age),
        labels = round(axis_labels, 2),
        line = 1, col = col.timescale, lwd = 3,
        cex.axis = 1.3, col.axis = col.timescale
      )
    }
    mtext("Time before present", side = 1, line = 2.5, cex = 1.3, col = col.timescale)
  }
}

#' Morpho object
#'
#' Create a morpho object.
#'
#' @param sequences A list containing all of the sequences simulated. This can contain sequences for taxa
#' at the tips or the tree, along the nodes, and if present, for sampled ancestors (SA)
#' @param trees A list containing the trees and branch lengths used for the simulation. EvolTree contains a
#' phylogenetic tree with branch lengths representing evolutionary distance. TimeTree (if present)
#' contains the same tree with branch lengths in unit of time. BrRates can either be a single value, when
#' simulating under a strict clock, or a vector of values representing the rate/branch
#' @param model A list containing all model attributes. Model specifies the components specified to simulate under.
#' RateVar contains the relative values drawn from the specified distribution. RateVarTrait species the rate used
#' to simulate each trait
#' @param root.states A vector supplying the root state for each character
#' @param fossil Fossil object used to simulate data
#' @param transition_history The constant character transitions along the branches
#'
#' @return
#' An object of class \code{"morpho"} containing the simulated morphological
#' data and associated information. The object includes the simulated
#' sequences, phylogenetic trees and branch rates used for the simulation,
#' model parameters, root states, fossil information (if provided), and the
#' character transition history.
#'
#' @examples
#' phy <- ape::rtree(10)
#' morpho_data <- sim.morpho(tree = phy, k = 2, trait.num = 5)
#' is.morpho(morpho_data) # TRUE
#'
#' @export
morpho <- function(sequences = NULL, trees = NULL, model = NULL,
                   transition_history = NULL, root.states = NULL,
                   fossil = NULL) {

  # Create a list to store sequences, tree, and model
  morpho.list <- list(
    sequences = sequences,
    trees = trees,
    model = model,
    transition_history = transition_history,
    root.states = root.states,
    fossil = fossil
  )

  # assign class "morpho" to the object
  attr(morpho.list, "class") <- c("morpho", class(morpho.list))
  return(morpho.list)
}


#' @export
#' @aliases morpho
print.morpho <- function(x, max.length = 5, ...){

  # Convert sequences into a data frame
  seq.data <- t(as.data.frame(x$sequences$tips))

  if(ncol(seq.data) > 4){
    print(seq.data[,1:max.length])
  } else {
    print(seq.data)
  }

  # Print a summary of the morphological data
  cat("Morphological data for", length(x$sequences$tips), "taxa with",
      length(x$sequences$tips[[1]]), "traits per taxon and", sort(unique(as.vector(seq.data))),
      "as character states\n")
  cat("Showing maximum of", max.length, "traits here for now\n")
}

#' @export
#' @aliases morpho
summary.morpho <- function(object, max.length = 5, ...){

  # Use print.morpho to show the data
  print(object, max.length = max.length)

  # Additional information about the tree and model
  cat("Tree with", length(object$tree$tip.label), "tips\n")
  cat("Model:", object$model, "\n")
}



#' @export
#' @rdname morpho
as.morpho <- function(sequences, trees, model = NULL,
                      transition_history = NULL, root.states = NULL,
                      fossil = NULL) {
  UseMethod("as.morpho")
}

#' @export
as.morpho.default <- function(sequences, ...) {
  morpho(sequences, ...)
}

#' @export
#' @rdname morpho
is.morpho <- function(sequences) {
  inherits(sequences, "morpho")
}

#' Generate a symmetric Q matrix
#'
#' Creates a K x K symmetric rate matrix (Q matrix) with equal transition rates between states.
#' The diagonal elements are set such that each row sums to zero.
#'
#' @param K Integer. The number of states.
#'
#' @return A K x K numeric matrix representing the symmetric Q matrix.
#' @examples
#' symmetric.Q.matrix(4)
#' @export
symmetric.Q.matrix <- function(K) {

  # Create an empty matrix of size K x K
  Q <- matrix(0, nrow = K, ncol = K)

  # transition rate for each state
  transition.rate <- 1 / (K)

  for (i in 1:K) {
    for (j in 1:K) {
      if (i != j) {
        Q[i, j] = transition.rate
      }
    }
    Q[i, i] = -(transition.rate*(K-1))
  }
  return(Q)
}


#' Get discrete gamma rates
#'
#' Computes a set of discrete gamma rates for rate variation across sites or characters.
#' This function is adapted from the \code{phangorn} package.
#'
#' @param alpha Numeric. The shape parameter of the gamma distribution.
#' @param k Integer. The number of rate categories.
#'
#' @return Numeric vector of length k representing the discrete gamma rates.
#' @examples
#' get_gamma_rates(alpha = 0.5, k = 4)
#' @export
get_gamma_rates <- function(alpha, k) {
  if (k == 1) return(1)
  # Compute quantiles to divide the gamma distribution into k categories
  quants <- qgamma((1:(k - 1)) / k, shape = alpha, rate = alpha)
  diff(c(0, pgamma(quants * alpha, alpha + 1), 1)) * k
}


#' Get discrete log-normal rates
#'
#' Computes a set of discrete log-normal rates for rate variation across sites or characters.
#' The rates are normalized so that the mean rate equals 1.
#'
#' @param meanlog Numeric. Mean on the log scale.
#' @param sdlog Numeric. Standard deviation on the log scale.
#' @param k Integer. Number of rate categories.
#'
#' @return Numeric vector of length k representing the discrete log-normal rates.
#' @examples
#' get_lognormal_rates(meanlog = 0, sdlog = 1, k = 4)
#' @export
get_lognormal_rates <- function(meanlog, sdlog, k) {
  if (k == 1) return(1)
  quants <- qlnorm((1:(k - 1)) / k, meanlog = meanlog, sdlog = sdlog)
  probs <- diff(c(0, plnorm(quants, meanlog = meanlog, sdlog = sdlog), 1))
  # normalize to k categories so mean rate = 1
  rates <- qlnorm(((1:k) - 0.5) / k, meanlog = meanlog, sdlog = sdlog)
  rates / weighted.mean(rates, probs)
}

#' Color branches for plotting a reconstructed tree
#'
#' @description
#' This function generates colors for branches when plotting a reconstructed tree
#' from a morpho object containing fossil data. Branches that are part of the
#' reconstructed tree or have fossils along them are colored black; all others are grey.
#'
#' @param data A morpho object which contains fossil data and a time-calibrated tree.
#'
#' @return A list of length 2:
#'   \item{b.colours}{Vector of branch colors for plotting.}
#'   \item{rem}{Indices of branches with fossils.}
#'
#' @importFrom ape nodepath
#'
reconstruct.tree <- function(data) {

  # Reconstruct the tree and extract tips
  recon <- FossilSim::reconstructed.tree.fossils.objects(
    data$fossil, data$trees$TimeTree,
    tip_order = "youngest_first"
  )

  tps <- recon$tree$tip.label
  reconTreeTips <- gsub("_1$", "", tps[grepl("_1$", tps)])

  tree  <- data$trees$TimeTree
  ntips <- ape::Ntip(tree)

  # Initialise colour vector
  b.colours <- rep("grey", length(tree$edge.length))

  #  Mark reconstructed-tree branches as black
  tip_matches <- which(tree$tip.label %in% reconTreeTips)

  mark_path_black <- function(node) {
    path <- ape::nodepath(tree, from = node, to = ntips + 1)
    edges <- which(tree$edge[, 2] %in% path)
    b.colours[edges] <<- "black"
  }

  lapply(tip_matches, mark_path_black)

  ## older fossil sampling events
  old <- c()
  if (any(as.numeric(sub("t", "", reconTreeTips)) >
          length(data$trees$EvolTree$tip.label))){

    old_number <- which(as.numeric(sub("t", "", reconTreeTips)) >
                          length(data$trees$EvolTree$tip.label))
    old <- c(old, reconTreeTips[old_number])

  }

  #  For remaining branches, update based on fossils
  rem <- integer(0)
  # get the branch old are on
  if (length(old) > 0){
    for ( o in 1:length(old)){
      ed <- which (data$trees$EvolTree$edge[,2] == as.numeric(sub("t", "", old[o])))
      rem <- c(rem, ed)
    }
  }
  #  Identify fossil-bearing branches
  fossil.branches <- data$fossil$ape.branch

  # branches still grey
  remaining <- which(b.colours == "grey")



  for (rb in remaining) {
    if (rb %in% fossil.branches) {
      node <- tree$edge[rb, 2]
      path <- ape::nodepath(tree, from = node, to = ntips + 1)

      # colour path
      edges <- which(tree$edge[, 2] %in% path)
      b.colours[edges] <- "black"

      # store branches that correspond to fossil tips
      if (any(path <= ntips)) {
        rem <- c(rem, rb)
      }
    }
  }

  # Return list of colours and fossil-branches
  r.cols <- list(NA, NA)
  r.cols[[1]] <- b.colours
  r.cols[[2]] <-  as.matrix(rem)

  return(r.cols)
}


#' Get reconstructed matrix
#' @description This function returns the moprhological matrix for tips in the
#' reconstructed tree.
#'
#' @param data A `morpho` object with fossil data
reconstruct.matrix <- function(data){

  if (!is.morpho(data)) stop ("Error: must provide a morpho object")
  if (is.null(data$fossil)) stop ("Error: need a fossil object for reconstruction")

  r_tree <- FossilSim::reconstructed.tree.fossils.objects(fossils  = data$fossil,
                                                          tree = data$trees$TimeTree,
                                                          tip_order = "youngest_first")

  SA_tips <- c()
  tps <- unname(r_tree$tree$tip.label)
  matches <- grepl("_1$", tps )
  # Extract elements that match
  reconTreeTips <- gsub("_1$", "", tps[matches])

  ## add these tip labels to the file + plus all sampled ancestor
  seq_tips <- which(names(data$sequences$tips) %in% reconTreeTips)

  ## older fossil sampling events
  old <- c()
  if (any(as.numeric(sub("t", "", reconTreeTips)) >
          length(data$trees$EvolTree$tip.label))){

    old_number <- which(as.numeric(sub("t", "", reconTreeTips)) >
                          length(data$trees$EvolTree$tip.label))
    old <- c(old, reconTreeTips[old_number])

  }

  matches <- grepl("_2$", tps )
  reconSA <-  gsub("_2$", "", tps[matches])


  transformation <- matrix(ncol = 2, nrow = length(reconSA))
  colnames(transformation) <- c("Morphsim", "Fossilsim")
  if (length(reconSA) > 0){

    for (l in 1:length(reconSA)){
      t_label <- which(data$trees$TimeTree$tip.label == reconSA[l])
      b_num <- which(data$trees$TimeTree$edge[,2] == t_label)
      spec_min <- min(data$fossil$hmin[data$fossil$ape.branch == b_num])
      spec_num <- data$fossil$specimen[ data$fossil$hmin == spec_min ]
      SA_tips <- rbind(SA_tips, c(paste0(spec_num, "_", b_num)))

      transformation[l,"Morphsim"] <- SA_tips[l]
      transformation[l,"Fossilsim"] <- reconSA[l]
    }

    total_tips <- c(data$sequences$tips[c(seq_tips)], data$sequences$SA[c(SA_tips)])
  } else {
    total_tips <- data$sequences$tips[c(seq_tips)]
  }

  ## need to change the sequence names to match the reconstructed tree
  for ( l in 1:length(seq_tips)){
    current <- names(data$sequences$tips[c(seq_tips)[l]])
    names(total_tips)[names(total_tips) == current ] <- paste0(current, "_1")
  }

  if (length(reconSA) > 0){
    for( l in 1:length(SA_tips)){
      rematch <- unname(transformation[l,"Fossilsim"])
      names(total_tips)[names(total_tips) == SA_tips[l]] <- paste0(rematch, "_2")
    }
  }

  # deals with fossil sampling events not on tips
  if (length(old) > 0){
    sa_names <- strsplit(names(data$sequences$SA), "_")
    name_matrix <- t(sapply(sa_names, function(x) as.numeric(x)))
    for ( o in 1:length(old)){
      ed <- which (data$trees$EvolTree$edge[,2] == as.numeric(sub("t", "", old[o])))
      matches <- name_matrix[name_matrix[,2] == ed, , drop=FALSE]
      youngest <- matches[order(matches[,1]), , drop=FALSE][1,]
      fs_name <- paste0(old[o], "_1")
      ms_name <- paste0(youngest[1], "_",youngest[2])
      total_tips[[fs_name]] <- data$sequences$SA[[ms_name]]
    }
  }
  return(total_tips)

}

#' Remove morphological character data
#'
#' @description
#' This function removes characters from a morphological matrix simulated using morphsim
#' @param data A `morpho` object with sequence data.
#' @param seq Character. Which sequence data to use: "tips", "nodes", or "SA".
#' @param method Character. Method for removing data. Options:
#' \itemize{
#'   \item \code{"random"}: removes characters randomly across the entire matrix.
#'   \item \code{"partition"}: removes characters by partition
#'   (one probability per partition).
#'   \item \code{"rate"}: removes characters by rate category
#'   (one probability per rate category, ordered from slowest to fastest).
#'   \item \code{"trait"}: removes characters from specific traits across all taxa.
#'   \item \code{"taxa"}: removes characters from specific taxa across all traits.
#'   \item \code{"extinct"}: removes data from extinct taxa only.
#'   Requires \code{seq = "tips"} as extinct taxa are only present at the tips of the tree.
#'   \item \code{"trait_taxa"}: removes characters at the intersection of specific traits and
#'     specific taxa, reflecting scenarios where certain anatomical regions are less likely
#'     to be preserved in particular lineages.
#' }
#' @param probability Numeric. Probability of missing data (single value or vector depending on method).
#' @param traits Integer vector. Indices of traits to remove. Required when
#'   \code{method = "trait"} or \code{method = "trait_taxa"}.
#' @param taxa Character vector. Names of taxa to remove data from. Required when
#'   \code{method = "taxa"} or \code{method = "trait_taxa"}.
#'
#' @return An object of class morpho.
#'
#' @export
#' @examples
#' # simulate a phylogenetic tree
#' phy <- ape::rtree(10)
#'
#' # simulate characters along the branches of the tree
#' morpho_data <-  sim.morpho(tree = phy,
#'                            k = c(2,3,4),
#'                            trait.num = 20,
#'                            ancestral = TRUE,
#'                            partition = c(10,5,5),
#'                            ACRV = "gamma",
#'                            variable = TRUE,
#'                            ACRV.ncats = 4,
#'                            define.Q = NULL)
#'
#' # randomly remove data
#' missing.data <- sim.missing.data(data = morpho_data,
#'                                   method = "random",
#'                                   seq = "tips",
#'                                   probability = 0.5)
#'
#'
#' # remove data based on the partition
#' missing.data <- sim.missing.data(data = morpho_data,
#'                                  method = "partition",
#'                                  seq = "tips",
#'                                  probability = c(0.7, 0, 0.5))
#'
#' # remove data based on the rate it was simulated under
#' missing.data <- sim.missing.data(data = morpho_data,
#'                                         method = "rate",
#'                                         seq = "tips",
#'                                         probability = c(0,0,0.2,1))
#'
#' # remove characters from specific traits
#' missing.data <- sim.missing.data(data = morpho_data,
#'                                  method = "trait",
#'                                  seq = "tips",
#'                                  probability = 1,
#'                                  traits = c(1,2,5))
#'
#' # remove characters from specific taxa
#' missing.data <- sim.missing.data(data = morpho_data,
#'                                  method = "taxa",
#'                                  seq = "tips",
#'                                  probability = 1,
#'                                  taxa = c("t1", "t2"))
#'
#' # remove specific character traits from specific taxa
#' missing.data <- sim.missing.data(data = morpho_data,
#'                                  method = "trait_taxa",
#'                                  seq = "tips",
#'                                  probability = 0.8,
#'                                  traits = c(1,2,5),
#'                                  taxa = c("t1", "t2"))
sim.missing.data <- function(data = NULL, seq = NULL, method = NULL, probability = NULL,
                             traits = NULL, taxa = NULL){
  #### Checks
  if (is.null(method) || !method %in% c("random", "partition", "rate", "trait", "taxa", "extinct", "trait_taxa")) {
    stop("Error: must specify a `method`: 'random', 'partition', 'rate', 'trait', 'taxa', 'extinct', or 'trait_taxa'.")
  }
  if (is.null(data) || !inherits(data, "morpho")) {
    stop("Error: `data` must be a morpho object.")
  }
  if (is.null(seq) || !seq %in% c("tips", "nodes", "SA")) {
    stop("Error: `seq` must be one of 'tips', 'nodes', or 'SA'.")
  }
  if (is.null(probability)) {
    stop("Error: You must provide a `probability` value.")
  }
  if (!is.numeric(probability) || any(probability < 0 | probability > 1)) {
    stop("Error: `probability` must be between 0 and 1.")
  }
  if(method == "rate" && !is.null(data$combined)){
    stop ("Error: Cannot use rates on data sets that are combined from different models.
          Simulate missing data first and then combine.")
  }
  if (method == "rate" && length(unique(data$model$RateVarTrait[1,])) != length(probability)){
    stop("Error: Number of probabilities must match the number of rate categories used. See data$model$RateVarTrait" )
  }
  ## Create data frame
  x <- t(as.data.frame(data$sequences[[seq]]))
  trait.num  <- length(x[1,])
  taxa.num <-   length(x[,1])
  ## Method: Random
  if (method == "random"){
    if (length(probability) > 1) stop("For method = 'random', provide a single probability.")
    remove <- round((trait.num* taxa.num)* probability, 0)
    total_cells <- taxa.num*trait.num
    all_combinations <- expand.grid(Row = 1:taxa.num, Column = 1:trait.num)
    random_cells<- all_combinations[sample(1:total_cells, remove, replace = FALSE), ]
    for (i in 1:remove){
      x[random_cells$Row[i], random_cells$Column[i]] <- "?"
    }
  }
  ## Method: Rate
  if (method == "rate"){
    rates <- data$model$RateVarTrait
    sorted_cats <- sort(unique(rates[1, ]))  # ascending: cat 1 (slowest) to cat N (fastest)
    if (length(probability) != length(sorted_cats)) {
      stop("Length of `probability` must match the number of rate categories.")
    }
    for (j in seq_along(sorted_cats)){
      traits_per_rate <- which(rates == sorted_cats[j])
      remove <- round((length(traits_per_rate)* taxa.num)* probability[j], 0)
      total_cells <- length(traits_per_rate)*taxa.num
      all_combinations <- expand.grid(Row = 1:taxa.num, Column = traits_per_rate)
      random_cells <- all_combinations[sample(1:total_cells, remove, replace = FALSE), ]
      for (i in 1:remove){
        x[random_cells$Row[i], random_cells$Column[i]] <- "?"
      }
    }
  }
  ## Method: Partition
  if(method == "partition"){
    if (length(probability) != length(data$model$Specified)){
      stop("Vector of probabilities does not match the number of partitions")
    }
    start_col <- 1
    for ( j in 1:length(data$model$Specified)){
      traits_per_partition <-  as.numeric(sub(".*Part:(\\d+).*", "\\1",
                                              data[["model"]][["Specified"]][j]))
      remove <- round((traits_per_partition* taxa.num)* probability[j], 0)
      total_cells <- traits_per_partition*taxa.num
      all_combinations <- expand.grid(Row = 1:taxa.num,
                                      Column =  start_col:(start_col + traits_per_partition -1))
      random_cells <- all_combinations[sample(1:total_cells, remove, replace = FALSE), ]
      for ( i in 1:remove){
        x[random_cells$Row[i], random_cells$Column[i]] <- "?"
      }
      start_col <- start_col + traits_per_partition
    }
  }
  ## Method: Trait
  if ( method == "trait"){
    if (length(probability) > 1) {
      stop("For method = 'trait', provide a single probability.")
    }
    if (is.null(traits)) stop("For method = 'trait', you must specify `traits`.")
    remove <- round((length(traits)* taxa.num)* probability, 0)
    total_cells <- length(traits)*taxa.num
    all_combinations <- expand.grid(Row = 1:taxa.num, Column = traits)
    random_cells <- all_combinations[sample(1:total_cells, remove, replace = FALSE), ]
    for ( i in 1:remove){
      x[random_cells$Row[i], random_cells$Column[i]] <- "?"
    }
  }
  ## Method: Taxa
  if ( method == "taxa"){
    if (length(probability) > 1) {
      stop("For method = 'taxa', provide a single probability.")
    }
    if (is.null(taxa)) stop("For method = 'taxa', you must specify `taxa`.")
    remove <- round((length(taxa)* trait.num)* probability, 0)
    total_cells <- length(taxa)* trait.num
    all_combinations <- expand.grid(Column = 1:trait.num,  Row = taxa)
    random_cells <- all_combinations[sample(1:total_cells, remove, replace = FALSE), ]
    for ( i in 1:remove){
      x[as.character(random_cells$Row[i]), random_cells$Column[i]] <- "?"
    }
  }
  ## Method: Extinct
  if ( method == "extinct"){
    if (length(probability) > 1) {
      stop("For method = 'extinct', provide a single probability.")
    }
    if (seq != "tips") {
      stop("For method = 'extinct', `seq` must be 'tips' as extinct taxa are only present at the tips of the tree.")
    }
    tip_depths <- ape::node.depth.edgelength(data$trees$TimeTree)[1:length(data$trees$TimeTree$tip.label)]
    tree_height <- max(ape::node.depth.edgelength(data$trees$TimeTree))
    tip_ages <-   round(abs(tip_depths - tree_height),3)
    extinct <- data$trees$EvolTree$tip.label[which(tip_ages != 0) ]
    remove <- round((length(extinct)* trait.num)* probability, 0)
    total_cells <- length(extinct)* trait.num
    all_combinations <- expand.grid(Column = 1:trait.num,  Row = extinct)
    random_cells <- all_combinations[sample(1:total_cells, remove, replace = FALSE), ]
    for ( i in 1:remove){
      x[as.character(random_cells$Row[i]), random_cells$Column[i]] <- "?"
    }
  }
  ## Method: Trait_Taxa
  if (method == "trait_taxa"){
    if (length(probability) > 1) stop("For method = 'trait_taxa', provide a single probability.")
    if (is.null(traits)) stop("For method = 'trait_taxa', you must specify `traits`.")
    if (is.null(taxa)) stop("For method = 'trait_taxa', you must specify `taxa`.")
    remove <- round((length(traits) * length(taxa)) * probability, 0)
    total_cells <- length(traits) * length(taxa)
    all_combinations <- expand.grid(Column = traits, Row = taxa)
    random_cells <- all_combinations[sample(1:total_cells, remove, replace = FALSE), ]
    for (i in 1:remove){
      x[as.character(random_cells$Row[i]), random_cells$Column[i]] <- "?"
    }
  }
  ## update morpho object
  tax <- rownames(x)
  for ( i in 1:length(tax)){
    data$sequences[[seq]][[tax[i]]] <- x[tax[i],]
  }
  return(data)
}

---
title: "Introduction to MorphSim"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Introduction to MorphSim}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  fig.width = 7,
  fig.height = 5
)
```

MorphSim is an R package for simulating discrete morphological character data
along phylogenetic trees. It is designed to integrate with existing R packages
such as TreeSim and FossilSim, enabling simulation of datasets that include
both extant and extinct taxa. This vignette walks through the main features of
the package.

## Getting started

To simulate morphological data you first need a phylogenetic tree. MorphSim
accepts either a tree with branch lengths in evolutionary distance or a
time-scaled tree. Here we simulate a birth-death tree using TreeSim.

```{r tree}
library(MorphSim)
library(TreeSim)
library(FossilSim)

set.seed(1234)
tree <- TreeSim::sim.bd.taxa(n = 10, numbsim = 1,
                              lambda = 0.1, mu = 0.05)[[1]]
```

## Basic simulation

The core function is `sim.morpho`. At minimum you need a tree, the number of
character states (`k`), the total number of traits (`trait.num`), and if using
a time tree, the branch rates (`br.rates`).

```{r basic}
morpho_data <- sim.morpho(
  time.tree = tree,
  k = 2,
  trait.num = 10,
  br.rates = 0.1
)
morpho_data
```

## The morpho object

All simulation output is stored in a `morpho` object. This is a list with the
following components:

**`sequences`** — a list with up to three elements:

- `tips`: character states at the tips of the tree. Stored as a named list
  where each element is a vector of trait values for one taxon.
- `nodes`: character states at internal nodes (when `ancestral = TRUE`).
- `SA`: character states for sampled ancestors (when a fossil object is
  provided). Named using the specimen number and the branch number along
  which the fossil was sampled.

**`trees`** — a list containing:

- `EvolTree`: the tree with branch lengths in evolutionary distance.
- `TimeTree`: the time-scaled tree (if provided).
- `BrRates`: the branch rates used for simulation.

**`model`** — a list describing the model used:

- `Specified`: a string summarising the model per partition, including the
  number of traits and character states.
- `RateVar`: the discrete rate categories drawn from the specified
  distribution (if ACRV was used).
- `RateVarTrait`: which rate category was assigned to each trait. Values
  are listed from lowest rate (1) to highest.

**`transition_history`** — a list of data frames, one per trait. Each data
frame records every state transition that occurred during simulation,
including the branch number (`edge`), the new state (`state`), and the
position along the branch where the transition occurred (`hmin`).

**`root.states`** — a vector of root states, one per trait.

**`fossil`** — the FossilSim fossil object (if provided). The naming scheme
matches that of FossilSim.

You can check whether an object is a morpho object using `is.morpho()`.

```{r morpho_check}
is.morpho(morpho_data)
```

### Accessing data

MorphSim provides helper functions for extracting data from the morpho object.

```{r access}
# Tip data as a matrix
get.matrix(morpho_data, seq = "tips")

# Transition history for a specific trait (including root state)
get.transitions(morpho_data, trait = 1)
```

## Clock models

A single value for `br.rates` applies a strict clock. A vector of rates
(one per branch) applies a relaxed clock. You can use the SimClock package
to generate branch rates under different clock models.

```{r clock}
# Strict clock
strict_data <- sim.morpho(time.tree = tree, k = 2, trait.num = 10,
                           br.rates = 0.1)

# Relaxed clock (random rates per branch for illustration)
relaxed_rates <- runif(nrow(tree$edge), min = 0.01, max = 0.2)
relaxed_data <- sim.morpho(time.tree = tree, k = 2, trait.num = 10,
                            br.rates = relaxed_rates)
```

## Model extensions

### MkV model

Setting `variable = TRUE` ensures all simulated characters are variable
across taxa, matching the MkV model of Lewis (2001).

```{r mkv}
mkv_data <- sim.morpho(time.tree = tree, k = 2, trait.num = 10,
                        br.rates = 0.1, variable = TRUE)
```

### Among-character rate variation (ACRV)

Not all morphological characters evolve at the same rate. MorphSim can
model this using among-character rate variation (ACRV), where each trait is
assigned a rate drawn from a discretized distribution. Two distributions
are available.

**Gamma distribution** — controlled by two parameters: `alpha.gamma`, the
shape parameter of the gamma distribution (smaller values produce greater
rate heterogeneity, with more characters evolving very slowly or very
quickly; larger values concentrate rates closer to the mean), and
`ACRV.ncats`, the number of discrete rate categories used to approximate
the continuous distribution (commonly set to 4, following standard practice
in phylogenetic inference).
```{r acrv_gamma}
gamma_data <- sim.morpho(
  time.tree = tree, k = 2, trait.num = 10, br.rates = 0.1,
  ACRV = "gamma", alpha.gamma = 1, ACRV.ncats = 4
)

# The discrete rate categories
gamma_data$model$RateVar

# Which category was assigned to each trait
gamma_data$model$RateVarTrait
```

**Lognormal distribution** — controlled by `meanlog` and `sdlog`, the mean
and standard deviation on the log scale, and `ACRV.ncats` as above. The
lognormal can produce heavier tails than the gamma, meaning a greater
spread between the slowest and fastest evolving characters.
```{r acrv_lgn}
lgn_data <- sim.morpho(
  time.tree = tree, k = 2, trait.num = 10, br.rates = 0.1,
  ACRV = "lgn", meanlog = 0, sdlog = 1, ACRV.ncats = 4
)

# The discrete rate categories
lgn_data$model$RateVar
```

**User-defined rates** — you can also provide your own rate categories
directly using `ACRV = "user"` and passing a vector of rates to
`define.ACRV.rates`.
```{r acrv_user}
user_data <- sim.morpho(
  time.tree = tree, k = 2, trait.num = 10, br.rates = 0.1,
  ACRV = "user", define.ACRV.rates = c(0.5, 1, 1.5, 2),
  ACRV.ncats = 4
)
```

### MkV + Gamma

These can be combined to simulate under an MkV + Gamma model, consistent
with models implemented in RevBayes and BEAST.

```{r mkv_gamma}
mkvg_data <- sim.morpho(
  time.tree = tree, k = 3, trait.num = 10, br.rates = 0.1,
  variable = TRUE, ACRV = "gamma", alpha.gamma = 1, ACRV.ncats = 4
)
```

### Custom Q matrices

You can provide your own transition rate matrix. For example, an ordered
model where transitions can only occur between adjacent states (0 ↔ 1 ↔ 2):

```{r custom_q}
ord_Q <- matrix(c(
  -0.5, 0.5, 0.0,
   0.3333333, -0.6666667, 0.3333333,
   0.0, 0.5, -0.5
), nrow = 3, byrow = TRUE)

ordered_data <- sim.morpho(
  time.tree = tree, k = 3, trait.num = 10, br.rates = 0.1,
  define.Q = ord_Q
)
```

Note that when using a custom Q matrix, all traits in that simulation share
the same matrix. If you want to simulate different partitions under different
Q matrices, for example, some ordered and some unordered characters, you
can simulate them separately and combine the results. See the section on
combining morpho objects below.

### Specifying the root state

By default the root state is sampled uniformly. You can fix it using
`root.state`:

```{r root_state}
fixed_root <- sim.morpho(
  time.tree = tree, k = 3, trait.num = 10,
  br.rates = 0.1, root.state = 0
)
```

### Ensure full state space

Setting `full.states = TRUE` ensures that all character states specified by `k`
are present in at least one tip for each trait. This is useful when you want to guarantee
that the full range of states specified in the Q matrix is observed at the
tips. Without this, it is possible for a character with `k = 3` to only
show states 0 and 1 at the tips if state 2 was never reached or was lost
before the present.
```{r strict}
strict_data <- sim.morpho(
  time.tree = tree, k = 3, trait.num = 10,
  br.rates = 0.1, full.states = TRUE
)
```

### Partitions

Different groups of characters can be simulated with different numbers of
states. The `partition` argument specifies how many traits belong to each
partition, and `k` gives the number of states per partition.

```{r partitions}
part_data <- sim.morpho(
  time.tree = tree,
  k = c(2, 3, 4),
  trait.num = 20,
  partition = c(10, 5, 5),
  br.rates = 0.1
)

# Model specification per partition
part_data$model$Specified
```

The total number of traits defined by trait.num and partition must match or the function will return an error.

### Combining morpho objects

It is not possible to simulate multiple partitions under different models in
a single call to `sim.morpho`. For example, you might want some characters
to evolve under an ordered model with a custom Q matrix and others under a
standard Mk model with gamma rate variation. To do this, simulate each
partition separately and combine them using `combine.morpho`.

```{r combine}
# Partition 1: ordered characters with a custom Q matrix
ord_Q <- matrix(c(
  -0.5, 0.5, 0.0,
   0.3333333, -0.6666667, 0.3333333,
   0.0, 0.5, -0.5
), nrow = 3, byrow = TRUE)

partition_1 <- sim.morpho(
  time.tree = tree, k = 3, trait.num = 10,
  br.rates = 0.1, define.Q = ord_Q
)

# Partition 2: unordered binary characters with MkV + Gamma
partition_2 <- sim.morpho(
  time.tree = tree, k = 2, trait.num = 10,
  br.rates = 0.1, variable = TRUE,
  ACRV = "gamma", alpha.gamma = 1, ACRV.ncats = 4
)

# Combine into a single morpho object
combined <- combine.morpho(partition_1, partition_2)

# The combined object has 20 traits total
length(combined$sequences$tips[[1]])

# Model specifications from both partitions are preserved
combined$model$Specified
```

Both morpho objects must share the same tree, branch lengths, and fossil
object (if present). The combined object merges the sequences, transition
histories, root states, and model information from both inputs.

## FossilSim integration

MorphSim integrates with FossilSim to simulate character data for sampled
ancestors, fossils that represent direct ancestors of other lineages.
Because MorphSim tracks where transitions occur along branches, it generates
trait values appropriate to the time and lineage at which each fossil was
sampled.

```{r fossils}
# Simulate fossil and extant sampling
f <- FossilSim::sim.fossils.poisson(rate = 0.1, tree = tree,
                                     root.edge = FALSE)
f2 <- FossilSim::sim.extant.samples(fossils = f, tree = tree, rho = 0.5)

# Simulate morphological data with fossils
fossil_data <- sim.morpho(
  time.tree = tree, k = c(2,3), trait.num = 10, partition = c(5,5),
  br.rates = 0.1, fossil = f2
)

# Sampled ancestor data
names(fossil_data$sequences$SA)
```

## Missing data

Fossil morphological data are often incomplete. `sim.missing.data` allows
you to introduce missing characters under different scenarios, replacing
character values with `"?"`.

### Random missing data

Characters are removed uniformly at random across the entire matrix. A
single probability value controls the proportion of data that is removed.

```{r missing_random}
missing_random <- sim.missing.data(
  data = fossil_data, method = "random",
  seq = "tips", probability = 0.3
)
```

### Missing data by partition

Different partitions can have different probabilities of missing data. This
reflects how different anatomical regions may have different preservation
potential, for example, robust skeletal elements are more likely to be
preserved than soft tissue features. The probability vector must match the
number of partitions.

```{r missing_partition}
part_fossil <- sim.morpho(
  time.tree = tree, k = c(2, 3), trait.num = 10,
  partition = c(5, 5), br.rates = 0.1, fossil = f2
)

# Hard tissues (partition 1): 10% missing
# Soft tissues (partition 2): 70% missing
missing_part <- sim.missing.data(
  data = part_fossil, method = "partition",
  seq = "tips", probability = c(0.1, 0.7)
)
```

### Missing data by rate category

When data is simulated with ACRV, characters can be removed according to
the rate category they were simulated under. The probability vector must
match the number of rate categories. This can be used to explore what
happens when faster or slower evolving characters are preferentially lost.

```{r missing_rate}
gamma_fossil <- sim.morpho(
  time.tree = tree, k = 2, trait.num = 20, br.rates = 0.1,
  ACRV = "gamma", alpha.gamma = 1, ACRV.ncats = 4, fossil = f2
)

# Slow characters preserved, fast characters increasingly lost
missing_rate <- sim.missing.data(
  data = gamma_fossil, method = "rate",
  seq = "tips", probability = c(0, 0.2, 0.5, 0.8)
)
```

### Missing data by specific traits

Remove characters from specific traits. This is useful for exploring the
impact of losing particular characters.

```{r missing_trait}
missing_trait <- sim.missing.data(
  data = fossil_data, method = "trait",
  seq = "tips", probability = 1, traits = c(1, 2, 3)
)
```

### Missing data by specific taxa

Remove characters from named taxa. This can be used to simulate scenarios
where certain specimens are poorly preserved.

```{r missing_taxa}
# Remove all data from the first two taxa
taxa_to_remove <- names(fossil_data$sequences$tips)[1:2]
missing_taxa <- sim.missing.data(
  data = fossil_data, method = "taxa",
  seq = "tips", probability = 1, taxa = taxa_to_remove
)
```

### Missing data from extinct taxa only

Remove characters only from extinct taxa while leaving extant taxa complete.
This reflects the common scenario where fossil specimens have more missing
data than living species.

```{r missing_extinct, eval = FALSE}
missing_extinct <- sim.missing.data(
  data = fossil_data, method = "extinct",
  seq = "tips", probability = 0.5
)
```

### Missing data for specific traits and taxa

Remove characters from specific traits and specific taxa.
This reflects scenarios where certain anatomical regions are less likely to
be preserved in particular lineages.
```{r missing_trait_taxa}
missing_trait_taxa <- sim.missing.data(
  data = fossil_data, method = "trait_taxa",
  seq = "tips", probability = 0.8,
  traits = c(1, 2, 3),
  taxa = taxa_to_remove
)
```

## Exploring simulated data

### Summary statistics

`stats.morpho` computes the consistency index, retention index, identifies
convergent traits, and summarises the tree structure. The consistency index (CI; Kluge and Farris, 1969) measures the minimum number of character changes required on a tree relative to the observed number of changes, with values closer to 1 indicating less homoplasy. The retention index (RI; Farris, 1989) quantifies how well synapomorphies are retained on the tree, with higher values indicating stronger phylogenetic signal. Both metrics range from 0 to 1.

```{r stats}
stats <- stats.morpho(fossil_data)
stats$Statistics
stats$Tree
```

### Convergence detection

`get.convergent` identifies traits where the same state arose independently
more than once. Called without a trait number, it returns a summary across
all traits. Called with a specific trait, it shows which tips inherited their
state from which evolutionary origin. Tips sharing the same origin inherited
their state from a single transition event. Tips with the same state but
different origins represent convergent evolution.

```{r convergent}
# Which traits are convergent?
get.convergent(fossil_data)

# Detail for a specific trait
get.convergent(fossil_data, trait = 1)
```

### Transition histories

`get.transitions` returns the full transition history for a trait, including
the root state (shown as edge 0).

```{r transitions}
get.transitions(fossil_data, trait = 1)
```

## Plotting

### Trait history on the tree

`plot.morpho` displays the evolutionary history of a trait along the tree.
Transition boxes show where state changes occurred along each branch. We
start with the simplest version, just the trait history on the distance
tree, with all additional features turned off.

```{r plot_basic, fig.height = 6}
plot(fossil_data, trait = 6, timetree = FALSE,
     show.fossil = FALSE, reconstructed = FALSE,
     show.convergent = FALSE)
```

### Adding fossils

When `show.fossil = TRUE`, fossils are plotted along the branches of the
tree at the time they were sampled. Extant taxa are shown as green circles
at the tips and fossil samples as black diamonds along the branches. This
requires a time tree, so we also set `timetree = TRUE`.

```{r plot_fossils, fig.height = 6}
plot(fossil_data, trait = 6, timetree = TRUE,
     show.fossil = TRUE, reconstructed = FALSE,
     show.convergent = FALSE)
```

### Showing the reconstructed tree

When `reconstructed = TRUE`, the plot distinguishes between the full
(true) tree and the reconstructed tree, the tree containing only the
sampled lineages. Branches that are part of the reconstructed tree are
drawn in black, while unsampled lineages are shown in grey. This helps
visualise how much of the true evolutionary history is captured by the
available samples.

```{r plot_recon, fig.height = 6}
plot(fossil_data, trait = 6, timetree = TRUE,
     show.fossil = TRUE, reconstructed = TRUE,
     show.convergent = FALSE)
```

### Highlighting convergent evolution

When `show.convergent = TRUE`, transition boxes for any state that arose
independently more than once are highlighted in blue. This includes
transitions that were subsequently reversed on descendant branches, giving
a complete picture of where convergence occurred during the simulation.

```{r plot_convergent, fig.height = 6}
plot(fossil_data, trait = 6, timetree = TRUE,
     show.fossil = TRUE, reconstructed = TRUE,
     show.convergent = TRUE)
```

By default, `plot.morpho` shows all available features, the time tree,
fossils, reconstructed tree, and convergence highlighting. If any of these
are not available in the morpho object (e.g., no fossil data was provided),
the plot falls back gracefully and informs the user. So the plot above is
equivalent to the default:

```{r plot_default, eval = FALSE}
plot(fossil_data, trait = 7)
```

### Character matrix

`plotMorphoGrid` displays the full character matrix at the tips.

```{r grid, fig.height = 5}
plotMorphoGrid(fossil_data, seq = "tips")
```

## Exporting data

`write.morpho` provides a single interface for exporting trees, character
matrices, and fossil ages. The `type` argument specifies what to export,
and `reconstructed` controls whether the full or reconstructed version is
written.

```{r export, eval = FALSE}
# Full tree
write.morpho(fossil_data, file = "tree.tre", type = "tree")

# Reconstructed tree
write.morpho(fossil_data, file = "recon_tree.tre", type = "tree",
             reconstructed = TRUE)

# Character matrix
write.morpho(fossil_data, file = "matrix.nex", type = "matrix")

# Reconstructed matrix
write.morpho(fossil_data, file = "recon_matrix.nex", type = "matrix",
             reconstructed = TRUE)

# Fossil ages (compatible with RevBayes)
write.morpho(fossil_data, file = "ages.tsv", type = "ages")

# With age uncertainty
write.morpho(fossil_data, file = "ages.tsv", type = "ages",
             uncertainty = 2)
```

Standard ape export functions also work since the morpho object stores
trees as `phylo` objects:

```{r export_ape, eval = FALSE}
ape::write.tree(fossil_data$trees$EvolTree, file = "full_tree.tre")
ape::write.nexus.data(fossil_data$sequences$tips, file = "full_matrix.nex",
                      format = "standard")
```

## Using with ggtree

The morpho object can be converted to a `treedata` object for use with
ggtree. This requires the treeio, ggtree, tibble, and ggplot2 packages.

```{r ggtree, eval = FALSE}
library(treeio)
library(ggtree)
library(ggplot2)
library(tibble)

# Convert tip data to a data frame
tip_df <- as.data.frame(t(as.data.frame(fossil_data$sequences$tips)))
colnames(tip_df) <- paste0("trait_", seq_len(ncol(tip_df)))
tip_df$node <- match(rownames(tip_df),
                     fossil_data$trees$EvolTree$tip.label)

# Create treedata object
td <- treedata(phylo = fossil_data$trees$EvolTree,
               data = as_tibble(tip_df))

# Plot tree with coloured tip points for one trait
ggtree(td) +
  geom_tiplab(offset = 0.01) +
  geom_tippoint(aes(color = factor(trait_1)), size = 3) +
  scale_color_brewer(palette = "Set2", name = "State") +
  theme(legend.position = "right")

# Plot tree with heatmap of all traits
heat_df <- tip_df[, grep("trait_", colnames(tip_df))]
heat_df[] <- lapply(heat_df, factor)

gheatmap(ggtree(td) + geom_tiplab(offset = 0.05),
         heat_df, offset = 0.02, width = 0.5,
         colnames_angle = 90, hjust = 1) +
  scale_fill_brewer(palette = "Set2", name = "State")
```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.morpho.R
\name{plot.morpho}
\alias{plot.morpho}
\title{Plot full evolutionary history}
\usage{
\method{plot}{morpho}(
  x = NULL,
  trait = 1,
  timetree = TRUE,
  show.fossil = TRUE,
  reconstructed = TRUE,
  show.convergent = TRUE,
  root.edge = FALSE,
  edge.width = 1,
  label.offset = 0.01,
  e.cex = 0.5,
  f.cex = 1,
  box.cex = 4,
  col = c("#fdfdfd", "lightgray", "lightblue", "pink", "yellow", "green", "orange"),
  col.timescale = "darkgrey",
  ...
)
}
\arguments{
\item{x}{A morpho object}

\item{trait}{The trait number to plot.}

\item{timetree}{TRUE or FALSE. Indicate whether you want to plot a time
tree or not. Default = FALSE (uses distance tree if FALSE).}

\item{show.fossil}{Plot the fossil along the tree. Default = FALSE.}

\item{reconstructed}{Plot the reconstructed tree. Default = FALSE.}

\item{show.convergent}{When convergent = TRUE, transition boxes are highlighted
for any state that arose independently more than once during the simulation.
This includes transitions that were subsequently reversed on descendant
branches}

\item{root.edge}{If TRUE plot the root edge. Default = FALSE.}

\item{edge.width}{Width of the branches.}

\item{label.offset}{Distance of tip label to tree tips.}

\item{e.cex}{Size of extant taxa.}

\item{f.cex}{Size of fossils.}

\item{box.cex}{Size of traits on plot}

\item{col}{A vector of colors that should be the same length or longer than
the number of different character states (k). If not specified, the traits
from 0 to 6 can be differentiated.}

\item{col.timescale}{A single color for the timescale. Default = "darkgrey".}

\item{...}{Other arguments to be passed to methods, such as graphical
parameters.}
}
\value{
No return value, called for its side effect of producing a plot.
}
\description{
This function creates a plot showing continuous evolution of discrete traits.
}
\examples{
# simulate a phylogenetic tree
data(morpho_data)
plot(morpho_data, trait = 4, timetree = FALSE, show.fossil = FALSE,
     root.edge = FALSE, reconstructed = FALSE)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/summary.r
\name{stats.morpho}
\alias{stats.morpho}
\title{Calculates statistics for a morpho object}
\usage{
stats.morpho(data)
}
\arguments{
\item{data}{A morpho object}
}
\value{
A list with three elements:
  - \code{Statistics}: data.frame with CI and RI
  - \code{Convergent_Traits}: data.frame listing convergent traits
  - \code{Tree}: data.frame summarizing extant/extinct tips and sampled ancestors
}
\description{
Computes three key pieces of information for a morpho object:
1. The Consistency Index (CI) and Retention Index (RI) based on the tip sequence data.
2. Convergent traits, identifying traits that have evolved independently multiple times.
3. Summary information about the size and structure of the tree.
}
\examples{
data(morpho_data)

summary <- stats.morpho(data = morpho_data)

}
\seealso{
\code{\link{get.convergent}} for detailed per-trait
convergence information
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data_morpho.R
\docType{data}
\name{morpho_data}
\alias{morpho_data}
\title{Example morpho dataset}
\format{
A list with components:
\describe{
  \item{sequences}{List of sequences for each taxon}
  \item{trees}{List containing the evolutionary tree}
}
}
\usage{
data(morpho_data)
}
\description{
Small example dataset for testing morpho functions.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/morpho.R
\name{morpho}
\alias{morpho}
\alias{as.morpho}
\alias{is.morpho}
\title{Morpho object}
\usage{
morpho(
  sequences = NULL,
  trees = NULL,
  model = NULL,
  transition_history = NULL,
  root.states = NULL,
  fossil = NULL
)

as.morpho(
  sequences,
  trees,
  model = NULL,
  transition_history = NULL,
  root.states = NULL,
  fossil = NULL
)

is.morpho(sequences)
}
\arguments{
\item{sequences}{A list containing all of the sequences simulated. This can contain sequences for taxa
at the tips or the tree, along the nodes, and if present, for sampled ancestors (SA)}

\item{trees}{A list containing the trees and branch lengths used for the simulation. EvolTree contains a
phylogenetic tree with branch lengths representing evolutionary distance. TimeTree (if present)
contains the same tree with branch lengths in unit of time. BrRates can either be a single value, when
simulating under a strict clock, or a vector of values representing the rate/branch}

\item{model}{A list containing all model attributes. Model specifies the components specified to simulate under.
RateVar contains the relative values drawn from the specified distribution. RateVarTrait species the rate used
to simulate each trait}

\item{transition_history}{The constant character transitions along the branches}

\item{root.states}{A vector supplying the root state for each character}

\item{fossil}{Fossil object used to simulate data}
}
\value{
An object of class \code{"morpho"} containing the simulated morphological
data and associated information. The object includes the simulated
sequences, phylogenetic trees and branch rates used for the simulation,
model parameters, root states, fossil information (if provided), and the
character transition history.
}
\description{
Create a morpho object.
}
\examples{
phy <- ape::rtree(10)
morpho_data <- sim.morpho(tree = phy, k = 2, trait.num = 5)
is.morpho(morpho_data) # TRUE

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/reconstruct.tree.R
\name{reconstruct.tree}
\alias{reconstruct.tree}
\title{Color branches for plotting a reconstructed tree}
\usage{
reconstruct.tree(data)
}
\arguments{
\item{data}{A morpho object which contains fossil data and a time-calibrated tree.}
}
\value{
A list of length 2:
  \item{b.colours}{Vector of branch colors for plotting.}
  \item{rem}{Indices of branches with fossils.}
}
\description{
This function generates colors for branches when plotting a reconstructed tree
from a morpho object containing fossil data. Branches that are part of the
reconstructed tree or have fossils along them are colored black; all others are grey.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/summary.r
\name{combine.morpho}
\alias{combine.morpho}
\title{Combine Two Morpho Objects}
\usage{
combine.morpho(x, y)
}
\arguments{
\item{x}{A `morpho` object.}

\item{y}{A `morpho` object.}
}
\value{
A combined `morpho` object.
}
\description{
This function merges two `morpho` objects, combining their sequences,
model parameters, and transition histories, while ensuring tree and fossil
consistency.
}
\examples{
phy <- ape::rtree(10)

# simulate characters along the branches of the tree
morpho1 <-  sim.morpho(tree = phy,
                           k = c(2,3,4),
                           trait.num = 20,
                           ancestral = TRUE,
                           partition = c(10,5,5),
                           ACRV = "gamma",
                           variable = TRUE,
                           ACRV.ncats = 4,
                           define.Q = NULL)

morpho2 <-  sim.morpho(tree = phy,
                           k = c(2,3,4),
                           trait.num = 20,
                           ancestral = TRUE,
                           partition = c(10,5,5),
                           ACRV = "gamma",
                           variable = TRUE,
                           ACRV.ncats = 4,
                           define.Q = NULL)

combined <- combine.morpho(morpho1, morpho2)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/summary.r
\name{convergent_evol}
\alias{convergent_evol}
\title{Determines the number of convergently evolved traits}
\usage{
convergent_evol(data = NULL)
}
\arguments{
\item{data}{A morpho object}
}
\value{
A data.frame listing convergent traits, their state, and number of transitions
}
\description{
Identifies which traits have evolved independently multiple times
(convergent evolution) in a morpho object.
}
\examples{
phy <- ape::rtree(10)
morpho_data <- sim.morpho(tree = phy, k = 2, trait.num = 20)
convergent_evol(morpho_data)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotMorphoGrid.R
\name{plotMorphoGrid}
\alias{plotMorphoGrid}
\title{Plots morphological matrix}
\usage{
plotMorphoGrid(
  data = NULL,
  timetree = FALSE,
  seq = "tips",
  num.trait = "all",
  col = c("lavender", "white", "lightskyblue1", "pink", "gold2", "forestgreen", "coral")
)
}
\arguments{
\item{data}{A morpho object}

\item{timetree}{TRUE or FALSE Indicate whether you want to plot a time tree or not. default FALSE, uses distance tree if FALSE}

\item{seq}{the sequence data to plot: "tips", "nodes", "SA", or "recon"}

\item{num.trait}{default is set to "all" which plots all traits in black font. If you
want to focus on a specific trait set it here, e.g. num.trait = 1 and this trait will
be highlighted}

\item{col}{A vector of colors that should be the same length or longer than the number of different character states (k). if not specified, the traits from 0 to 6 can be differentiated}
}
\value{
No return value, called for its side effect of producing a plot.
}
\description{
This function plots the full morphological matrix assocaited with the character data
at the tips of a tree. Requires a morpho object as input.
}
\examples{
data(morpho_data)
# plot the character matrix
plotMorphoGrid(data = morpho_data, seq = "tips", num.trait = "all")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/write.r
\name{write.recon.matrix}
\alias{write.recon.matrix}
\title{Write reconstructed character matrix to file}
\usage{
write.recon.matrix(data, file = NULL)
}
\arguments{
\item{data}{Morpho object}

\item{file}{File name}
}
\value{
No return value, called for its side effect of writing data to a file.
}
\description{
Write the character matrix for the reconstructed tree to a nexus file
}
\examples{
data(morpho_data)
tmp <- tempfile(fileext = ".nex")
write.recon.matrix(data = morpho_data, file = tmp)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/reconstruct.tree.R
\name{reconstruct.matrix}
\alias{reconstruct.matrix}
\title{Get reconstructed matrix}
\usage{
reconstruct.matrix(data)
}
\arguments{
\item{data}{A `morpho` object with fossil data}
}
\description{
This function returns the moprhological matrix for tips in the
reconstructed tree.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/GenerateQ.matrices.r
\name{get_lognormal_rates}
\alias{get_lognormal_rates}
\title{Get discrete log-normal rates}
\usage{
get_lognormal_rates(meanlog, sdlog, k)
}
\arguments{
\item{meanlog}{Numeric. Mean on the log scale.}

\item{sdlog}{Numeric. Standard deviation on the log scale.}

\item{k}{Integer. Number of rate categories.}
}
\value{
Numeric vector of length k representing the discrete log-normal rates.
}
\description{
Computes a set of discrete log-normal rates for rate variation across sites or characters.
The rates are normalized so that the mean rate equals 1.
}
\examples{
get_lognormal_rates(meanlog = 0, sdlog = 1, k = 4)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/write.r
\name{write.recon.tree}
\alias{write.recon.tree}
\title{Write reconstructed tree to file}
\usage{
write.recon.tree(data = NULL, file = NULL)
}
\arguments{
\item{data}{Morpho object}

\item{file}{File name}
}
\value{
No return value, called for its side effect of writing data to a file.
}
\description{
Write the reconstructed tree to Newick string
}
\examples{
data(morpho_data)
tmp <- tempfile(fileext = ".tre")
write.recon.tree(data = morpho_data, file = tmp)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/GenerateQ.matrices.r
\name{get_gamma_rates}
\alias{get_gamma_rates}
\title{Get discrete gamma rates}
\usage{
get_gamma_rates(alpha, k)
}
\arguments{
\item{alpha}{Numeric. The shape parameter of the gamma distribution.}

\item{k}{Integer. The number of rate categories.}
}
\value{
Numeric vector of length k representing the discrete gamma rates.
}
\description{
Computes a set of discrete gamma rates for rate variation across sites or characters.
This function is adapted from the \code{phangorn} package.
}
\examples{
get_gamma_rates(alpha = 0.5, k = 4)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/summary.r
\name{find_path_to_tip}
\alias{find_path_to_tip}
\title{Determines the route (nodes and branches) for a tip in a phylogenetic tree}
\usage{
find_path_to_tip(tree, tip)
}
\arguments{
\item{tree}{A phylogenetic tree of class \code{phylo}}

\item{tip}{Tip label (character)}
}
\value{
A matrix with columns \code{parent} and \code{child} representing the path
}
\description{
Traverses the tree to determine the evolutionary path (branches) from root to a given tip.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sim.morpho.R
\name{sim.morpho}
\alias{sim.morpho}
\title{Simulate characters along branches in a tree}
\usage{
sim.morpho(
  tree = NULL,
  time.tree = NULL,
  k = 2,
  trait.num,
  partition = NULL,
  br.rates = NULL,
  root.state = NULL,
  variable = FALSE,
  full.states = FALSE,
  parsimony = NULL,
  ACRV = NULL,
  alpha.gamma = 1,
  ACRV.ncats = 4,
  meanlog = NULL,
  sdlog = NULL,
  define.ACRV.rates = NULL,
  ancestral = TRUE,
  fossil = NULL,
  define.Q = NULL
)
}
\arguments{
\item{tree}{A phylogenetic tree (class \code{"phylo"}) with branches
representing genetic distance.}

\item{time.tree}{A phylogenetic tree (class \code{"phylo"}) with branches
representing time.}

\item{k}{Integer (>= 2). Number of trait states. Can be a vector if using
partitions.}

\item{trait.num}{Integer (> 0). The total number of traits to simulate.}

\item{partition}{Integer vector. Specifies the number of traits per
partition. Must sum to \code{trait.num}.}

\item{br.rates}{Numeric. Clock rates per branch. Can be a single value
(strict clock) or a numeric vector of length equal to the number of
branches in the tree, one rate per branch in the same order as
\code{tree$edge}. Custom vectors of rates can be provided directly here,
for example rates simulated using \code{simclock::relaxed.tree()}.}

\item{root.state}{Integer. The root state at the start of the simulation.
Must be within the range of states implied by \code{k}.}

\item{variable}{Logical. If \code{TRUE}, simulate only varying characters
(MkV model). Default is \code{FALSE}.}

\item{full.states}{Logical. If \code{TRUE}, ensures that all character
states specified by \code{k} are present in at least one tip for each
trait. Default is \code{FALSE}.}

\item{parsimony}{Character. If \code{"standard"}, retain only characters
where at least two states each occur in two or more taxa; autapomorphic
states are tolerated. If \code{"strict"}, every observed state must occur
in two or more taxa, excluding characters with any autapomorphic states
entirely. If \code{NULL}, no parsimony filter is applied. Default is
\code{NULL}.}

\item{ACRV}{Character. Among-character rate variation model. One of
\code{"gamma"}, \code{"lgn"}, \code{"user"}, or \code{NULL}.
\code{"gamma"} draws rates from a discretized gamma distribution (Yang
1994); requires \code{ACRV.ncats} and \code{alpha.gamma}.
\code{"lgn"} draws rates from a discretized lognormal distribution
(Wagner 2012); requires \code{ACRV.ncats}, \code{meanlog}, and
\code{sdlog}. \code{"user"} uses user-supplied rates via
\code{define.ACRV.rates}. \code{NULL} simulates all traits under the
same rate. Default is \code{NULL}.}

\item{alpha.gamma}{Numeric. Shape parameter \eqn{\alpha} for the gamma
distribution. Default is 1.}

\item{ACRV.ncats}{Integer. Number of discrete rate categories for
among-character rate variation.}

\item{meanlog}{Numeric. Mean of the lognormal distribution on the log
scale. Required when \code{ACRV = "lgn"}.}

\item{sdlog}{Numeric. Standard deviation of the lognormal distribution on
the log scale. Required when \code{ACRV = "lgn"}.}

\item{define.ACRV.rates}{Numeric vector. User-supplied rate categories.
Required when \code{ACRV = "user"}.}

\item{ancestral}{Logical. If \code{TRUE}, return the states at all
ancestral nodes. Default is \code{TRUE}.}

\item{fossil}{A fossil object from \code{FossilSim} to simulate
morphological characters for sampled ancestors.}

\item{define.Q}{Numeric matrix. A custom transition rate matrix Q for
simulation. Must be square and rows must sum to zero.}
}
\value{
An object of class `morpho`, with the following components:
\describe{
  \item{sequences}{A list containing up to 3 elements: morphological data for the `tips`
  of the tree, the `nodes`, and, if provided, the sampled ancestors (`SA`). For `SA`,
  the naming scheme differs from that of `FossilSim`: the morphological data are named
  using the specimen number (`data$fossil$specimen`) and the branch number along which
  the fossil was sampled.}

  \item{tree}{A list containing up to 3 elements: the `EvolTree` (branch lengths in
  genetic distance), the `TimeTree` (branch lengths in time units), and `BrRates`
  (the evolutionary rate per branch).}

  \item{model}{Information about the model used to simulate the data. `Specified`
  states the exact model used per partition, as well as the number of traits and
  character states respectively. `RateVar` contains the relative rates used to simulate
  the data, and `RateVarTrait` contains information about which rate category was used
  to simulate each trait. These values are listed from lowest rate (1) to highest.}

  \item{transition_history}{A list containing *n* data frames, where *n* is the number
  of simulated traits. Each data frame contains information about transitions that
  occurred for that trait, including the branch number (`edge`), the new state number (`state`),
  and the point along the branch where the transition occurred (`hmin`).}

  \item{root.states}{A vector of root states for each trait.}

  \item{fossil}{The fossil object provided to `morphsim` from `FossilSim`. The naming
  scheme therefore matches that of `FossilSim`.}
}
}
\description{
This function simulates discrete character data along the branches of a
phylogenetic tree using a continuous-time Markov chain. It implements the
Mk model and its extensions (Lewis 2001), including among-character rate
variation (ACRV) using discretized gamma distributions (Yang 1994) or
lognormal distributions (Wagner 2012), following the implementation
described in Capobianco and Hohna (2025). It can be used with either a
time tree or a tree with branch lengths in evolutionary distance. If using
a time tree, branch rates can be specified either as a single value for all
branches or as a vector with different rates per branch. If no branch rates
are specified a default of 0.1 is applied to all branches.
}
\examples{
# simulated tree
phy <- ape::rtree(10)

# simulate characters along the branches of the tree
morpho_data <-  sim.morpho(tree = phy,
                           k = c(2,3,4),
                           trait.num = 20,
                           partition = c(10,5,5),
                           ACRV = "gamma",
                           ACRV.ncats = 4,
                           variable = TRUE,
                           ancestral = TRUE,
                           define.Q = NULL)

# To simulate ordered characters:
# First define a Q-matrix. The following is for ordered characters where transitions can only occur
# between states 0 and 1 and 1 and 2

ord_Q <- matrix(c(
-0.5, 0.5, 0.0,
0.3333333, -0.6666667, 0.3333333,
0.0, 0.5, -0.5
), nrow = 3, byrow = TRUE)

# This Q matrix can be then used to simulate character data.

morpho_data <- sim.morpho(tree = phy,
                          k = 3,
                          trait.num = 20,
                          ancestral = TRUE,
                          ACRV = "gamma",
                          variable = TRUE,
                          ACRV.ncats = 4,
                         define.Q = ord_Q)
}
\references{
Lewis, P.O. (2001) A likelihood approach to estimating phylogeny from
discrete morphological character data. \emph{Systematic Biology} 50:913-925.

Yang, Z. (1994) Maximum likelihood phylogenetic estimation from DNA
sequences with variable rates over sites: approximate methods.
\emph{Journal of Molecular Evolution} 39:306-314.

Wagner, P.J. (2012) Modelling rate distributions using character
compatibility: implications for morphological evolution among fossil
invertebrates. \emph{Biology Letters} 8:143-146.

Capobianco, A. and Hohna, S. (2025) On the MkV model with
among-character rate variation. \emph{Systematic Biology}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/summary.r
\name{get.transitions}
\alias{get.transitions}
\title{Get Tranisition History}
\usage{
get.transitions(data, trait)
}
\arguments{
\item{data}{A morpho object}

\item{trait}{The trait number .}
}
\value{
A data frame with columns edge (branch number), state, and
hmin (where along the branch the transition occurred)
}
\description{
Returns the full transition history for a given trait,
including the root state.
}
\examples{
phy <- ape::rtree(10)
morpho_data <- sim.morpho(tree = phy, k = 2, trait.num = 20)
get.transitions(morpho_data, trait = 2)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sim.missing.data.R
\name{sim.missing.data}
\alias{sim.missing.data}
\title{Remove morphological character data}
\usage{
sim.missing.data(
  data = NULL,
  seq = NULL,
  method = NULL,
  probability = NULL,
  traits = NULL,
  taxa = NULL
)
}
\arguments{
\item{data}{A `morpho` object with sequence data.}

\item{seq}{Character. Which sequence data to use: "tips", "nodes", or "SA".}

\item{method}{Character. Method for removing data. Options:
\itemize{
  \item \code{"random"}: removes characters randomly across the entire matrix.
  \item \code{"partition"}: removes characters by partition
  (one probability per partition).
  \item \code{"rate"}: removes characters by rate category
  (one probability per rate category, ordered from slowest to fastest).
  \item \code{"trait"}: removes characters from specific traits across all taxa.
  \item \code{"taxa"}: removes characters from specific taxa across all traits.
  \item \code{"extinct"}: removes data from extinct taxa only.
  Requires \code{seq = "tips"} as extinct taxa are only present at the tips of the tree.
  \item \code{"trait_taxa"}: removes characters at the intersection of specific traits and
    specific taxa, reflecting scenarios where certain anatomical regions are less likely
    to be preserved in particular lineages.
}}

\item{probability}{Numeric. Probability of missing data (single value or vector depending on method).}

\item{traits}{Integer vector. Indices of traits to remove. Required when
\code{method = "trait"} or \code{method = "trait_taxa"}.}

\item{taxa}{Character vector. Names of taxa to remove data from. Required when
\code{method = "taxa"} or \code{method = "trait_taxa"}.}
}
\value{
An object of class morpho.
}
\description{
This function removes characters from a morphological matrix simulated using morphsim
}
\examples{
# simulate a phylogenetic tree
phy <- ape::rtree(10)

# simulate characters along the branches of the tree
morpho_data <-  sim.morpho(tree = phy,
                           k = c(2,3,4),
                           trait.num = 20,
                           ancestral = TRUE,
                           partition = c(10,5,5),
                           ACRV = "gamma",
                           variable = TRUE,
                           ACRV.ncats = 4,
                           define.Q = NULL)

# randomly remove data
missing.data <- sim.missing.data(data = morpho_data,
                                  method = "random",
                                  seq = "tips",
                                  probability = 0.5)


# remove data based on the partition
missing.data <- sim.missing.data(data = morpho_data,
                                 method = "partition",
                                 seq = "tips",
                                 probability = c(0.7, 0, 0.5))

# remove data based on the rate it was simulated under
missing.data <- sim.missing.data(data = morpho_data,
                                        method = "rate",
                                        seq = "tips",
                                        probability = c(0,0,0.2,1))

# remove characters from specific traits
missing.data <- sim.missing.data(data = morpho_data,
                                 method = "trait",
                                 seq = "tips",
                                 probability = 1,
                                 traits = c(1,2,5))

# remove characters from specific taxa
missing.data <- sim.missing.data(data = morpho_data,
                                 method = "taxa",
                                 seq = "tips",
                                 probability = 1,
                                 taxa = c("t1", "t2"))

# remove specific character traits from specific taxa
missing.data <- sim.missing.data(data = morpho_data,
                                 method = "trait_taxa",
                                 seq = "tips",
                                 probability = 0.8,
                                 traits = c(1,2,5),
                                 taxa = c("t1", "t2"))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/summary.r
\name{get.reconstructed}
\alias{get.reconstructed}
\title{Add reconstructed tree and matrix to morpho object}
\usage{
get.reconstructed(data)
}
\arguments{
\item{data}{`morpho object` containing a fossil object}
}
\value{
a `morpho object`
}
\description{
This function should be called if you want to access the
reconstructed tree and character matrix directly from the morpho object.
It adds the reconstructed tree in Newick format ($trees$Recon), containing
only the sampled lineages, and a character matrix ($sequences$recon)
containing only the taxa present in that reconstructed tree.
}
\examples{
# simulate tree
lambda = 0.1
mu = 0.05
tips = 10
t = TreeSim::sim.bd.taxa(n = tips, numbsim = 1, lambda = lambda, mu = mu)[[1]]

# Simulate fossils and extant taxa
rate = 0.1 # poisson sampling rate
f = FossilSim::sim.fossils.poisson(rate = rate, tree = t, root.edge = FALSE)
rho = 0.5
f2 = FossilSim::sim.extant.samples(fossils = f, tree = t, rho = rho)
morpho_data <-  sim.morpho(k = c(2,3),
                           time.tree = t,
                           trait.num = 6,
                           ancestral = TRUE,
                           br.rates = 0.1,
                           partition = c(4,2),
                           ACRV = "gamma",
                           variable = TRUE,
                           ACRV.ncats = 4,
                           fossil = f2)

re <- get.reconstructed(morpho_data)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/write.r
\name{write.morpho}
\alias{write.morpho}
\title{Write morpho data to file}
\usage{
write.morpho(data, file, type = "tree", reconstructed = FALSE, uncertainty = 0)
}
\arguments{
\item{data}{A morpho object}

\item{file}{File name}

\item{type}{type to write: "tree", "matrix", or "ages"}

\item{reconstructed}{If TRUE, write the reconstructed version. Default FALSE.}

\item{uncertainty}{Numeric. Age uncertainty for fossil ages. Default 0.}
}
\value{
No return value, called for its side effect of writing data to a file.
}
\description{
Export components of a morpho object to file. Can write trees, character
matrices, or fossil ages in various formats.
}
\examples{
data(morpho_data)
tmp <- tempfile(fileext = ".tre")
write.morpho(morpho_data, file = tmp, type = "tree")

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/write.r
\name{write.recon.tsv}
\alias{write.recon.tsv}
\title{Write the taxa ages of reconstructed tree}
\usage{
write.recon.tsv(data, file, uncertainty = 0)
}
\arguments{
\item{data}{Morpho object}

\item{file}{File name}

\item{uncertainty}{Numeric. Adds uncertainty to fossil ages in the morpho object.
The ages in the object are point estimates by default; setting `uncertainty`
will create an age range of ± this value (in millions of years).}
}
\value{
No return value, called for its side effect of writing data to a file.
}
\description{
Writes the ages of the specimen in the reconstructed tree to a file. The tsv format used
here is directly compatible with RevBayes
}
\examples{
data(morpho_data)
tmp <- tempfile(fileext = ".tsv")
write.recon.tsv(data = morpho_data, file = tmp)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/write.r
\name{morphsim_fossilsim}
\alias{morphsim_fossilsim}
\title{Match sampled ancestor labels}
\usage{
morphsim_fossilsim(data = NULL)
}
\arguments{
\item{data}{Morpho object containing fossils}
}
\value{
A character matrix mapping sampled ancestor labels between the naming
conventions used by \code{Morphsim} and \code{Fossilsim}
}
\description{
Match the sampled ancestor labels from \code{Morphsim} and \code{Fossilsim}
}
\examples{
data(morpho_data)
morphsim_fossilsim <- function(data = morpho_data)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/GenerateQ.matrices.r
\name{symmetric.Q.matrix}
\alias{symmetric.Q.matrix}
\title{Generate a symmetric Q matrix}
\usage{
symmetric.Q.matrix(K)
}
\arguments{
\item{K}{Integer. The number of states.}
}
\value{
A K x K numeric matrix representing the symmetric Q matrix.
}
\description{
Creates a K x K symmetric rate matrix (Q matrix) with equal transition rates between states.
The diagonal elements are set such that each row sums to zero.
}
\examples{
symmetric.Q.matrix(4)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/summary.r
\name{get.convergent}
\alias{get.convergent}
\title{Identify convergent traits}
\usage{
get.convergent(data, trait = NULL)
}
\arguments{
\item{data}{A morpho object}

\item{trait}{The trait number to examine. If \code{NULL} (default), returns a summary.}
}
\value{
A data frame. Without \code{trait}: trait index and number of convergent states.
  With \code{trait}: state, origin, tips, and whether the state is convergent.
}
\description{
Identifies convergent evolution in a morpho object. When called without a
trait, returns a summary of which traits are convergent. When called with a
specific trait, returns tip groupings by state and evolutionary origin.
}
\examples{
phy <- ape::rtree(10)
morpho_data <- sim.morpho(tree = phy, k = 2, trait.num = 20)
get.convergent(morpho_data)
get.convergent(morpho_data, trait = 3)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/summary.r
\name{get.matrix}
\alias{get.matrix}
\title{Get Specific Morphological Matrix}
\usage{
get.matrix(data, seq)
}
\arguments{
\item{data}{A morpho object}

\item{seq}{The sequence data you want to extract}
}
\value{
A data frame with columns edge (branch number), state, and
hmin (where along the branch the transition occurred)
}
\description{
Return data frame of morphological matrix
}
\examples{
phy <- ape::rtree(10)
morpho_data <- sim.morpho(tree = phy, k = 2, trait.num = 20)
get.matrix(morpho_data, seq = "tips")
}
